/**
 * \file gre/lrgeneric.h
 * \brief GRE::LAYER_GENERIC definitions.
 *
 * \if NODOC
 * $Id: lrgeneric.h_v 1.7 2013/02/11 17:49:36 mju Exp $
 *
 * $Log: lrgeneric.h_v $
 * Revision 1.7  2013/02/11 17:49:36  mju
 * Remove redundant hdr.
 *
 * Revision 1.6  2012/11/16 16:14:24  mju
 * Remove datatip delegate support as now in base class.
 *
 * Revision 1.5  2011/10/20 21:47:43  mju
 * Add delegate for 3D drawing.
 *
 * Revision 1.4  2011/03/14 15:48:43  mju
 * Add setDelegateOnControlPanel.
 *
 * Revision 1.3  2010/11/17 16:02:18  mju
 * Add datatip support.
 *
 * Revision 1.2  2007/08/31 21:29:32  mju
 * Add setExtents.
 *
 * Revision 1.1  2007/08/31 20:36:16  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRGENERIC_H
#define	INC_GRE_LRGENERIC_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

namespace GRE {
//===================================================================================================================

//! Generic layer that uses delegates to perform actions.
class LAYER_GENERIC : public GRE::LAYER {
	public:

		typedef FastDelegate<int(MDLGPARENT)> DELEGATE_CONTROLPANEL;
		typedef FastDelegate<int(GRE::LAYERDC&)> DELEGATE_DRAW;

		//! Construct layer and insert into list
		explicit LAYER_GENERIC (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_GENERIC ();

		void SetDelegateOnControlPanel (
			DELEGATE_CONTROLPANEL delegate
			) { m_DelegateControlPanel = delegate; }

		DEPRECATED_MSG("Use SetDelegateOnDraw2D") void SetDelegateOnDraw (
			DELEGATE_DRAW delegate
			) { SetDelegateOnDraw2D(delegate); }

		void SetDelegateOnDraw2D (
			DELEGATE_DRAW delegate
			) { m_DelegateDraw2D = delegate; SetCanViewIn2D(!!delegate); }

		void SetDelegateOnDraw3D (
			DELEGATE_DRAW delegate
			) { m_DelegateDraw3D = delegate; SetCanViewIn3D(!!delegate); }

		void SetDelegateOnUpdateExtents (
			DELEGATE_ERRVALUE_NOPARMS delegate
			) { m_DelegateUpdateExtents = delegate; }

		void SetExtents (
			const REGION2D& region
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class DISPPARM : public LAYER::DISPPARM {
			public:
				DISPPARM ();
				virtual ~DISPPARM ();
			private:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
			};

		DISPPARM m_dispparm;
		DELEGATE_CONTROLPANEL m_DelegateControlPanel;
		DELEGATE_DRAW m_DelegateDraw2D;
		DELEGATE_DRAW m_DelegateDraw3D;
		DELEGATE_ERRVALUE_NOPARMS m_DelegateUpdateExtents;

		// Overrides from GRE::LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *NewGroup) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const char* v_SerialGetTagName () const;
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_GENERIC (const LAYER_GENERIC&);
		LAYER_GENERIC& operator= (const LAYER_GENERIC&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRGENERIC_H
