/**
 * \file gre/msgextracted.h
 * \brief GRE::MSG_EXTRACTED definition.
 *
 * \if NODOC
 * $Id: msgextracted.h_v 1.5 2013/02/15 20:51:42 mju Exp $
 *
 * $Log: msgextracted.h_v $
 * Revision 1.5  2013/02/15 20:51:42  mju
 * Fix dox hdr.
 *
 * Revision 1.4  2005/10/26 16:40:51  mju
 * Use gre namespace instead of legacy names.
 *
 * Revision 1.3  2005/10/19 17:34:11  mju
 * Excluded deprecated typedef if compile for no_deprecated.
 *
 * Revision 1.2  2005/10/17 20:26:42  mju
 * Use gre namespace.
 *
 * Revision 1.1  2005/01/20 16:58:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_MSGEXTRACTED_H
#define	INC_GRE_MSGEXTRACTED_H

#ifndef INC_GRE_MSG_H
	#include <gre/msg.h>
#endif

#ifndef INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

namespace GRE {
//===================================================================================================================

class MSG_EXTRACTED : public GRE::MSG {
	public:

		//! Constructor.
		MSG_EXTRACTED (
			GRE::LAYER *layer,						//!< Layer extracted from
			const RVC::OBJITEM& objitem		//!< Object extracted to
			): MSG(ACTION_ExtractedFrom,layer), m_objitem(objitem), m_processed(false) { }

		//! Get item extracted to.
		const RVC::OBJITEM& GetObjItem (
			) const { return (m_objitem); }

		//! Determine if message has been marked as "processed".
		bool IsProcessed (
			) const { return (m_processed); }

		//! Mark message as having been processed.
		void SetProcessed (
			) const { m_processed = true; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::OBJITEM m_objitem;
		mutable bool m_processed;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_MSGEXTRACTED_H
