/**
 * \file mgui/dlgcustomize.h
 * \brief MGUI::DLG_CUSTOMIZE definition
 *
 * \if NODOC
 * $Id: dlgcustomize.h_v 1.2 2011/12/28 14:07:12 mju Exp $
 *
 * $Log: dlgcustomize.h_v $
 * Revision 1.2  2011/12/28 14:07:12  mju
 * Use imagelst hdr.
 *
 * Revision 1.1  2008/04/28 20:19:12  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGCUSTOMIZE_H
#define	INC_MGUI_DLGCUSTOMIZE_H

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_TREE_H
	#include <mgui/tree.h>
#endif

#ifndef	INC_MGUI_IMAGELST_H
	#include <mgui/imagelst.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

namespace MGUI {

class CTRL_TOOLBAR;
class MENU;

//=====================================================================================================================

class DLG_CUSTOMIZE : public DLGSHELL {
	public:

		DLG_CUSTOMIZE ();

		~DLG_CUSTOMIZE ();

		//! Add menu with default label.
		void AddMenu (
			MGUI::MENU& menu
			);

		//! Add menu with specific label.
		void AddMenu (
			MGUI::MENU& menu,
			const MISTRING& label
			);

		//! Add toolbar with default label.
		void AddToolBar (
			MGUI::CTRL_TOOLBAR& toolbar
			);

		//! Add toolbar with specific label.
		void AddToolBar (
			MGUI::CTRL_TOOLBAR& toolbar,
			const MISTRING& label
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_CUSTOMIZE THISCLASS;
		typedef MILIST<MGUI::MENU> MENULIST;

		enum STATEIDX {
			STATEIDX_None,
			STATEIDX_Off,
			STATEIDX_On,
			STATEIDX_OnDisabled
			};

		MGUI::CTRL_TREE m_Tree;
		MENULIST m_MenuList;
		MISTRINGLIST m_LabelList;
		MGUI::IMAGELIST m_ImageList;

		void AddImageListItems (const void *pPrivParent);
		void AddItemsToTree (MGUI::CTRL_TREE::ITEMHND hndParent, const void *pPrivParent);
		bool OnTreePointerDown (MGUI::CTRL_TREE::ITEMHND hndItem, MGUI::CTRL_TREE::HITRESULT HitResult, const LPOINT2D& point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void UpdateCustomSettings (MGUI::CTRL_TREE::ITEMHND hndParent, const void *vpPrivParent);

		// SHELL/DLGSHELL overrides
		virtual int v_CreateContent ();
		virtual void v_OnApply ();

		#endif // DOXYGEN
		
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGCUSTOMIZE_H
