/*****************************************************************************
 * \file <mgui/dlglogindb.h>
 * \brief MGUI::DLG_LOGIN_DB definitions.
 *
 * \if NODOC
 * $Id: dlglogindb.h_v 1.7 2011/11/21 21:26:16 mju Exp $
 *
 * $Log: dlglogindb.h_v $
 * Revision 1.7  2011/11/21 21:26:16  mju
 * Remove redundant hdr.
 *
 * Revision 1.6  2011/06/21 19:44:02  mju
 * Rename.
 *
 * Revision 1.5  2007/08/03 22:05:24  dwilliss
 * added support for databases that can't retrieve database list but require
 * you to specify it anyway (Oracle)
 *
 * Revision 1.3  2006/04/14 19:09:37  dwilliss
 * Added support for schema prompting
 *
 * Revision 1.1  2006/03/28 17:46:07  dwilliss
 * Initial revision
 * \endif
****************************************************************************/

#ifndef INC_MGUI_DLGLOGINDB_H
#define INC_MGUI_DLGLOGINDB_H

#ifndef INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
	#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Base for dialogs to login to database server.
class DLG_LOGIN_DB : public MGUI::DLGSHELL {
	public:

		DLG_LOGIN_DB ();

		virtual ~DLG_LOGIN_DB ();

		void GetConnectionString (
			MISTRING& mistr
			);

	protected:

		void GetConnectionStringLow (
			MISTRING& mistr,
			bool bForGettingDatabaseList
			);

		bool GetUseWindowsLogin (
			) const { return (m_CtrlWindowsLogin.GetValue()); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_LOGIN_DB THISCLASS;

		FORM_EDIT_STRING m_CtrlHost;
		FORM_EDIT_NUMBER m_CtrlPort;
		FORM_EDIT_STRING m_CtrlUser;
		FORM_EDIT_STRING m_CtrlPassword;
		FORM_EDIT_STRING m_CtrlService;
		FORM_COMBOBOX m_CtrlDatabaseList;
		FORM_COMBOBOX m_CtrlSchemaList;
		CTRL_PUSHBUTTON m_CtrlGetDBList;
		CTRL_PUSHBUTTON m_CtrlGetSchemaList;
		CTRL_TOGGLEBUTTON m_CtrlWindowsLogin;
		MISTRINGLIST m_DatabaseList;
		MISTRINGLIST m_SchemaList;

		void OnGetDBList();
		void OnGetSchemaList();
		void OnToggleWindowsLogin();

		virtual ERRVALUE v_CreateContent();
		virtual void v_OnOK();

		#endif

		// Overridables.

		//! Return true to have the "Use Windows Login" toggle.
		//! Default implementation returns false.
		virtual bool v_CanUseWindowsLogin () const;

		//! Return the connection string given the current settings.
		//! if bForGettingDatabaseList is true, the connection string should
		//! be whatever you'll need in v_GetDatabaseList.  
		virtual void v_GetConnectionString (
			MISTRING& ConnectionString,
			bool bForGettingDatabaseList,
			const MISTRING& Host,
			const MISTRING& User,
			const MISTRING& Password,
			const MISTRING& Database,
			const MISTRING& Schema,
			INT32 port
			) = 0;

		//! Given the current settings, get the list of available databases.
		//! Default implementation will work in most cases.
		virtual ERRVALUE v_GetDatabaseList (
			MISTRINGLIST& List
			);

		//! Get the default database name.
		//! This is the database name to use when connecting to query for the
		//! list of databases.  The default is ""
		virtual const char* v_GetDefaultDatabaseName () const;

		//! Get the default port number for this database type.
		//! Return -1 if it is not user-settable.
		virtual INT32 v_GetDefaultPort () const;

		//! Get the default schema name.
		//! This should be the default schema if none is set in ini
		virtual const char* v_GetDefaultSchemaName () const;

		//! Return the group used to store last used settings in ini file.
		//! The default is the same as the family.
		virtual const char* v_GetINIGroup () const = 0;

		//! Given the current settings, get the list of available schemas.
		virtual ERRVALUE v_GetSchemaList (
			MISTRINGLIST& List
			);

		virtual bool v_NeedServiceName () const;

		//! Save format-specific properties in the ini file.
		virtual void v_SaveDefaults ();

		//! Return true if the user should be able to select a default schema.
		//! Default implementation returns false.
		virtual bool v_ShowSchemaSelection () const;

	};

//=====================================================================================================================

}	// End namespace MGUI

#endif // INC_MGUI_DLGLOGINDB_H
