/**
 * \file mgui/formgeocatalog.h
 *
 * \if NODOC
 * $Id: formgeocatalog.h_v 1.6 2011/11/29 22:22:46 mju Exp $
 *
 * $Log: formgeocatalog.h_v $
 * Revision 1.6  2011/11/29 22:22:46  mju
 * Remove redundant hdr.
 *
 * Revision 1.5  2010/10/27 20:44:21  bcolombe
 * Conver to use form_list
 *
 * Revision 1.4  2009/10/30 17:47:36  mju
 * Add hasResults/Selected and getObjItemListResults.
 *
 * Revision 1.3  2009/07/02 18:02:59  bcolombe
 * Changes for setting CRS for displaying extents
 *
 * Revision 1.2  2009/04/20 15:54:44  bcolombe
 * Changed ui for searching by extents, uses listbox with options
 * Added finding point by geonames
 *
 * Revision 1.1  2009/04/15 20:09:27  bcolombe
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_MGUI_FORMGEOCATALOG_H
#define	INC_MGUI_FORMGEOCATALOG_H

#ifndef INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef INC_MGUI_FORM_EDIT_EXTENTS_H
#include <mgui/formextents.h>
#endif

#ifndef INC_MGUI_FORM_EDIT_COORD_H
#include <mgui/formcoord.h>
#endif

#ifndef INC_MGUI_MENU_H
#include <mgui/menu.h>
#endif

#ifndef INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_MI32_GEOCAT_H
#include <mi32/geocat.h>
#endif

#ifndef INC_MI32_MICON_H
#include <mi32/micon.h>
#endif

#ifndef INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

namespace MGUI {

//! Form for geocatalog browse/search
class FORM_GEOCATALOG : public MGUI::FORM_COMPOSITE {
	public:
		FORM_GEOCATALOG (
			);

		~FORM_GEOCATALOG (
			);

		//! Create Search Form
		ERRVALUE CreateSearchForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Create Results Form
		ERRVALUE CreateResultsForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Create List Form
		ERRVALUE CreateListForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Perform search
		void DoSearch ();

		//! Get all results.
		void GetObjItemListResults (
			RVC::OBJITEMLIST& ObjItemList
			);

		//! Get selected objitems
		void GetObjItemListSelected (
			RVC::OBJITEMLIST& ObjItemList
			);

		//! Determine if has any results.
		bool HasResults (
			) const;

		//! Determine if has any items selected in results.
		bool HasSelected (
			) const;

		//! Set allowed objtypes
		void SetObjTypeSet (
			const RVC::OBJTYPESET& objtypeset
			) { m_SearchObjTypes = objtypeset; m_NeedNewSearch = true; }

		//! Set extensions to match
		void SetFilePattern (
			const MISTRING& pattern
			) { m_Pattern = pattern; m_NeedNewSearch = true; }

		//! delegate when user clicks on item in results grid
		void SetClickDelegate (
			FastDelegate<void(const RVC::OBJITEM&)> delegate
			) { m_ClickDelegate = delegate; }

		//! delegate when user double clicks on item in results grid
		void SetDoubleClickDelegate (
			FastDelegate<void(const RVC::OBJITEM&)> delegate
			) { m_DoubleClickDelegate = delegate; }

		//! delegate when user wants to search
		void SetSearchDelegate (
			FastDelegate<void()> delegate
			) { m_SearchDelegate = delegate; }

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_COL_Location;
		int m_COL_NameTooltip;
		int m_COL_Name;
		int m_COL_Type;
		int m_COL_Format;
		int m_COL_RecCreateDate;
		int m_COL_RecModDate;
		int m_COL_ObjModDate;
		int m_COL_LonWest;
		int m_COL_LonEast;
		int m_COL_LatSouth;
		int m_COL_LatNorth;
		int m_COL_CRS;
		int m_COL_Dimensions;
		int m_COL_Scale;
		int m_COL_ScaleX;
		int m_COL_ScaleY;
		int m_COL_DataType;
		int m_COL_NumCols;

		bool m_NeedNewSearch;
		bool m_SortAscending;
		mutable bool m_CatFileListChanged;
		MILIST<GEOCAT*> m_GeoCatList;
		RVC::OBJTYPESET m_SearchObjTypes;
		MISTRING m_Pattern;
		RVC::OBJITEMLIST m_ObjItemListSearchAreas;
		SIMPLE_ARRAY<bool> m_CatUseArray;
		UOM::CONVERTER m_converter;
		MGUI::MENU m_UnitMenu;
		UINT32 m_BaseNumItems;
		SPATREF::COORDREFSYS m_CRS;

		MGUI::FORM_LIST m_resultslist;
		MGUI::FORM_LIST m_catalogslist;
		MGUI::LAYOUT_PANE m_ParentPane;
		MGUI::LAYOUT_PANE m_RangePane;
		MGUI::LAYOUT_PANE m_PointPane;
		MGUI::LAYOUT_PANE m_ObjectPane;
		MGUI::CTRL_PUSHBUTTON m_AddFilesButton;
		MGUI::CTRL_PUSHBUTTON m_RemoveFilesButton;
		MGUI::CTRL_PUSHBUTTON m_SearchButton;
		MGUI::CTRL_TOGGLEBUTTON m_ExtentsToggle;
		MGUI::CTRL_COMBOBOX m_ContainmentCombo;
		MGUI::CTRL_LISTBOX m_ListBox;
		MGUI::CTRL_PUSHBUTTON m_AddObjectPB;
		MGUI::CTRL_PUSHBUTTON m_FindLocationPB;
		MGUI::FORM_EDIT_STRING m_FormSearchText;
		MGUI::FORM_EDIT_EXTENTS m_FormExtents;
		MGUI::FORM_EDIT_EXTENTS m_FormObjectExtents;
		MGUI::FORM_EDIT_COORD m_FormEditCoord;

		void AddCatalogFile ();
		void OnAddObject ();
		void OnFindLocation ();
		void OnExtentsToggle ();
		void OnNeedSearch ();
		void OnObjectSelect ();
		void RemoveCatalogFile ();
		void SaveCatFileListIni () const;
		ERRVALUE UpdateRegions ();
		void OnUnitMenu (MGUI::ID id, bool state);
		void GetLatLonExtents(const GEOCAT::CONST_ITERATOR& iter, int pos);

		bool OnClick(int itemnum, int fieldnum, const MGUI::POINT& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		bool OnDoubleClick(int itemnum, int fieldnum, const MGUI::POINT& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		int OnDimensionsCompare(int itemnum1, int itemnum2, int fieldnum);
		int OnScaleCompare(int itemnum1, int itemnum2, int fieldnum);
		bool OnToolTip(int itemnum, int fieldnum, MGUI::CTRL_TOOLTIP& tooltip);

		bool OnListClick(int itemnum, int fieldnum, const MGUI::POINT& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);

		void OnPageInitCatalogs (MGUI::LAYOUT_PAGE& page);
		void OnPageInitSearch (MGUI::LAYOUT_PAGE& page);
		void OnPageInitResults (MGUI::LAYOUT_PAGE& page);
		void OnPageSetActiveResults (MGUI::LAYOUT_PAGE& page);

		FastDelegate<void(const RVC::OBJITEM&)> m_ClickDelegate;
		FastDelegate<void(const RVC::OBJITEM&)> m_DoubleClickDelegate;
		FastDelegate<void()> m_SearchDelegate;

		#endif
	};

} //end MGUI namespace

#endif //INC_MGUI_FORMGEOCATALOG_H
