/** 
 * \file mgui/formselectlanguage.h 
 * \brief Definitions for MGUI::FORM_SELECT_LANGUAGE class.
 *
 * \if NODOC
 * $Id: formselectlanguage.h_v 1.2 2007/02/19 17:07:40 scowan Exp $
 *
 * $Log: formselectlanguage.h_v $
 * Revision 1.2  2007/02/19 17:07:40  scowan
 * Changed to derive from combobox.
 *
 * Revision 1.1  2007/02/15 20:32:21  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMSELECTLANGUAGE_H
#define	INC_MGUI_FORMSELECTLANGUAGE_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Form to allow selection of a language for the user interface of MicroImages products.
class FORM_SELECT_LANGUAGE : public MGUI::FORM_COMBOBOX {
	public:
	
		//! Constructor.
		FORM_SELECT_LANGUAGE (
			);

		//! Destructor.
		virtual ~FORM_SELECT_LANGUAGE (
			);

		//! Set the currently selected language
		//! @return The ISO639-2 language code (the three character code, Eg: 'enu' for english).
		const char* GetSelectedLanguage (
			) const;

		//! Save the currently selected language into the appropriate places based on the
		//! Operating and GUI System. 
		void SaveSelectedLanguage (
			);

		//! Set the selected language in the combobox
		void SetSelectedLanguage (
			const char* Language				//! Either type of ISO639 language code.  ('enu' or 'en-us' for English)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_OrigLang;
		
		virtual void v_CreateEnd ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End of MGUI namespace

#endif		//	INC_MGUI_FORMSELECTLANGUAGE_H

