/**
 * \file mi32/enumset.h 
 * \brief Define template ENUMSET class.
 *
 * \if NODOC
 * $Id: enumset.h_v 1.4 2011/10/04 12:58:50 mju Exp $
 *
 * $Log: enumset.h_v $
 * Revision 1.4  2011/10/04 12:58:50  mju
 * Fix dox so does not declare wrong class name.
 *
 * Revision 1.1  2006/06/13 16:40:03  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_ENUMSET_H
#define	INC_MI32_ENUMSET_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_STRING_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <string.h>		// Includes defn for memset()
#define	INC_STRING_H
#endif
#endif

//! Container for "set" of enum values.
template <typename _ENUMTYPE, _ENUMTYPE _LAST, _ENUMTYPE _ALL = static_cast<_ENUMTYPE>(-1)> class ENUMSET {
	public:

		//! Default constructor, creates empty set.
		ENUMSET (
			) { Clear(); }

		//! Implicit constructor from single _ENUMTYPE value.
		//! Allows a single _ENUMTYPE value to be passed wherever ENUMSET is specified.
		ENUMSET (
			_ENUMTYPE enumval
			) { Clear(); operator+=(enumval); }

		//! Convenience constructor for specifying two _ENUMTYPE values.
		ENUMSET (
			_ENUMTYPE enumval1,
			_ENUMTYPE enumval2
			) { Clear(); operator+=(enumval1); operator+=(enumval2); }

		//! Convenience constructor for specifying three _ENUMTYPE values.
		ENUMSET (
			_ENUMTYPE enumval1,
			_ENUMTYPE enumval2,
			_ENUMTYPE enumval3
			) { Clear(); operator+=(enumval1); operator+=(enumval2); operator+=(enumval3); }

		//! Add type to set.
		ENUMSET& operator+= (
			_ENUMTYPE rhs
			) { if (rhs == _ALL) SetAll(); else m_enumvalbits[rhs/32] |= 1 << (rhs % 32); return (*this); }

		//! Add type set to set.
		ENUMSET& operator+= (
			const ENUMSET<_ENUMTYPE, _LAST, _ALL>& rhs
			) { for (INT32 i = 0;(i < (_LAST+31)/32);++i) m_enumvalbits[i] |= rhs.m_enumvalbits[i]; return (*this); }

		//! Remove type from set.
		ENUMSET& operator-= (
			_ENUMTYPE rhs
			) { if (rhs == _ALL) Clear(); else m_enumvalbits[rhs/32] &= ~(1 << (rhs % 32)); return (*this); }

		//! Clear set.
		void Clear (
			) { memset(m_enumvalbits,0,sizeof(m_enumvalbits)); return; }

		//! Determine if set contains specified enumect type.
		bool Contains (
			_ENUMTYPE enumval
			) const { return ((m_enumvalbits[enumval/32] & (1 << (enumval % 32))) != 0); }

		//! Get first enumect type in set.
		//! @return First enumect type or ENUM_LAST if no more in set.
		_ENUMTYPE GetFirst (
			) const { return (GetNext(_ALL)); }

		//! Get next enumect type in set.
		_ENUMTYPE GetNext (
			_ENUMTYPE enumval
			) const {
			do {
				++enumval;
				if (Contains(enumval)) break;
				} while (enumval < _LAST);
			return (enumval);
			}

		void SetAll (
			) { memset(m_enumvalbits,255,sizeof(m_enumvalbits)); return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_enumvalbits[(_LAST+31)/32];
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#endif	// INC_MI32_ENUMSET_H
