 /*
 *	\file framemem.h <mi32/framemem.h>
 *	\brief Definition of FRAMEMEM class
 *
 * \if NODOC
 * $Id: framemem.h_v 1.10 2011/10/28 21:11:58 mju Exp $
 *
 * $Log: framemem.h_v $
 * Revision 1.10  2011/10/28 21:11:58  mju
 * Specify frame buffer index when use getImagePointer.
 *
 * Revision 1.9  2011/10/13 12:50:08  mju
 * Remove redundant hdr.
 *
 * Revision 1.8  2006/07/14 16:52:28  dwilliss
 * got rid of sml class that's no longer necessary here
 *
 * Revision 1.1  2001/04/04 22:34:22  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_FRAMEMEM_H
#define	INC_MI32_FRAMEMEM_H

#ifndef	INC_MI32_MG2MEM24_H
#include <mi32/mg2mem24.h>
#endif

#ifndef	INC_GRE_BASE_H
#include <gre/base.h>
#endif


//! FRAMEMEM is designed with purpose of getting images as MEMORY24 from any view,
//! combaining them together, drawing to them using mgd functions and putting them to movie.

class FRAMEMEM {


public:

	FRAMEMEM (										//! Copy constructor
		const FRAMEMEM &rhs
		);

	FRAMEMEM (										//! Constructor by GRE::VIEW
		const GRE::VIEW &view						//!< View (2d or 3d)
		);

	FRAMEMEM (										//! Constructor by width and height
		const INT32 &width,						//!< Width of the frame
		const INT32 &height						//!< Height of the frame
		);

	~FRAMEMEM (										//! Destructor
		);

	FRAMEMEM& operator= (						//! Assignment operator
		const FRAMEMEM& rhs
		);

	operator const UINT8* (							//! Cast to const UINT8*.
		) const {
		if (!IsValid()) return (0);
		return (m_Display->GetImagePointer(0,0,0,0));
		}

	operator UINT8* (									 //! Cast to UINT8*.
		) {
		if (!IsValid()) return (0);
		return (m_Display->GetImagePointer(0,0,0,0));
		}


	//!	Create MGD::CONTEXT
	//!
	//! @return gc
	MGD::CONTEXT* CreateGC (
		);

	//!	Get height of the frame
	//!
	//! @return height
	inline INT32 GetHeight (
		) const {
		return m_Height;
		}

	//!	Get width of the frame
	//!
	//! @return width
	inline INT32 GetWidth (
		) const {
		return m_Width;
		}

	//!	Copy image from another frame
	//!
	//! @return ERRVALUE
	ERRVALUE Copy (
		const FRAMEMEM &frame,						//!< Source frame
		const INT32 dstX = 0,						//!< Destination x coordinate
		const INT32 dstY = 0,						//!< Destination y coordinate
		const INT32 srcX = 0,						//!< Source x coordinate
		const INT32 srcY = 0,						//!< Source y coordinate
		const INT32 srcW = INT32_MAX,		  		//!< Source width
		const INT32 srcH = INT32_MAX				//!< Source height
		);

	//!	Copy image from View
	//!
	//! @return ERRVALUE
	ERRVALUE CopyFromView (
		const GRE::VIEW &view,						//!< View (2d or 3d)
		const INT32 dstX = 0,						//!< Destination x coordinate
		const INT32 dstY = 0,						//!< Destination y coordinate
		const INT32 srcX = 0,						//!< Source x coordinate
		const INT32 srcY = 0,						//!< Source y coordinate
		const INT32 srcW = INT32_MAX,				//!< Source width
		const INT32 srcH = INT32_MAX				//!< Source height
		);

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT

	INT32 m_Height;									//!< Widht of the frame
	INT32 m_Width;										//!< Height of the frame
	MGD::DEVICE_MEM24 *m_Display;							
	MGD::CONTEXT *m_Gc;										
	ERRVALUE m_Error;									//!<	Initialization error;

	FRAMEMEM (											//! Default constructor (not implemented)
		);

	void Init (
		);

	void Close (
		);

	inline bool IsValid (
		) const {
		return ((m_Width > 0) && (m_Height > 0) && (m_Gc != 0) && (m_Display != 0));
		}

	ERRVALUE CopyFromDisplay (
		MGD::DEVICE &display,				
		const INT32 dstX,						
		const INT32 dstY,						
		const INT32 srcX,						
		const INT32 srcY,						
		const INT32 srcW,			
		const INT32 srcH				
		);
	#endif // GENERATING_DOXYGEN_OUTPUT

	};



#endif
