/**
 * \file miexcept.h <mi32/miexcept.h>
 * \brief MicroImages Exception definition file
 *
 * \if NODOC
 * $Id: miexcept.h_v 1.3 2003/09/15 13:49:56 fileserver!dwilliss Exp $
 *
 * $Log: miexcept.h_v $
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2000/06/20 20:41:33  sparsons
 * Genitor documentation.
 *
 * Revision 1.1  1999/06/03 19:18:26  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_MI32_MIEXCEPT_H
#define INC_MI32_MIEXCEPT_H

class MIEXCEPTION {
public:

// CONSTRUCTION / DESTRUCTION
	
	//! Constructor.
	MIEXCEPTION (	 
		const int ErrorCode,
		const int LineNum,
		const char*const RcsID
		);

	//! Destructor.
	virtual ~MIEXCEPTION (
		);

#ifndef GENERATING_DOXYGEN_OUTPUT
//!	void AddParameter (					Add a parameter to the output report
//!		const UCSTRING& str					String to add
//!		);
#endif //!< GENERATING_DOXYGEN_OUTPUT

	//! Add a line of program source to the output.
	void AddSourceLine (					
		const char*const rcsid,				//!< RCSID string
		const int LineNum						//!< __LINE__
		);

	//! Get the MI error code number for this exception.
	int GetErrorCode (					
		);

#ifndef GENERATING_DOXYGEN_OUTPUT
//!	UCSTRING& GetErrorReport (			Get the full error report including stack trace
//!		UCSTRING& str
//!		);

//!	UCSTRING& GetErrorLine (			Get the first line of the error report
//!		UCSTRING& str
//!		);
#endif //!< GENERATING_DOXYGEN_OUTPUT

protected:
	
	//! Get true error code from derived classes.
	virtual int GenerateErrorCode (	
		);

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT

//!	bool GetErrorMsgLine (				Get error message line that corresponds to error code
//!		UCSTRING& result
//!		);

	int ErrCode;							//!< The MI error code number
	int LineRepeatCount;					//!< Lines repeated in the stack trace
	int LastLineNum;						//!< Last line number used in stack trace
//!	std::string LastModName;			Last module name used in stack trace
//!	std::string StackTrace;				List of RCSID's and line numbers

#if defined(WINNT)
#pragma warning (push)
#pragma warning (disable:4251)		//! Disable warning that this needs a DLL interface
#endif

//!	std::vector<UCSTRING> ParmsList;	Parameters list

#if defined(WINNT)
#pragma warning (pop)
#endif
		#endif // GENERATING_DOXYGEN_OUTPUT
};


#if defined(WINNT) && defined(_CPPUNWIND)

#include <exception>

#define	MITRY		try
#define	MICATCH	catch

#define MICATCHRETURN								\
	catch(MIEXCEPTION& ex) {						\
		return (SetErrPosnExc(ex));				\
		}													\
	catch(std::bad_alloc& ex) {					\
		return (SetErrPosnC(EOutOfMemoryB));	\
		}

#define MICATCHSETERR							\
	catch(MIEXCEPTION& ex) {					\
		err = SetErrPosnExc(ex);				\
		}												\
	catch(std::bad_alloc& ex) {				\
		err = SetErrPosnC(EOutOfMemoryB);	\
		}

#define MICATCHGOTO								\
	catch(MIEXCEPTION& ex) {					\
		err = SetErrPosnExc(ex);				\
		goto Error;									\
		}												\
	catch(std::bad_alloc& ex) {				\
		err = SetErrPosnC(EOutOfMemoryB);	\
		goto Error;									\
		}

#define MICATCHDISPLAY(_w_)		 					\
	catch(MIEXCEPTION& ex) {							\
		MdlgDisplayErrorCode(_w_, ex);				\
		}														\
	catch(std::bad_alloc& ex) {						\
		MdlgDisplayErrorCode(_w_, EOutOfMemoryB);	\
		}

#define MICATCHDISPLAYRETURN(_w_) 					\
	catch(MIEXCEPTION& ex) {							\
		MdlgDisplayErrorCode(_w_, ex);				\
		return;												\
		}														\
	catch(std::bad_alloc& ex) {						\
		MdlgDisplayErrorCode(_w_, EOutOfMemoryB);	\
		return;												\
		}

#define MICATCHIGNORE				\
	catch(MIEXCEPTION& ex) {}		\
	catch(std::bad_alloc& ex) {}

#define MICATCHRETHROW				 								\
	catch(MIEXCEPTION& ex) {										\
		ex.AddSourceLine(RCSID, __LINE__);						\
		throw;															\
		}																	\
	catch(std::bad_alloc& ex) {									\
		throw MIEXCEPTION(EOutOfMemoryB, __LINE__, RCSID);	\
		}

#else			//!< Macro's for exceptions that are not handled by the compile system

#define	MITRY
#define	MICATCH

#define MICATCHRETURN
#define MICATCHSETERR
#define MICATCHGOTO	goto Error;
#define MICATCHDISPLAY(_w_)
#define MICATCHDISPLAYRETURN(_w_)	return;
#define MICATCHIGNORE
#define MICATCHRETHROW

#endif

#endif		//!< INC_MI32_MIEXCEPT_H
