/**
 * \file sort.h <mi32/sort.h>
 * \brief Sorting functions
 *
 * \if NODOC
 * $Id: sort.h_v 1.1 2005/07/14 17:29:42 mju Exp $
 *
 * $Log: sort.h_v $
 * Revision 1.1  2005/07/14 17:29:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_SORT_H
#define	INC_MI32_SORT_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif


#if defined(MAC_OSX) && !defined(GENERATING_DOXYGEN_OUTPUT)
// Mac OSX standard library defines their own heapsort which takes a
// different compare function.  This makes sure we link to ours, not
// theirs.
#define heapsort heapsort_MicroImages
#endif

#if defined(__cplusplus)
extern "C" {
#endif

//!\addtogroup sort Sorting Functions
//!@{

//! Memory based heapsort.
//!
//! Sorts an array in memory.  Assumes it can use memcpy() to swap items.
LIBEXPORT void heapsort (
	void* array,								//!< Array to sort 
	INT32 n, 									//!< Number of elements in array
	INT32 size, 								//!< sizeof() each element
	int (*compare)(void*a, void*b, void*hookdata),  //! Compare function. Return 1 if *a <= *b.
	void* hookdata								//!< Passed as 3rd parameter to compare function
	);

//! Generic Heapsort.
LIBEXPORT int HeapSortGen (
	INT32 n, 									//!< Number of items to sort
	int (*compare)(INT32 a, INT32 b, void* hookdata), 	//! Return 1 if item a <= item b
	int (*swap)(INT32 a, INT32 b, void* hookdata), 		//! Swap two items
	void* hookdata  							//!< Passed as 3rd parameter to compare and swap functions
	);
//!@}

#if defined(__cplusplus)
}
#endif


#undef	LIBEXPORT

#endif	// INC_MI32_SORT_H
