/***
** Copyright 1992, 1993 by Markku Savela and
**	Technical Research Centre of Finland
 *
 * \if NODOC
 * $Log: text.h_v $
 * Revision 1.45  2011/11/08 13:33:17  mju
 * Add inclusion guard and remove redundant hdr.
 *
 * Revision 1.44  2008/09/03 13:28:25  mju
 * Fix doxygen warnings.
 *
 * Revision 1.43  2007/03/26 15:08:54  dwilliss
 * fixed typo in callback name
 *
 * Revision 1.42  2007/03/23 22:48:17  dwilliss
 * added another callback
 *
 * Revision 1.41  2005/03/31 16:57:21  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.40  2005/02/22 14:59:27  dwilliss
 * Needed another parameter on new prototype
 *
 * Revision 1.39  2005/02/22 14:55:41  dwilliss
 * Added new prototype XeTextOFfset2LinCol
 *
 * Revision 1.38  2004/11/19 17:27:21  dwilliss
 * Added support for forcing fixed-width font
 *
 * Revision 1.37  2004/09/13 22:59:41  dwilliss
 * *** empty log message ***
 *
 * Revision 1.36  2004/09/13 22:51:10  dwilliss
 * added a protototype for new function
 *
 * Revision 1.35  2004/08/06 22:23:55  dwilliss
 * Defined callback data for hyperlink callback
 *
 * Revision 1.34  2004/08/06 22:17:37  dwilliss
 * Defined hyper link callbacks
 *
 * Revision 1.33  2004/08/06 22:08:43  dwilliss
 * Added control codes for hyper links
 *
 * Revision 1.32  2004/06/15 16:43:47  dwilliss
 * *** empty log message ***
 *
 * Revision 1.31  2004/06/09 22:58:31  dwilliss
 * Started adding stuff for syntax highlighting
 *
 * Revision 1.30  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * \endif
*/

#ifndef INC_MI32_TEXT_H
#define INC_MI32_TEXT_H

#ifndef INC_MI32_XEWBASIC_H
#include <mi32/xewbasic.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct COLOR;
#endif

/* Parameters:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 alignment           XeAlignment        XeAlignment     XeAlignment_START
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 columnWidth         ColumnWidth	Dimension	0
 contentFile         ContentFile        String          NULL
 contentLength       ContentLength      int            Computed from content
 contentOffset       ContentOffset      int            0
 contentStream       ContentStream      FILE *          NULL
 contentString       ContentString      Pointer         NULL
 cursor              Cursor             Cursor          None
 destroyCallback     Callback		Pointer		NULL
 exportFormat        ExportFormat       XeTextExport    XeTextExport_ODIF
 firstLineOffset     Offset             int             0
 fonts               XeFonts            XeFont *        NULL
 format              Format             Boolean         False
 graphicRendition    Rendition          String          NULL
 height		     Height		Dimension	0
 indentation         Offset             Dimension       0
 initialState        Sequence           String          NULL
 itemization         Itemization        XeItemization * NULL
 kerningOffset       KerningOffset      Dimension *     NULL
 layoutCallback      LayoutCallback     Pointer         NULL
 lineLayoutTable     LineLayoutTable    XeTabStop *     NULL
 lineSpacing         LineSpacing        Dimension       Computed from font
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 proportional        Proportional       Boolean         False
 width		     Width		Dimension	0
 x		     Position		Position	0
 y		     Position		Position	0

*/

typedef enum 
    {
	XeAlignment_NONE,
	XeAlignment_START,
	XeAlignment_END,
	XeAlignment_CENTER,
	XeAlignment_JUSTIFIED,
	XeAlignment_AROUND
    } XeAlignment;

/*
** XeTextExport
**	Defines the valid values for the exportFormat resource. This
**	resource selects the format for the exported selections,
**	XeTextGetString and XeTextSaveAsFile. In selections, the formats
**	are mapped to target atoms
**
**		XeTextExport_STRING*	\<-\> STRING
**		XeTextExport_ODIF*	\<-\> COMPOUND_TEXT, STRING
**
**	If exportFormat is XeTextExport_STRING*, the widget will only
**	recognize STRING target; if XeTextExport_ODIF*, the widget will
**	recognize both STRING and COMPOUND_TEXT, and the ODIF variants
**	are returned as COMPOUND_TEXT.
**
**	The 'formatted' variants will include the line breaks caused
**	by possible word/line breaking algorithms. Source variant includes
**	only original "hard" line breaks. The 'formatted processable'
**	includes both source text and layout generated information
**	delimited by \<SOS\>..\<ST\> controls (as defined in ODA standard).
**
** NOTE: The numeric values have been designed (and are used) such that
**		value & 1 =\> include formatted information
**		value & 2 =\> produce ODIF (COMPOUND_TEXT)
**		value & 4 =\> produce ODIF \<SOS\> \<ST\> enclosings for layout
**				control functions
*/
typedef enum
    {
	XeTextExport_STRING = 0,	//!<  0, Latin1 source text 
	XeTextExport_STRING_F = 1,	//!<  1, Latin1 formatted text 
	XeTextExport_ODIF = 2,		//!<  2, COMPOUND_TEXT source text 
	XeTextExport_ODIF_F = 3,	//!<  3, COMPOUND_TEXT formatted text 
	XeTextExport_ODIF_FP = 7,	//!<  7, ODIF formatted processable 
	XeTextExport_TILDE = 8	//!<  8, MicroImages {~ ... } codes 
   } XeTextExport;

#define XeTextExportFormatted(f) ((f)&1)
#define XeTextExportOdif(f) ((f)&2)

typedef struct XeFontListRec *XeFontList; //!<  Defined in TextFont.c 

typedef struct XeFont
    {
	int font_size;
	XeFontList font_list;
    } XeFont;

typedef struct XeItemization
    {
	int identifier_alignment;
	int identifier_start_offset;
	int identifier_end_offset;
    } XeItemization;

typedef struct XeTabStop
    {
	String tabulation_reference;
	Dimension tabulation_position;
	int alignment;
	String alignment_character_string;
    } XeTabStop;
	
/*
** XeTextInsertContext
**	(Defined in TextImport)
*/
typedef struct XeTextInsertContextRec *XeTextInsertContext;
/*
** XeTextTag
*/
typedef XtPointer XeTextTag;

enum XeSyntaxState {
	XeSYNTAX_Normal = 0,
	XeSYNTAX_Keyword,
	XeSYNTAX_Constant,
	XeSYNTAX_SingleQuotedString,
	XeSYNTAX_DoubleQuotedString,
	XeSYNTAX_Comment,
	XeSYNTAX_Function
	};

typedef struct XeTextSyntaxParseCallbackData {
	const MIUNICODE* str;
	XeSyntaxState* state;	
	int len;
	XeSyntaxState initial_state;	// The last state value of the previous snip
	Boolean start_of_line;			// True if this is the start of a line
	} XeTextSyntaxParseCallbackData;

typedef struct XeTextHyperlinkCallbackData {
	const MIUNICODE* str;
	} XeTextHyperlinkCallbackData;

typedef struct XeTextToggleWYSIWYGCallbackData {
	Boolean state;
	} XeTextToggleWYSISYGCallbackData;

/*
** XeTextLayoutCallbackData
**	is the data supplied for the callbacks generated by the
**	layout process. This callback is made via XtNlayoutCallback.
*/
typedef struct XeTextLayoutCallbackData
    {
	int reason;		/* == XeCR_LAYOUT_CHANGE_TAG ==
				** == XeCR_LAYOUT_BEGIN_LINE ==
				*/
	XeTextTag Old, New;	//!<  Old and New tag values 
	XeTextInsertContext context; //!<  Insert context for the feed 
	int (*feed)(XeTextInsertContext context, const MIUNICODE *string, int length);
    } XeTextLayoutCallbackData;


/* The XeText widget supports the following escape sequences 
*	as defined in ISO 8613-6  11.1.8 Table 4
*/
#define XeTEXT_BoldOn			"\033[1m"
#define XeTEXT_BoldOff			"\033[22m"
#define XeTEXT_ItaliacOn		"\033[3m"
#define XeTEXT_ItaliacOff		"\033[23m"
#define XeTEXT_UnderlineOn		"\033[4m"
#define XeTEXT_UnderlineOff	"\033[24m"
#define XeTEXT_FrameBox			"\033[51m"
#define XeTEXT_FrameCircled	"\033[52m"	/* Not supported */
#define XeTEXT_OverlineOn		"\033[53m"	/* Not supported */
#define XeTEXT_FrameNone		"\033[54m"
#define XeTEXT_OverlineOff		"\033[55m"	/* Not supported */

#define XeTEXT_FGBlack			"\033[30m"
#define XeTEXT_FGRed				"\033[31m"
#define XeTEXT_FGGreen			"\033[32m"
#define XeTEXT_FGYellow			"\033[33m"
#define XeTEXT_FGBlue			"\033[34m"
#define XeTEXT_FGMagenta		"\033[35m"
#define XeTEXT_FGCyan			"\033[36m"
#define XeTEXT_FGWhite			"\033[37m"
#define XeTEXT_FGDefault		"\033[39m"
#define XeTEXT_BGBlack			"\033[40m"
#define XeTEXT_BGRed				"\033[41m"
#define XeTEXT_BGGreen			"\033[42m"
#define XeTEXT_BGYellow			"\033[43m"
#define XeTEXT_BGBlue			"\033[44m"
#define XeTEXT_BGMagenta		"\033[45m"
#define XeTEXT_BGCyan			"\033[46m"
#define XeTEXT_BGWhite			"\033[47m"
#define XeTEXT_BGDefault		"\033[49m"
#define XeTEXT_LinkHREF			"\033[5m"	// Note: these are really the Blink
#define XeTEXT_LinkText			"\033[6m"	// modes, but we reused them
#define XeTEXT_LinkEnd			"\033[25m"



/* New fields */
#define XtNalignment "alignment"
#define XtCAlignment "Alignment"
#define XtRXeAlignment "XeAlignment"
#define XtNexportFormat "exportFormat"
#define XtCExportFormat "ExportFormat"
#define XtRXeTextExport "XeTextExport"
#define XtNfonts "fonts"
#define XtCFonts "Fonts"
#define XtRXeFonts "XeFonts"
#define XtNfirstLineOffset "firstLineOffset"
#define XtCOffset "Offset"
#define XtNformat "format"
#define XtCFormat "Format"
#ifndef XtNinitialState
#define XtNinitialState "initialState"
#endif
#define XtCSequence "Sequence"
#define XtNgraphicRendition "graphicRendition"
#define XtCRendition "Rendition"
#define XtNitemization "itemization"
#define XtCItemization "Itemization"
#define XtRXeItemization "XeItemization"
#define XtNkerningOffset "kerningOffset"
#define XtCKerningOffset "KerningOffset"
#define XtRKerningOffset "XeKerningOffset"
#define XtNlineLayoutTable "lineLayoutTable"
#define XtCLineLayoutTable "LineLayoutTable"
#define XtRXeLineLayoutTable "XeLineLayoutTable"
#define XtNlineSpacing "lineSpacing"
#define XtCLineSpacing "LineSpacing"
#define XtNindentation "indentation"
#define XtCIndentation "Indentation"
#define XtNproportional "proportional"		// Don't use - has to do with vertical spacing
#define XtCProportional "Proportional"
#define XeNuseFixedWidthFont "useFixedWidthFont"
#define XeCUseFixedWidthFont "UseFixedWidthFont"
#define XtNcolumnWidth "columnWidth"
#define XtCColumnWidth "ColumnWidth"
#define XtNlayoutCallback "layoutCallback"
#define XtCLayoutCallback "LayoutCallback"
#define XtNsyntaxParserCallback "syntaxParserCallback"
#define XtCSyntaxParserCallback "SyntaxParserCallback"
#define XtNsyntaxHighlighting "syntaxHighlighting"
#define XtCSyntaxHighlighting "SyntaxHighlighting"
#define XtNhyperlinkCallback "hyperlinkCallback"
#define XtCHyperlinkCallback "HyperlinkCallback"
#define XtNtoggleWYSIWYGCallback "toggleWYSIWYGCallback"
#define XtCToggleWYSIWYGCallback "ToggleWYSIWYGCallback"

#define XeNtabInterval "tabInterval"
#define XeCTabInterval "TabInterval"
#define XeNhideCursorWhenNoFocus	"hideCursorWhenNoFocus"
#define XeCHideCursorWhenNoFocus	"HideCursorWhenNoFocus"
#define XeNkeepEndVisable "keepEndVisable"
#define XeCKeepEndVisable "KeepEndVisable"
#define XeNenableContextMenu "enableContextMenu"
#define XeCenableContextMenu "EnableContextMenu"
#define XeNwordWrap	"wordWrap"
#define XeCWordWrap	"WordWrap"
#define XeNwysiwyg	"wysiwyg"
#define XeCWysiwyg	"Wysiwig"
/* Class record constants */


typedef struct _XeTextClassRec *XeTextWidgetClass;
typedef struct _XeTextRec      *XeTextWidget;

_XFUNCPROTOBEGIN

extern WidgetClass xeTextWidgetClass;

/*
** XeTextLayout
**	Run all of the widget content trough the layout process and
**	redisplay the new content.
*/
void XeTextLayout (
	 Widget
	 );


#define XeTextMakeEndVisible XeTextMakeEndVisable

void XeTextMakeEndVisable(Widget);
void XeTextRemove(Widget);
Widget XeCreateText(Widget, char *, Arg *, Cardinal);
Widget XeCreateScrolledText(Widget, char *, Arg *, Cardinal);

_XFUNCPROTOEND

// Move these outsie the _XFUNCPROTOEND so they're C++ linkage, not C

//! Compute the width of a given string and set the widget to be just
//! wide enough to handle the text.
void XeTextSetWidthFromString (
	Widget widget,
	const MIUNICODE* str
	);

//! Compute the width of a given string and set the widget to be just
//! wide enough to handle the text.
void XeTextSetWidthFromString (
	Widget widget,
	const char* str
	);


//! Change the colors in the ANSI color map.
//! This changes the colors used for the ANSI escape sequences
//!   \<ESC\>[30m - \<ESC\>[37m   (foreground colors)
//! 	\<ESC\>[40m - \<ESC\>[47m	(background colors)
//! The index parameter must be in the range of 0 - 7
//! The default colors are...
//!
//!	0 - black
//!	1 - red
//!	2 - green
//!	3 - yellow
//!	4 - blue
//!	5 - magenta
//!	6 - cyan
//!	7 - white
void XeTextSetColor (
	Widget widget,
	int colorindex,
	const COLOR& color
	);

//! Convert an offset (from start of string) to line/col.
//! Numbers returned start at 0, so add 1 if showing to user.
//! Returns true if offset is within the current content, false if not.
bool XeTextOffset2LinCol (
	Widget widget,
	int offset,
	int& line,
	int& col
	);

#endif  // INC_MI32_TEXT_H
