/**
 * \file rvc/dbtable.h
 * \brief interface for the RVC::DBTABLE class.
 *
 * \if NODOC
 * $Id: dbtable.h_v 1.251 2013/05/09 20:40:03 mju Exp $
 *
 * $Log: dbtable.h_v $
 * Revision 1.251  2013/05/09 20:40:03  mju
 * Add op_increment for dbtable::usage.
 *
 * Revision 1.250  2013/02/01 18:57:24  mju
 * Add setHidden.
 *
 * Revision 1.249  2012/09/18 13:55:11  mju
 * Dox ctor fix.
 *
 * Revision 1.248  2012/06/22 17:26:44  mju
 * Add usage_FieldOfView.
 *
 * Revision 1.247  2012/05/15 18:52:48  mju
 * Add fieldinfo::isString.
 *
 * Revision 1.246  2011/12/08 15:43:50  mju
 * Remove record defns leftover from tntsim3d.
 * Just use single linkparms class with public ctors.
 *
 * Revision 1.245  2011/12/07 22:35:35  mju
 * Remove redundant hdrs.
 *
 * Revision 1.243  2010/09/13 19:24:38  mju
 * Add makeparms::useFieldNamesAsHeading.
 *
 * Revision 1.242  2010/08/16 17:03:12  mju
 * Add usage_Properties.
 *
 * Revision 1.240  2010/02/17 14:16:16  mju
 * Add open using mistring.
 *
 * Revision 1.237  2010/02/15 17:28:41  mju
 * Add record::value class.
 *
 * Revision 1.236  2009/10/20 13:32:53  mju
 * Add clarified field types.
 *
 * Revision 1.234  2009/09/02 22:11:18  mju
 * Add m_bufSize so don't have to get from table every time.
 *
 * Revision 1.232  2009/04/01 14:56:12  mju
 * Allow increment op for fieldinfo::type enum.
 *
 * Revision 1.230  2009/03/27 13:45:14  mju
 * Add usage_picklist.
 *
 * Revision 1.229  2009/02/12 20:35:14  mju
 * Add makeparm_style_element::setupStaticFields.
 *
 * Revision 1.228  2008/12/04 22:07:48  bcolombe
 * Add bGetUnitsShown to GetUnits to convert from shown to stored units
 *
 * Revision 1.227  2008/10/01 22:34:38  scowan
 * Added makeparms for oracle.  Used to create a table in oracle database.
 *
 * Revision 1.226  2008/09/29 19:47:22  mju
 * Use dbfieldconstraint enum values for enum values in dbtable:;constraint.
 *
 * Revision 1.224  2008/09/25 22:26:09  scowan
 * Added copyMatchingFieldsTo method.
 *
 * Revision 1.222  2008/09/05 17:47:01  scowan
 * Added USAGE Shape and isUserDefined method.
 *
 * Revision 1.220  2008/09/03 22:48:32  scowan
 * Added getFieldInfo for record class.
 *
 * Revision 1.219  2008/08/28 20:57:32  mju
 * Add recordreftype enum.
 *
 * Revision 1.218  2008/06/16 20:10:15  scowan
 * Added shape and table object link to las files.
 *
 * Revision 1.217  2008/05/07 17:58:22  vdronov
 * substituted Unknkwn attachment by NoRestrictions
 *
 * Revision 1.213  2007/11/01 18:00:51  scowan
 * Added table encoding field to record.
 *
 * Revision 1.206  2007/06/25 20:07:03  dwilliss
 * added table usage for enumeration tables
 *
 * Revision 1.202  2007/05/08 21:41:08  dwilliss
 * Added SetPromptOnAddElement method to DBTABLE
 *
 * Revision 1.196  2007/03/19 20:36:48  dwilliss
 * TestIsJoinable need CopyFlags parameter
 *
 * Revision 1.184  2006/10/05 14:59:54  bcolombe
 * Added optional threshold value to CompareField and CompareFieldOrdered
 * Added CompareRecords function which works same as operator== function for comparing records but takes threshold
 *
 * Revision 1.178  2006/09/06 22:17:16  dwilliss
 * Changed the FIELDINFO::SetName() method that took a TEXTID to take an MISTRING
 * instead.  Since MISTRING has implicit ctor from TEXTID, it still works either way.
 *
 * Revision 1.156  2006/06/09 15:52:39  dwilliss
 * Added MakeDistinctTableFromField
 *
 * Revision 1.147  2006/04/19 20:50:58  dwilliss
 * defined FORMAT enum and some methods to get/set it
 *
 * Revision 1.140  2006/01/25 15:40:33  dwilliss
 * Added ComputeMinimumAttachmentType
 *
 * Revision 1.138  2006/01/09 20:25:14  dwilliss
 * Added optional encoding parameters to a couple methods
 *
 * Revision 1.132  2005/11/18 23:11:49  scowan
 * Added findFieldByIdentifier.
 *
 * Revision 1.127  2005/10/28 16:45:42  dwilliss
 * Added a GetBookmark to table iterator
 * deprecated get record num
 *
 * Revision 1.125  2005/10/12 16:47:48  dwilliss
 * Added a Write method that takes a bookmark
 *
 * Revision 1.1  2001/05/23 20:24:54  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBTABLE_H
#define INC_RVC_DBTABLE_H

#ifndef INC_RVC_DBASE_H
	#include <rvc/dbase.h>
#endif

#ifndef INC_RVC_DBFIELDCONSTRAINT_H
	#include <rvc/dbfieldconstraint.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#ifndef INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif

#ifndef INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif

#ifndef	INC_MI32_GREMODE_H
	#include <mi32/gremode.h>
#endif

#ifndef	INC_MI32_ENUMSET_H
	#include <mi32/enumset.h>
#endif

#ifndef	INC_MI32_DATETIME_H
	#include <mi32/datetime.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_UOM_H
	#include <mi32/uom.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define INC_VECTOR
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
struct UNITCONV;
struct DBFIELDINFO;
struct DBCONSTRAINT;
namespace SML {
	class TABLEDATA;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
namespace RVC {
	struct DBCALLBACKSTRUCT;
	class SHAPE;
	class STYLE;
	class DBASE_ELEMENT;
	class TABLEJOININFO;
	class DBTRANSINFO;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Table objects.  
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Table.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Table.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Database
//!		- RVC::OBJTYPE_Database_Raster
//!		- RVC::OBJTYPE_Database_VectorPoint
//!		- RVC::OBJTYPE_Database_VectorLine
//!		- RVC::OBJTYPE_Database_VectorPolygon
//!		- RVC::OBJTYPE_Database_CAD
//!		- RVC::OBJTYPE_Database_TIN
//!		- RVC::OBJTYPE_Database_Shape
class DBTABLE : public RVC::OBJECT {
	public:

		//! Database table attachment type 
		enum ATTACHMENT {
			ATTACHMENT_Unknown = 0,				//!< Your standard free-for-all
			ATTACHMENT_OneRecordPerElement,	//!< But may use record for multiple elements
			ATTACHMENT_OneElementPerRecord,	//!< But element may have multiple records
			ATTACHMENT_OneToOne,					//!< At most one record per element and one element per record
			ATTACHMENT_RelatedOnly,				//!< No attachments at all, only related
			ATTACHMENT_Identity,					//!< Record number is element number
			ATTACHMENT_NoRestrictions			//!< Your standard attached to element free-for-all
			};
			
		//! Predefined table ID's
		enum DEFINEDID {
			DEFINEDID_Internal =		 -1,			//!< The "Built-in" table.  Not valid for TIN internal tables
			DEFINEDID_InternalNode = -2,			//!< Not valid for TIN internal tables
			DEFINEDID_Invalid = 		-98,
			DEFINEDID_New =	 		 -9
			};
			
		//! Flags for finding attached record methods (i.e. RVCDBTABLE::GetAttachedRecords())
		enum FINDFLAG {
			FINDFLAG_AllAttached = 0,
			FINDFLAG_OnlyDirect = 2,
			FINDFLAG_FirstMatch = 8
			};

		//! Flags for searching on an index methods (i.e. RVCDBTABLE::SearchIndex())
		enum INDEXSEARCH {
			INDEXSEARCH_ReturnIndexOffset = 	0x00000000,
			INDEXSEARCH_CheckExists = 			0x00000001,
			INDEXSEARCH_ReturnRecordNumber = 0x00000002,
			INDEXSEARCH_ReturnInsertOffset = 0x00000004	
			};
			
		enum ITERATERECORD {
			ITERATERECORD_None = 			0x0000,
			ITERATERECORD_SkipDeleted = 	0x0001,
			ITERATERECORD_Sort = 			0x0002,
			ITERATERECORD_UniqueOnly = 	0x0004,
			ITERATERECORD_SaveIndex = 		0x0008,
			ITERATERECORD_SingleKey = 		0x8000
			};
			
		//! Type of external table
		enum LINKTYPE {
			LINKTYPE_Internal = 0,	//!< TNTmips internal format
			LINKTYPE_MIPS,				//!< MIPS external format (old RVF style) 
			LINKTYPE_DBASE3,			//!< dBase III+ format (external only)
			LINKTYPE_INFO,	 			//!< INFO format (external only)
			LINKTYPE_DBASE4,			//!< dBase IV (external only)
			LINKTYPE_RBASE, 			//!< R:BASE (external only)
			LINKTYPE_DBASE2,			//!< Dbase 2 (external only)
			LINKTYPE_ODBC,				//!< Link to ODBC server (external only)
			LINKTYPE_CSV,				//!< Comma Seperated values (export only)
			LINKTYPE_TAB,				//!< MapInfo TAB files
			LINKTYPE_ORACLE,			//!< Oracle server
			LINKTYPE_RDBMS,			//!< OLEDB or PostgreSQL
			LINKTYPE_LAS,				//!< LiDAR format 1.1 file
			LINKTYPE_NUMTYPES			//!< The number of link types
			};

		enum KEYFLAGS {
			KEYFLAG_None =					0x0000,
			KEYFLAG_PrimaryKey =			0x0001,	//!< Field is a primary key (force index)
			KEYFLAG_ForeignKey =			0x0002,	//!< Field is a foreign key (index optional)
			KEYFLAG_Indexed = 			0x0004,	//!< Field has index 
			KEYFLAG_RebuildIndex = 		0x2000,	//!< Rebuild the index if already there
			KEYFLAG_Sorted = 				0x4000	//!< Hint that table is already sorted
			};

		//! Record referencing type.
		enum RECORDREFTYPE {
			RECORDREFTYPE_New =			0,		//!< Reference to "new" record
			RECORDREFTYPE_Absolute =	1,		//!< Reference number is absolute
			RECORDREFTYPE_Bookmark =	2		//!< Reference number is bookmark value
			};

		//! Flags for element referenced tables  (TIN is the only one so far)
		enum REFELEM {
			REFELEM_All			= 0x00000000,
			REFELEM_Node		= 0x00000010,
			REFELEM_Edge 		= 0x00000020,
			REFELEM_Triangle	= 0x00000040,
			REFELEM_Hull		= 0x00000080	
			};
			
		//! Result values for CompareStructures() method
		enum STRUCTCOMPARE {
			STRUCTCOMPARE_DoNotMatch = 0,		//!< Table structures do not match
			STRUCTCOMPARE_Exact,					//!< Table structures match exactly
			STRUCTCOMPARE_Compatible,			//!< Table structures are compatible, record fields can be converted without loss of data
			STRUCTCOMPARE_SameTable				//!< Both tables point to the same linked table
			};
		
		//! Table capability flags, passed to RVCDBTABLE::HasCapability()
		enum CAPABILITY {
			CAPABILITY_Create	= 2048,
			CAPABILITY_Link,
			CAPABILITY_ReadMemo,
			CAPABILITY_WriteMemo,
			CAPABILITY_WriteRecord,
			CAPABILITY_MultiTable,		
			CAPABILITY_Server,
			CAPABILITY_Import,
			CAPABILITY_Export,
			CAPABILITY_DeleteRecord,
			CAPABILITY_UserWritable,
			CAPABILITY_CanSetUserWrite,
			CAPABILITY_ChangeFields,
			CAPABILITY_AddFields,
			CAPABILITY_DeleteFields,
			CAPABILITY_AddRecord,
			CAPABILITY_RandomAccess,
			CAPABILITY_ReadBinary,
			CAPABILITY_WriteBinary,
			CAPABILITY_WriteIfOpenForWrite,	//!< Will tell us if we _could_ get write permission if we tried, even if currently open for read only
			CAPABILITY_BookmarksAreNotAbsolute //!< If a table has this "capability", then bookmarks are hash values, not absolute record numbers
			};
			
		enum PERMISSIONS {
			PERMISSION_View = 0,
			PERMISSION_Modify,
			PERMISSION_Delete,
			PERMISSION_ViewStructure,
			PERMISSION_ModifyStructure,
			PERMISSION_Duplicate,
			PERMISSION_ReadACL,
			PERMISSION_ChangeACL
			};

		//! Table usage, defines specialized tables
		enum USAGE {
			USAGE_UserDefined =			0,
			USAGE_StdStats =				2,
	 		USAGE_ElementID =				3,
			USAGE_NodeTurn =				4,
			USAGE_ScaleSelect =			5,
			USAGE_Section =				6,
			USAGE_Internal =				7,
			USAGE_PointStyle =			8,
			USAGE_LineStyle =				9,
			USAGE_PolyStyle =				10,
			USAGE_TextStyle =				11,
	 		USAGE_MultiStyle =			12,
	 		USAGE_CADStyle =				USAGE_MultiStyle,
	 		USAGE_EditLog =				13,
	 		USAGE_Selection =				14,
			USAGE_FlightPath =			15,
			USAGE_PointOfInterest =		16,
			USAGE_LegendElement =		17,
			USAGE_StyleParameters =		18,
			USAGE_SelectParameters =	19,
			USAGE_ExtrudedPoints =		20,
			USAGE_InfluenceSphere =		21,
			USAGE_Sketch =					22,	//!< user-defined table on a sketch layer
			USAGE_ClumpRaster =			23,	//!< Output of Raster Clump process
			USAGE_Enumeration =			24,
			USAGE_Shape =					25,	//!< The 'directly' attached table for the shape object
			USAGE_PickList =				26,	//!< Pick list table
			USAGE_Properties =			27,	//!< Properties, typically for spatial object
			USAGE_FieldOfView =			28,	//!< Field of view (for viewshed)
			USAGE_LAST
			};

		enum VIEWMODE {
			VIEWMODE_Tabular			= 0,
			VIEWMODE_SingleRecord	= 1
			};
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class BOOKMARK;
		class CURSOR;
		class FIELDINFO;
		class GETOBJFILTER_ISSELECTABLE;
		class GETOBJFILTER_ISUSAGEVALID;
		class GETOBJFILTER_ISUSERDEFINED;
		class RECORD;
		friend class RECORD;
		class RECORD_ID_ELEMENT;
		class RECORD_ID_POINT;
		class RECORD_LEGENDELEMENT;
		class RECORD_NODETURN;
		class RECORD_SCALESELECT;
		class RECORD_SELECTION;
		class RECORD_STYLE_CAD;
		typedef RECORD_STYLE_CAD RECORD_STYLE_MULTI;
		class RECORD_STYLE_ELEMENT;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		typedef ENUMSET<USAGE, USAGE_LAST> USAGESET;
		typedef std::vector<FIELDINFO> FIELDINFOARRAY;
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CONSTRAINT;
		class LINKPARMS;
		class MAKEPARMS;
		class MAKEPARM_ID_ELEMENT;
		class MAKEPARM_ID_POINT;
		class MAKEPARM_STYLE_ELEMENT;
		class MAKEPARM_STYLE_CAD;
		typedef MAKEPARM_STYLE_CAD MAKEPARM_STYLE_MULTI;
		class MAKEPARM_STYLE_LINE;
		class MAKEPARM_STYLE_POINT;
		class MAKEPARM_STYLE_POLYGON;
		class MAKEPARM_SCALESELECT;
		class MAKEPARM_SELECTION;
		class MAKEPARM_LEGENDELEMENT;
		class MAKEPARM_NODETURN;
		class MAKEPARM_ORACLE;
		class RECORDITERATOR;
		class CONST_RECORDITERATOR;
		class OBSERVER;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Constraint class for the field in the table
		class CONSTRAINT {
			public:
				enum CASE {
					CASE_None =					0,
					CASE_ForceUpperCase =	DBFIELDCONSTRAINT_Case_AllUpper,
					CASE_ForceLowerCase =	DBFIELDCONSTRAINT_Case_AllLower,
					CASE_CapFirstLetter =	DBFIELDCONSTRAINT_Case_FirstLetterCap,
					CASE_CapFirstWord =		DBFIELDCONSTRAINT_Case_FirstWordCap,
					};
					
				enum PKTABLE {
					PKTABLE_None =					0,
					PKTABLE_MustExist =			DBFIELDCONSTRAINT_PrimaryKey_MustExist,
					PKTABLE_AutoAdd =				DBFIELDCONSTRAINT_PrimaryKey_AutoAdd,
					PKTABLE_AutoAddPrompt =		DBFIELDCONSTRAINT_PrimaryKey_PromptAdd,
					};
			
				enum KEYMODE {
					KEYMODE_None =					0,
					KEYMODE_ComboBox =			DBFIELDCONSTRAINT_UI_ComboBox,
					KEYMODE_OptionMenu =			DBFIELDCONSTRAINT_UI_OptionMenu,
					KEYMODE_CSVListAllowDups =	DBFIELDCONSTRAINT_ValueList_AllowDups,
					KEYMODE_CSVListUnique =    DBFIELDCONSTRAINT_ValueList_NoDups,
					};
			
				enum RANGEFLAG {
					RANGEFLAG_None =				0,
					RANGEFLAG_MinExclusive =	DBFIELDCONSTRAINT_Range_MinExclusive,
					RANGEFLAG_MaxExclusive =	DBFIELDCONSTRAINT_Range_MaxExclusive,
					RANGEFLAG_MinInclusive =	DBFIELDCONSTRAINT_Range_MinInclusive,
					RANGEFLAG_MaxInclusive =	DBFIELDCONSTRAINT_Range_MaxInclusive,
					RANGEFLAG_MINMASK =			DBFIELDCONSTRAINT_Range_Min_MASK,
					RANGEFLAG_MAXMASK =			DBFIELDCONSTRAINT_Range_Max_MASK,
					};
			
				enum DEFAULTFLAG {
					DEFAULT_Zero =					0,
					DEFAULT_Constant = 			DBFIELDCONSTRAINT_Default_Constant,
					DEFAULT_Blank = 				DBFIELDCONSTRAINT_Default_Empty,
					};

				//! Default Constructor
				CONSTRAINT (
					);
			
				//! Copy Constructor
				CONSTRAINT (
					const CONSTRAINT& rhs
					);
			
				//! Destructor
				~CONSTRAINT (
					) {}

			#ifndef NO_DEPRECATED
				//! \deprecated Redundant with FIELDINFO::FLAG_NotNULL
				DEPRECATED bool CanBeEmpty (
					) const {return (!m_NotNull);}
			#endif
					
				void Clear (
					);

				CASE GetCaseFlags (
					) const {return (m_Case);}
				
				double GetDefaultNum (
					) const {return (m_DefaultNum);}
					
				DEFAULTFLAG GetDefaultFlags (
					) const {return (m_DefaultFlags);}
					
				KEYMODE GetKeyMode (
					) const {return (m_KeyMode);}
					
				const MISTRING& GetPrefixLabel (
					) const {return (m_Label);}
					
				const MISTRING& GetPostfixLabel (
					) const {return (m_LabelAfter);}
					
				PKTABLE GetPkTableFlags (
					) const {return (m_PkTable);}
				
				const DOUBLE_RANGE& GetRange (
					) const {return (m_Range);}
					
				RANGEFLAG GetRangeFlags (
					) const {return (m_RangeFlags);}
					
				bool IsAutoComplete (
					) const {return (m_AutoComplete);}
					
				void SetAutoComplete (
					bool value
					) {m_AutoComplete = value;}
					
				void SetCaseFlags (
					CASE flags
					) {m_Case = flags;}
				
				void SetDefaultNum (
					double number
					) {m_DefaultNum = number;}
					
			#ifndef NO_DEPRECATED
				//! \deprecated Redundant with FIELDINFO::FLAG_NotNULL
				DEPRECATED void SetEmpty (
					bool value
					) {m_NotNull = !value;}
			#endif
					
				void SetDefaultFlags (
					DEFAULTFLAG flags
					) {m_DefaultFlags = flags;}
					
				void SetKeyMode (
					KEYMODE flags
					) {m_KeyMode = flags;}
					
				void SetPrefixLabel (
					const MISTRING& label
					) {m_Label = label;}
					
				void SetPostfixLabel (
					const MISTRING& label
					) {m_LabelAfter = label;}
					
				void SetPkTableFlags (
					PKTABLE flags
					) {m_PkTable = flags;}
				
				void SetRange (
					const DOUBLE_RANGE& range
					) {m_Range = range;}
			
				void SetRangeFlags (
					RANGEFLAG flags
					) {m_RangeFlags = flags;}
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
			
				RANGEFLAG m_RangeFlags;
				DEFAULTFLAG m_DefaultFlags;
				KEYMODE m_KeyMode;
				CASE m_Case;
				PKTABLE m_PkTable;
				DOUBLE_RANGE m_Range;
				double m_DefaultNum;
				bool m_NotNull;
				bool m_AutoComplete;
				MISTRING m_Label;
				MISTRING m_LabelAfter;
				
				void GetConstraint (DBCONSTRAINT& rhs) const;
				void SetConstraint (const DBCONSTRAINT& rhs);

				friend class DBTABLE;
				friend class SHAPE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		class FIELDINFO {
			public:

				//!	pseudo fields for FIELDINFO.next_field.  Some database schemas
				//! require a 2 part key where the primary key is made up of 2 fields
				//! but the foreign key is made up of 1 field and the name of the field
				//! or one field and a "domid" which is looked up from a 3rd table by
				//! using the table/field name.  We look those up on import and record
				//! them in the FIELDINFO.domid.  I don't know of anything that would
				//! use the table name, but I added it for completeness 
				enum NEXTFIELD {
					NEXTFIELD_ThisFieldName = 0xFFFF,	//!< Name of this field is the key data 
					NEXTFIELD_ThisTableName = 0xFFFE,	//!< Name of this table is the key data 
					NEXTFIELD_DomID			= 0xFFFD,	//!< Key data is the "domid" field of this field 
					NEXTFIELD_PseudoField	= 0xFFFC		//!< If next_field is > this, it's one of these 
					};
					
				enum FLAGS {
					FLAG_None =					0x0000,
					FLAG_PrimaryKey =			0x0001,	//!< Field is a primary key (force index)
					FLAG_PK = 					FLAG_PrimaryKey,
					FLAG_ForeignKey =			0x0002,	//!< Field is a foreign key related to field in another table
					FLAG_Indexed = 			0x0004,	//!< Field has index 
					FLAG_ComputeOnRead = 	0x0008,	//!< Field is computed 
					FLAG_NotNULL = 			0x0010,	//!< Value cannot be null/empty
					FLAG_Unique = 				0x0020,	//!< value must be unique 
					FLAG_ReadOnly = 			0x0040,	//!< User can't change value 
					FLAG_SecondaryKey =		0x0080,	//!< Secondary (or higher) key 
					FLAG_Invisible = 			0x0100,	//!< Don't show the field at all 
					FLAG_Closest = 			0x0200,	//!< Related to "closest" value
					FLAG_RangeAbove = 		0x0400,	//!< Related to value above or equal
					FLAG_RangeBelow = 		0x0800,	//!< Related to value below or equal
					FLAG_ComputeOnWrite = 	0x1000,	//!< Compute field on write 
					FLAG_ComputedMASK = 		0x1008,
					FLAG_LocalTime = 			0x2000,	//!< TYPE_DateTime field is stored in local time
					FLAG_AutoIncrement =		0x4000,	//!< Assignes the "AutoIncrement" value to the field and then increments "AutoIncrement" (default start is '1')
					FLAG_SSIndex =				0x10000,	//!< Field has index (in ODBC), but we can't change it
					FLAG_Searchable =			(FLAG_Indexed | FLAG_SSIndex)
					};

				enum TYPE {	
					TYPE_Invalid = 0,			//!< Invalid field type
					TYPE_String = 1,			//!< ASCII character string, null-terminated 
					TYPE_Char = 2,				//!< Single ASCII character 
					TYPE_UINT8 = 3,			//!< Unsigned 1-byte integer (0-255)
					TYPE_SINT8 = 4,			//!< Signed 1-byte integer (-128-128)
					TYPE_UINT16 = 5,			//!< Unsigned 2-byte integer (0-65535)
					TYPE_SINT16 = 6,			//!< Signed 2-byte integer (-32768-32767)
					TYPE_UINT32 = 7,			//!< Unsigned 4-byte integer (0 - 2^32-1)
					TYPE_SINT32 = 8,			//!< Signed 4-byte integer (-2^31 - 2^31-1)
					TYPE_Float = 9,			//!< 4-byte floating-point number
					TYPE_Double = 10,			//!< 8-byte floating-point number
					TYPE_Logical = 11,		//!< 1-byte logical (?/Y/y/N/n/T/t/F/f/0/1)
					TYPE_Date = 12,			//!< 4-byte integer date YYYYMMDD
					TYPE_UINT64 = 13,			//!< Unsigned 8-byte integer (0 - 2^64-1)
					TYPE_SINT64 = 14,			//!< Signed 8-byte integer (-2^63 - 2^63-1)
					TYPE_Color = 15,			//!< Color spec. Always 8 bytes (COLOR)
					TYPE_Binary = 16,			//!< Binary field
					TYPE_Memo = 17,			//!< Memo field
					TYPE_Unicode = 18,		//!< MIUNICODE string, null-terminated
					TYPE_DateTime = 19,		//!< Date/Time field, 100-nanosecond intervals from 1-Jan-1601
					TYPE_NUMTYPES,				//!< Number of defined field types
				#ifndef GENERATING_DOXYGEN_OUTPUT
					TYPE_UByte = TYPE_UINT8,
					TYPE_Byte = TYPE_SINT8,
					TYPE_UWord = TYPE_UINT16,
					TYPE_Word = TYPE_SINT16,
					TYPE_ULong = TYPE_UINT32,
					TYPE_Long = TYPE_SINT32,
					TYPE_UDLong = TYPE_UINT64,
					TYPE_DLong = TYPE_SINT64,
				#endif
					};
					
				enum USAGE {			
					USAGE_Undefined = 0,
					USAGE_XCoord = 1,
					USAGE_YCoord,
					USAGE_ZCoord,
					USAGE_Label,
					USAGE_Filename,
					USAGE_ObjName,
					USAGE_ImageBLOB,				//<! For Binary BLOBs only
					USAGE_ImageFile,				//<! Field holds a filename pointing to an image
					USAGE_ImageURL,				//<! Field holds a URL pointing to an image
					USAGE_ImageName,
					USAGE_Spatial,
					USAGE_HyperLink,				//<! A URL to anything
					USAGE_DataTip,
					USAGE_Heading,					
					USAGE_Velocity,				//!< Units member will specify the units
					USAGE_CRS,
					USAGE_DateTime,
					USAGE_ImageThumbnail,		//!< Image thumbnail in a blob.  Thumbnail is for preceding USAGE_ImageBLOB or USAGE_ImageFile field
					USAGE_ImageDescription,		//!< Fields holds the description of the image
					USAGE_RecordModTime,			//!< Time the record was last written -- automatically updated
					USAGE_EnumLabel,
					USAGE_EnumToolTip,
					USAGE_ElementID,				//!< Initially from GML 2.0.  Each element can have a built in name, description and ID values
					USAGE_ElementName,
					USAGE_ElementDesc,
					USAGE_LAST
					};
					
				// Note: format is stored as a UINT8 in the fieldinfo
				enum FORMAT {			
					FORMAT_Undefined = 0,
					FORMAT_OGC_WKB = 1,		//!< OGC Well Known Binary
					FORMAT_OGC_WKT,			//!< OGC Well Known Text
					FORMAT_ImageJPG,				//!< A JPEG file stored in a blob
					FORMAT_ImagePNG,				//!< A PNG file stored in a blob
					};
					
				typedef ENUMSET<USAGE, USAGE_LAST> USAGESET;
					
				//! Default constructor
				FIELDINFO (
					);

				//! Copy constructor
				FIELDINFO (
					const FIELDINFO& rhs
					);

				~FIELDINFO (
					);

				//! Assignment
				FIELDINFO& operator= (
					const FIELDINFO& rhs
					);		
				
				//! Clear the field info of current values
				void Clear (
					);
					
				//! Clear a single flag
				void ClearFlag (
					FLAGS flag
					);
					
				//! Clear a link from this field to another table
				void ClearLink (
					);
					
				//! Clear the next field number that this field is linked to
				void ClearNextLinkField (
					);
					
				//! Compare two field types and return best field type to convert to
				//! @return this->GetType() if exact match, TYPE_??? depending if the types are
				//! not the best match but still matchable, TYPE_Invalid if types do not match
				TYPE CompareFieldTypes (
					const FIELDINFO& rhs,
					bool& ExactMatch					//!< Set to 'true' if the two fields match exactly, 'false' if they do not
					) const;
					
				//! Get the AutoIncrement number for the next added record
				//! @return Next AutoIncrement number
				UINT32 GetAutoIncrement (
					) const;
			
				//! Get number of decimal places to display for floating point fields
				//! @return Number of decimal places to display		
				int GetDecimalPlaces (
					) const;
					
				//! Get the display units for this field
				//! @return Display unit
				int GetDispUnits (
					) const;
					
				//! Get number of columns to display
				//! @return Number of columns to display		
				int GetDispWidth (
					) const;
					
				//! Get the domain ID for this field
				//! @return Domain ID
				const RVC::OBJECTNAME& GetDomainID (
					) const;
					
				//! Get the flags for this field
				//! @return Field flags (FIELDINFO::FLAGS_...)
				FLAGS GetFlags (
					) const;
					
				//! Get the format of the field.
				//! Currently only used for BLOB fields to record what kind of data is
				//!	stored in the blob.
				//! @return Field usage (FIELDINFO::FORMAT_...)
				FORMAT GetFormat (
					) const;
					
				const RVC::DBTABLE::CONSTRAINT& GetConstraint (
					) const { return (m_Constraint); }

				const RVC::DESCRIPTOR& GetDescriptor (
					) const { return (m_Descriptor); }

			#ifndef NO_DEPRECATED
				//! Get the name of this field
				//! \deprecated Use GetShortName()
				//! @return Field name
				DEPRECATED const RVC::OBJECTNAME& GetName (
					) const {return (GetShortName());}
			#endif

				//! Get the name of this field
				//! \deprecated Call GetDescriptor().GetShortName()
				//! @return Field name
				const RVC::OBJECTNAME& GetShortName (
					) const;
					
				//! Get the name of this field
				//! \deprecated Call GetDescriptor().GetShortName()
				//! @return Field name
				const MISTRING& GetFullName (
					) const {return (m_Descriptor.GetFullName());}

				//! Get the next field number that this field is linked to
				//! @return Next link field number
				INT32 GetNextLinkField (
					) const;
					
				INT32 GetOffset (
					) const;

				//! Get the next field pseudo number that this field is linked to
				//! @return Next link field pseudo number (FIELDINFO::NEXTFIELD_...)
				NEXTFIELD GetPseudoNextLinkField (
					) const;
					
				//! Get the field number that this field is linked to, use HasRelation() to 
				//! determine if the field is related to another table / field
				//! @return Linked field number
				INT32 GetRelatedField (
					) const;
					
				//! Get the table number that this field is linked to, use HasRelation() to 
				//! determine if the field is related to another table / field
				//! @return Related table number
				INT32 GetRelatedTable (
					) const;
					
				//! Get the size of the field in bytes
				//! @return Size of field in bytes
				INT32 GetSize (
					) const;
					
				//! Get the type of the field
				//! @return Type of field (FIELDINFO::TYPE_...)
				TYPE GetType (
					) const;
			
				//! Get the numeric units for this field for unit conversion
				void GetUnits (
					UNITCONV& conv,
					bool bGetUnitsShown = false
					) const;

				//! Get the numeric units for this field. 
				//! By default this method returns the units stored in the database.
				//! You can also get user's preferred units to view the field in, but
				//! that's mostly for historical reasons, as this preference is usually
				//! stored in a subobject of the table, not in the field info.
				UOM::UNIT GetUnits (
					bool bGetUnitsShown = false
					) const;
					
				int GetUnitType () const;

				//! Get the usage of the field
				//! @return Field usage (FIELDINFO::USAGE_...)
				USAGE GetUsage () const;
					
				//! Is the field related to another table / field
				//! @return 'True' if field is related
				bool HasRelation () const;
			
				//! Does the field have a next key field link?
				//! @return 'True' if field has a next key field
				bool HasNextKey () const;
			
				//! Does the field have a next key pseudo field link?
				//! @return 'True' if field has a next pseudo key field
				bool HasPseudoNextKey () const;
					
				//! Is the field numeric?  (not a string that can be converted to a number)
				//! @return 'True' if field is a numeric field
				bool IsNumeric () const;

				//! Is the field a string?
				//! @return 'True' if field is a string.
				bool IsString () const;

				//! Reset position information about the field			
				void ResetPosition ();
					
				//! Set the AutoIncrement number for the next added record
				void SetAutoIncrement (
					UINT32 count
					);

				void SetConstraint (
					const RVC::DBTABLE::CONSTRAINT& Constraint
					) { m_Constraint = Constraint; }
			
				void SetDescription (
					const MISTRING& Description
					) { m_Descriptor.SetDescription(Description); }

				void SetDescriptor (
					const RVC::DESCRIPTOR& Descriptor
					) { m_Descriptor = Descriptor; }

				//! Set the number of decimal places for floating point fields
				void SetDecimalPlaces (
					int decplaces
					);
					
				//! Set the display units for this field
				void SetDispUnits (
					int unit
					);
					
				//! Set the display width (number of columns) for the field
				void SetDispWidth (
					int width
					);
					
				//! Set the Domain ID for this field
				void SetDomainID (
					const RVC::OBJECTNAME& name
					);
					
				//! Set field flags
				void SetFlags (
					FLAGS flags
					);
					
				//! Set a single flag
				void SetFlag (
					FLAGS flag
					);
					
				//! get the format of the field.
				//! Currently only used for BLOB fields to record what kind of data is
				//!	stored in the blob.
				void SetFormat (
					FORMAT format
					);
					
				//! Set this field to link to a field in another table
				void SetLink (
					INT32 TableNum,
					INT32 FieldNum,
					INT32 NextFieldNum = -1
					);
					
				//! Set this field to link to a field in another table
				void SetLink (
					INT32 TableNum,
					INT32 FieldNum,
					NEXTFIELD NextFieldNum
					);
					
				//! Set the name of this field
				void SetName (
					const RVC::OBJECTNAME& name
					);
					
				//! Set the name of this field
				void SetName (
					const MISTRING& name
					);
					
				//! Set the name of this field
				//! The TEXTID version is necessary to resolve an abmiguious function 
				//! call since MISTRING and RVC::OBJECTNAME both have implicit ctors
				//! from TEXTID
				void SetName (
					TEXTID name
					);
					
				//! Set the next field number that this field is linked to
				void SetNextLinkField (
					INT32 NextFieldNum
					);
					
				//! Set this field to link to a field in another table
				void SetNextLinkField (
					NEXTFIELD NextFieldNum
					);
					
				//! Set the offset into the buffer for data of this field.
				//! You don't normally have to set this.  It will be set for you when
				//! you create the table.
				void SetOffset (
					int offset
					);

				//! Set the size of this field in bytes
				void SetSize (
					INT32 size
					);
					
				//! Set the field type
				void SetType (
					TYPE type
					);
			
				//! Set the field units
				//! Note: this sets the unit type and the "Units Stored"
				//! but leaves the "Units Shown" alone.  
				void SetUnits (
					const UNITCONV& conv
					);

				//! Set the field units
				//! This method allows you to choose to set only the units stored or
				//! the units shown.  The "units shown" is mostly for historical 
				//! reasons.
				void SetUnits (
					const UOM::UNIT& units,
					bool bSetUnitsStored = true,
					bool bSetUnitsShown = true
					);
					
				//! Set the field units
				//! This method allows you to choose to set only the units stored or
				//! the units shown.  The "units shown" is mostly for historical 
				//! reasons.
				void SetUnits (
					UOM::IDNUM units,
					bool bSetUnitsStored = true,
					bool bSetUnitsShown = true
					);
					
				//! Set the field units
				//! Note: This will only set the "Units Shown", which is only used by
				//!	Single Record View.  And it will refuse to change it if the UnitType
				//!	of the conv parameter does not match the field's unit type.
				void SetUnitsShown (
					const UNITCONV& conv
					);
								
				//! Set the field usage
				void SetUsage (
					USAGE usage
					);
			
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DBFIELDINFO* m_pFieldInfo;
				RVC::DESCRIPTOR m_Descriptor;
				RVC::DBTABLE::CONSTRAINT m_Constraint;

				const DBFIELDINFO* GetFieldInfoPtr () const {return (m_pFieldInfo); }
				DBFIELDINFO* GetFieldInfoPtr () {return (m_pFieldInfo); }
				
				friend class DBTABLE;
				friend class DBTABLE::CURSOR;
				friend class DBTABLE::RECORD;
				friend class SHAPE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Get TEXTID for attachment.
		static TEXTID GetAttachmentTextID (
			ATTACHMENT attachment			//!< Attachment type
			);
		
		//! Determine if a database type supports a given capability.
		static bool HasDBTypeCapability (
			LINKTYPE LinkType,
			CAPABILITY Capability
			);

		//! Default constructor
		DBTABLE (
			);

		//! Copy constructor
		DBTABLE (								//! Copy Constructor
			const RVC::DBTABLE& rhs
			);

		//! Destructor
		virtual ~DBTABLE (
			);

		//! Assignment
		RVC::DBTABLE& operator= (
			const RVC::DBTABLE& rhs
			);

		//! Add a record to the table
		//! @return Record number for the new record
		//! Note: You should not depend on this returning the record number.
		//!	If the table is a link to any external format, it may be returning
		//!	a bookmark number instead.  Eventually this method will be modified
		//! 	to only return ERRVALUE.  This version would be deprecated, but 
		//!	it's the one you would call if you don't care what the new record
		//!	number is.
		INT32 AddRecord (
			RECORD& Record
			);
		
		//! Add a record to the table
		ERRVALUE AddRecord (
			RECORD& Record,		//!< Record to write
			BOOKMARK& Bookmark	//!< Bookmark of new record RETURNED
			);
		
		//! Create an index on a field in the table
		ERRVALUE BuildIndex (
			INT32 FieldNum,
			KEYFLAGS flags = KEYFLAG_Indexed		//!< Type of key to make
			);

		//! Set table attachment type
		//! Note: This method resolves existing attached/related records and
		//! tries to keep everything correct.  
		ERRVALUE ChangeAttachmentType (
			ATTACHMENT Attachment,
			ELEMTYPE ElemType = ELEMTYPE_Empty	
			);

	#ifndef NO_DEPRECATED
      //! Combine rhs table with this table
      DEPRECATED ERRVALUE Combine (
         const RVC::DBTABLE& rhs,
         RVC::DBASE::JOINFLAGS flags,
         RVC::DBASE& dest,          //!< Database in which to store table
         const MISTRING& name,            //!< Output table name
         const MISTRING& desc             //!< Output table description
         );
	#endif

      //! Combine rhs table with this table
      ERRVALUE Combine (
         const RVC::DBTABLE& rhs,
         RVC::DBASE::JOINFLAGS flags,
         RVC::DBASE& dest,          //!< Database in which to store table
         const RVC::DESCRIPTOR& DestDescriptor	//!< Descriptor of new table
         );

		//! Compare two table structures
		//! @return STRUCTCOMPARE enum value describing if and how the table structures match
		STRUCTCOMPARE CompareStructures (
			const RVC::DBTABLE& rhs
			) const;
			
		//! Determine the minimum ATTACHMENT type based on current attachmets.
		//!
		//! Looks at current element to record attachments to determine the
		//! minimum setting for attachment.  
		//!
		//! If the table is RelatedOnly the returned type will be one of the
		//!	attached (non-related) attachment types.  Call ChangeAttachmentType
		//!	to actually resolve the attachments.
		//! If the table is already OneToOne or Identity it will return that.
		//! It will never return suggest Identity unless the table is already 
		//!	Identity.
		ATTACHMENT ComputeMinimumAttachmentType (
			ELEMTYPE ElemType = ELEMTYPE_Empty
			) const;

		//! Copy the direct record attachments from one table to another
		ERRVALUE CopyRecordAttachmentsTo (
			RVC::DBTABLE& DestTable,
			const SIMPLE_ARRAY<INT32>& SrcToDestTrans
			) const;

		//! Copy a table to another database
		//! \param DestDatabase The database to copy to
		//! \param pDestDescriptor The new table's descriptor.  If not specified,
		//!		the new table will get the same descriptor as the source table.
		//! \param pRecordTrans Optional SIMPLE_ARRAY in which will be returned
		//!		the translation from source record numbers to destination record
		//!		numbers.  This is only an issue if the source table has deleted
		//!		records, or if the Filter is used to only copy a portion of the
		//!		table.
		//! \param pNewTableNumber Pointer to INT32 into which to return the new
		//!		table number.
		//! \param Filter Delegate to call to determine if a record should be
		//!		copied.  Example:
		//!			DELEGATE_BOOL_UINT32(&bits, &BITSET::GetBit)
		//! \param Credentials The credentials to use when creating the table
		//! \param flags Supported copy flags:
		//!		RVC::DBASE::COPYFLAG_RemoveStdAttribTables -- will silently refuse to copy the table if it's a standard attribute table
		//!		RVC::DBASE::COPYFLAG_TableLinks -- If the table is a link to an external table, it will just make a new link in the destination database
		//!		RVC::DBASE::COPYFLAG_MinimizeTableSize
		//!		RVC::DBASE::COPYFLAG_FreezeComputedFields -- Any computed fields will be computed and frozen (no longer computed).
		ERRVALUE CopyTo (
			RVC::DBASE& DestDatabase,
			RVC::DESCRIPTOR *pDestDescriptor = 0, //!< New table's descriptor (will use source table's if not specified)
			INT32* pNewTableNumber = 0,
			SIMPLE_ARRAY<INT32>* pRecordTrans = 0,	
			DELEGATE_BOOL_UINT32 Filter = 0,	
			const RVC::CREDENTIALS *pCredentials = 0,
			RVC::DBASE::COPYFLAG flags = RVC::DBASE::COPYFLAG_None
			) const;

		//! Drop a table (delete)
		ERRVALUE Drop (
			bool CleanDBList						//!< Remove the entries in the DBlist that refer to this table
			);

		//! Delete a record from the table using the record number
		ERRVALUE DeleteRecord (
			INT32 RecordNum
			);

		//! Delete a record from the table using the cursor and a bookmark
		ERRVALUE DeleteRecord (
			CURSOR& cursor,
			const BOOKMARK& bookmark
			);

	#ifndef NO_DEPRECATED
		//! Find the field corresponding to the name
		//! @return Field number of >= 0, R_FieldNotFound if field does not exist, < 0 Error code
		//! \deprecated Use the MISTRING version or FindFieldByShortName(), because field names can be bigger than 16 characters now
		DEPRECATED INT32 FindField (
			const RVC::OBJECTNAME& name
			) const;
	#endif
			
		//! Find the field corresponding to the name
		//! @return Field number of >= 0, R_FieldNotFound if field does not exist, < 0 Error code
		INT32 FindField (
			const MISTRING& name
			) const;
			
		//! Find the field cooresponding to the name
		//! @return Field number of >= 0, R_FieldNotFound if field does not exist, < 0 Error code
		INT32 FindFieldByShortName (
			const RVC::OBJECTNAME& name
			) const;
			
		//! Find a field with a specific usage.
		//! Returns -1 if the table does not have a field with the specified 
		//! usage.  If the table has mutliple fields of the specified usage, it
		//! returns the first one.
		INT32 FindFieldByUsage (
			RVC::DBTABLE::FIELDINFO::USAGE Usage
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			double value,								//!< Buffer containing value to compare to
			SIMPLE_ARRAY<INT32>& FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const char* string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32>& FoundRecList,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const MIUNICODE* string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32>& FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const MISTRING& string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32>& FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const RECORD& record,					//!< Record containing stuff to compare to
			SIMPLE_ARRAY<INT32>& FoundRecList,
			INT32 RecordFieldNum = -1				//!< Record field number to use, default is 'FieldNum'
			) const;
			
		//! Get the records attached to the element for this table
		//! @return Number of records attached or <0 if error 
		INT32 GetAttachedRecords (
			INT32 ElementNum,
			ELEMTYPE ElemType,
			SIMPLE_ARRAY<INT32>& RecordList,
			FINDFLAG FindFlags = FINDFLAG_AllAttached,
			DBTABLE* SourceTable = 0	//!< Source table if ElemType == ELEMTYPE_Record
			) const;

		//! Get table attachment type
		//! @return Table attachment type
		ATTACHMENT GetAttachmentType (
			) const;

		//! Get table capability
		//! @return Table capability
		bool HasCapability (
			CAPABILITY capability
			) const;

		//! Get the computed field query for a field
		ERRVALUE GetComputedFieldQuery (
			INT32 FieldNum, 
			MISTRING& QueryStr
			) const;

		//! Get table encoding
		//! @return Table encoding
		CHAR_ENCODING GetEncoding (
			) const;
			
		//! Get the index for a field, will build one if needed
		ERRVALUE GetFieldIndex (
			INT32 FieldNum,
			SIMPLE_ARRAY<INT32>& Index, 				//!< List of record numbers in sorted order
			BITSET& DuplicateBits						//!< Bit set to true if value in the record for that field is equal to the previous record
			);

		//! Get the RVC::DESCRIPTOR for a single field
		ERRVALUE GetFieldDescriptor (
			INT32 FieldNum,
			RVC::DESCRIPTOR& Descriptor
			) const;

		//! Get the FIELDINFO for a single field
		ERRVALUE GetFieldInfo (
			INT32 FieldNum,
			FIELDINFO& FieldInfo
			) const;

		//! Get the FIELDINFO's for the table
		ERRVALUE GetFieldInfo (
			FIELDINFOARRAY& FieldArray
			) const;

		//! Get the list of fields linked to the starting field		
		ERRVALUE GetLinkFields (
			INT32 StartFieldNum,								//!< Field to start checking on
			SIMPLE_ARRAY<INT32>& FieldNumArray			//!< One item 'StartFieldNum' if no next_key links, > 1 items if next_key links exists
			) const;		 

		//! Get the link type of this table
		LINKTYPE GetLinkType (
			) const;

		//! Get the number of deleted records in the table
		INT32 GetNumDeletedRecords (
			) const;

		//! Get the number of field in the table
		//! @return Number of fields
		INT32 GetNumFields (
			) const;

		//! Get the number of records in the table
		//! More accurately, this method returns the maximum record number + 1.
		//! If the table has any deleted records, they are not subtracted from
		//! the count.
		//! @return Number of records
		INT32 GetNumRecords (
			) const;

		//! Get reference element (This table belongs to what element?)
		//! @return Reference flags
		REFELEM GetReferenceElement (
			) const;
			
		//! Get the records from this table attached to the record in the basetable
		//! @return Number of records attached or <0 if error 
		INT32 GetRelatedRecords (
			INT32 RecordNum,												//!< Record in the base table to get attachments for
			INT32 BaseTableNum,											//!< Basetable that contains the record stated above
			SIMPLE_ARRAY<INT32>& RecordList,							//!< List of records that are related RETURNED
			FINDFLAG FindFlags = FINDFLAG_AllAttached				//!< find flags
			) const;

		//! Get the records from this table attached to the given record
		//! @return Number of records attached or <0 if error 
		INT32 GetRelatedRecords (
			const RECORD& record,										//!< Record to get attachments for
			SIMPLE_ARRAY<INT32>& RecordList,							//!< List of records that are related RETURNED
			FINDFLAG FindFlags = FINDFLAG_AllAttached				//!< find flags
			) const;

		//! Get the metadata for the table.
		//! Note that this method differs from RVC::OBJECT::GetMetadata in that
		//! this one will make sure that the returned metadata includes 
		//! descriptors for all the fields, even if the actual object doesn't
		//! have metadata for them.
		ERRVALUE GetTableMetadata (
			SIMPLE_ARRAY<UINT8>& buffer
			) const;

		//! Get table number
		//! @return Table number
		INT32 GetTableNumber (
			) const;
			
		//! Get table usage
		//! @return Table usage
		USAGE GetUsage (
			) const;
			
		//! Get the default view mode
		VIEWMODE GetViewMode (
			) const;

		//! Get view position of table
		//! @return Table view position
		LRECT2D GetWindowRect (
			) const;
			
		//! Is the table hidden?
		//! @return 'True' if the table is hidden
		bool IsHidden (
			) const;

		//! Is table an internal (no storage representation) table
		//! @return 'True' if the table is internal
		bool IsInternal (
			) const;

		//! Is table missing.  Occurs with linked tables
		//! @return 'True' if the table is missing
		bool IsMissing (
			) const;
			
		//! Is table a prompt table.  Used for adding elements
		//! @return 'True' if the table is a prompt table
		bool IsPrompt (
			) const;

		//! Is table a UserDefined table.  (Currently USAGE_UserDefined||USAGE_Sketch||USAGE_Shape)
		//! @return 'True' if the table is a user defined table
		bool IsUserDefined (
			) const;

		//! Is table valid.  Occurs with deleted tables
		//! @return 'True' if the table is valid
		bool IsValid (
			) const;

		//! Can the table be written to?
		//! @return 'True' if the table is writable
		bool IsWritable (
			) const;

		//! Link to an external table, field info returned in the LINKPARMS
		ERRVALUE Link (
			RVC::DBASE& parent,
			const DESCRIPTOR& Descriptor,
			const LINKPARMS& LinkParms,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a table
		ERRVALUE Make (
			RVC::DBASE& parent,
			const DESCRIPTOR& Descriptor,
			MAKEPARMS& MakeParms,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new table with one field listing all distinct values of
		//! a specific field.
		ERRVALUE MakeDistinctTableFromField (
			RVC::DBASE& parent,
			const RVC::DBTABLE& SourceTable,
			INT32 Field,					//! Field number from source table
			const DESCRIPTOR& Descriptor,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Open an existing table using the table name
		ERRVALUE Open (
			const RVC::DBASE& parent,
			const MISTRING& TableName,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an existing table using the table name
		ERRVALUE Open (
			const RVC::DBASE& parent,
			const RVC::OBJECTNAME& ShortName,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an existing table using the table index
		ERRVALUE Open (
			const RVC::DBASE& parent,
			INT32 TableNum,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open a table using another instance of that table
		ERRVALUE Open (
			const DBTABLE& twin,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::DBTABLE instance using a RVC::OBJITEM instance
		//!
		//! This method will attach and open the file and database table object if possible
		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem,			//!< This contains the file and rvc object location information
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an <b>attached</b> RVC::DBTABLE instance.
		//! The instance must be attached for this to work.
		ERRVALUE Open (
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Open an existing table
		// This is a private version used by SML, which really already has the
		// table open, but since RVC interface classes aren't available inside
		// RVCSYS, it only has internal handles.  
		ERRVALUE Open (
			const SML::TABLEDATA* TableData,
			OPENMODE mode = OPENMODE_Read
			// doesn't need credentials table really already open.
			);
	#endif  // doxygen

		//! Read a record from the table using a bookmark (relative to the default bookmark cursor) into a pre-allocated buffer
		ERRVALUE Read (
			const BOOKMARK& Bookmark, 
			RECORD& Record
			) const {return (Read(0, 0, &Bookmark, Record, RECACC_Bookmark));}

		//! Read a record from the table using a record number into a pre-allocated buffer
		ERRVALUE Read (
			INT32 RecordNum, 
			RECORD& Record
			) const {return (Read(0, RecordNum, 0, Record, RECACC_RecordNum));}

		//! Read a record from the table using a cursor into a pre-allocated buffer
		ERRVALUE Read (
			CURSOR& cursor,
			RECORD& Record
			) const {return (Read(&cursor, 0, 0, Record, RECACC_Cursor));}
			
		//! Read a record from the table using a cursor / bookmark pair into a pre-allocated buffer
		ERRVALUE Read (
			CURSOR& cursor,
			const BOOKMARK& bookmark,
			RECORD& Record
			) const {return (Read(&cursor, 0, &bookmark, Record, RECACC_Bookmark));}
			
		//! Read a record from the table using a record number into a pre-allocated buffer
		ERRVALUE ReadDefault (
			RECORD& Record
			) const {return (Read(0, RECORD_New, 0, Record, RECACC_RecordNum));}

		ERRVALUE ReadConstraint (
			std::vector<CONSTRAINT>& ConstraintList
			) const;

		INT32 ReadInverseDBList (
			INT32 RecNum,
			SIMPLE_ARRAY<DBLISTELMT>& List
			);

		//! Get a constant iterator to the first viable record in the table
		void RecordBegin (
			CONST_RECORDITERATOR& Iterator, 
			ITERATERECORD IterateRecord = ITERATERECORD_SkipDeleted,
			INT32 FieldNum = 0
			) const;

		//! Get an iterator to the first viable record in the table
		void RecordBegin (
			RECORDITERATOR& Iterator, 
			ITERATERECORD IterateRecord = ITERATERECORD_SkipDeleted,
			INT32 FieldNum = 0
			);

		//! Remove duplicate records from the table
		ERRVALUE RemoveDuplicateRecords (
			);

		//! Remove unattached records from the table
		ERRVALUE RemoveUnattachedRecords (
			);

		//! Set the number of records in the table.  Does not work with ATTACHMENT_Identity tables
		ERRVALUE Resize (
			INT32 NewNumRecords,
			bool CleanDBList
			);

		//! Search field index with a value for the record match
		//! @return Number of matches or error < 0
		template <class _CT>
		INT32 SearchIndex (
			INT32 FieldNum,
			const _CT& item,
			const FIELDINFO& finfo, 
			INT32& offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII	//!< Encoding to use (ignored except for non-unicode string fields)
			) const {return (SearchIndex(FieldNum, &item, &finfo, &offset, flag));}

		//! Search field index with a value that matches the field to search for the record match
		//! @return Number of matches or error < 0
		template <class _CT>
		INT32 SearchIndex (
			INT32 FieldNum,
			const _CT& item,				//!< MUST MATCH 'FieldNum' field type and size
			INT32& offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII	//!< Encoding to use (ignored except for non-unicode string fields)
			) const {return (SearchIndex(FieldNum, &item, 0, &offset, flag, Encoding));}

		//! Search field index with a double value for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			double value,
			INT32& offset, 
			INDEXSEARCH flag
			) const;
			
		//! Search field index with a string value for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			const MISTRING& value,
			INT32& offset, 
			INDEXSEARCH flag
			) const;
			
		//! Search field index with a value in the RECORD for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			const RECORD& Record, 
			INT32 RecFieldNum,
			INT32& offset, 
			INDEXSEARCH flag
			) const;
		
		//! Set table attachment type
		//! Note: This just blindly sets the attachment type without resolving
		//! anything.  Call ChangeAttachmentType if you need to resolve existing
		//! relationships and attachments
		ERRVALUE SetAttachmentType (
			ATTACHMENT Attachment
			);

		//! Set the computed field query for a field
		ERRVALUE SetComputedFieldQuery (
			INT32 FieldNum,
			const MISTRING& QueryStr
			);
			
		//! Set table encoding
		void SetEncoding (
			CHAR_ENCODING Encoding
			);

		//! Set the RVC::DESCRIPTOR for a single field
		//! Note, there is no field number.  The ShortName of the descriptor
		//! should match the short name of an existing field
		ERRVALUE SetFieldDescriptor (
			const RVC::DESCRIPTOR& Descriptor
			);

		//! Modify specific table FIELDINFO
		ERRVALUE SetFieldInfo (
			INT32 FieldNum,
			const FIELDINFO& FieldInfo
			);

		//! Modify all table FIELDINFO's
		ERRVALUE SetFieldInfo (
			FIELDINFOARRAY& FieldArray
			);

		//! Modify all table FIELDINFO's
		ERRVALUE SetFieldInfo (
			FIELDINFOARRAY& FieldArray,
			const SIMPLE_ARRAY<int>& NewToOld
			);

		//! Set whether table is "hidden".
		ERRVALUE SetHidden (
			bool hidden
			);

		//! Set whether or not this table should "Prompt when adding elements"
		ERRVALUE SetPromptOnAddElement (
			bool prompt = true
			);

		//! Set the reference element type
		ERRVALUE SetReferenceElement (
			REFELEM RefElem
			);

		//! Set the "read only" bit in the table header
		//! (will go away in favor of ACL)
		ERRVALUE SetReadOnly (
			bool bReadOnly = true
			);

		//! Set table usage
		ERRVALUE SetUsage (
			DBTABLE::USAGE usage
			);

		//! Set the default view mode of the table
		ERRVALUE SetViewMode (
			VIEWMODE ViewMode
			);

		//! Set view position of table
		void SetWindowRect (
			const LRECT2D& rect
			);

		//! If a table is an ODBC link to Access or Excel, this method will
		//! convert it to an OLE-DB link.  WARNING! Doing so will render the
		//! link unusable in TNT 2005:71 and earlier.
		ERRVALUE UpgradeLinkToOLEDB (
			);

		//! Determine if the current user has permission to access this table
		//! \return 0 If user has access, error code < 0 if not
		ERRVALUE ValidateAccess (
			PERMISSIONS Permission,
			bool bGenerateErrorMsg = true	//!< Pass false to not SetErrPosn on error
			) const;

		//! Determine if the current user has permission to a given field in this table.
		//! \return 0 If user has access, error code < 0 if not
		ERRVALUE ValidateAccess (
			INT32 FieldNum,
			PERMISSIONS Permission,
			bool bGenerateErrorMsg = true	//!< Pass false to not SetErrPosn on error
			) const;

		//! Make sure that a table and field names are valid and unique
		//! This version assumes the table already exists.  There is also a 
		//! ValidateTableName method on DBASE for use when creating a new table
		ERRVALUE ValidateTableName (
			OBJECTNAME& NewTableName,
			SIMPLE_ARRAY<OBJECTNAME>& NewFieldNames
			);

		//! Write a record in the table using a record number from a buffer
		ERRVALUE Write (
			INT32 RecordNum, 
			RECORD& Record
			) {return (Write(0, RecordNum, 0, Record, RECACC_RecordNum));}

		//! Write a record to the table using a bookmark (relative to the default bookmark cursor) 
		ERRVALUE Write (
			const BOOKMARK& Bookmark, 
			RECORD& Record
			) {return (Write(0, 0, &Bookmark, Record, RECACC_Bookmark));}

		//! Write a record in the table using a cursor from a buffer
		ERRVALUE Write (
			CURSOR& cursor,
			RECORD& Record
			) {return (Write(&cursor,0, 0, Record, RECACC_Cursor));}

		//! Write a record in the table using a cursor / bookmark pair from a buffer
		ERRVALUE Write (
			CURSOR& cursor,
			const BOOKMARK& bookmark,
			RECORD& Record
			) {return (Write(&cursor, 0, &bookmark, Record, RECACC_Bookmark));}

		ERRVALUE WriteConstraint (
			const std::vector<CONSTRAINT>& ConstraintList
			);

	protected:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::DBASE m_ParentObj;	// Not currently used (8-Nov-05) but can't remove without requiring full rebuild.  Do at the end of the day.
		DATABASEHANDLE* m_DBHandle;
		OBJITEM m_DBObjItem;
		int m_TableID;		// Actually the table number
		
		DATABASEHANDLE* GetDBHandle() const {return (m_DBHandle);}

		virtual bool v_IsValidTable (const RVC::DBTABLE& table);
		virtual bool v_IsObjectOpen () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class TABLELIST;

		enum RECACC {
			RECACC_RecordNum,
			RECACC_Cursor,
			RECACC_Bookmark
			};

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual ERRVALUE v_GetDescriptor (RVC::DESCRIPTOR& Descriptor, const CREDENTIALS *pCredentials) const;
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual ERRVALUE v_GetParent (RVC::OBJECT& ObjParent) const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS *pCredentials);

		ERRVALUE CopyRecordAttachmentsLow (
			const RVC::DBTABLE* SrcTable1,
			TABLEJOININFO* TableJoinInfo1,
			const RVC::DBTABLE* SrcTable2,
			TABLEJOININFO* TableJoinInfo2,
			DBASE::JOINFLAGS
			);


		INT32 GetRecordSize () const;

		// Joins "this" table to another database
		// can return number of records?
		INT32 AppendTo (
			RVC::DBASE& DestDBase,
			const BITSET& recbit,
			RVC::DBTRANSINFO& TransInfo,
			RVC::DBTABLE::TABLELIST* TableEntry,
			INT32& rNextElemID,
			RVC::DBASE::JOINFLAGS flags
			);

		static INT32 TestIsJoinable (
			const DBTABLE& src,
			const DBTABLE& dest,
			TABLELIST* TableEntry,
			RVC::DBASE::COPYFLAG CopyFlags,
			RVC::DBASE::JOINFLAGS JoinFlags
			);
			
		ERRVALUE Read (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			RECORD& Record,
			RECACC flags
			) const;

		INT32 SearchIndex (
			INT32 FieldNum,
			void* buf,
			const FIELDINFO* finfo,
			INT32& offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING = CHAR_ENCODING_ASCII
			) const;

		static void TableCallbackFunc (DBCALLBACKSTRUCT *cb, void *data);

		ERRVALUE Write (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			RECORD& Record,
			RECACC flags
			);

		ERRVALUE WriteBinary (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			INT32 FieldNum,
			const SIMPLE_ARRAY<UINT8>& binary,
			RECACC flags
			);

		ERRVALUE WriteMemo (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			INT32 FieldNum,
			const MISTRING& string,
			RECACC flags
			);

		friend class CURSOR;
		friend class DBASE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};


//! Class to represent database bookmarks used in ODBC
class DBTABLE::BOOKMARK {
	public:
		
		//! Default constructor
		explicit BOOKMARK (
			UINT32 Bookmark = 0
			) : m_Bookmark(Bookmark) {}
	
		//! Copy constructor
		BOOKMARK (
			const BOOKMARK& rhs
			) : m_Bookmark(rhs.m_Bookmark) {}

		//! Assignment
		BOOKMARK& operator= (
			const BOOKMARK& rhs
			) {
			if (this != &rhs) m_Bookmark = rhs.m_Bookmark;
			return (*this);
			}
	
		bool operator== (
			const BOOKMARK& rhs
			) const {return (m_Bookmark == rhs.m_Bookmark);}
		
		UINT32 GetValue (
			) const { return m_Bookmark; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_Bookmark;
	
		friend class DBTABLE;
		friend class DBTABLE::CURSOR;
		friend class DBTABLE::RECORD;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Class to represent database table cursors used in ODBC
class DBTABLE::CURSOR {
	public:
		enum FUNCTION {
			FUNCTION_None =	 		0x00000000,
			FUNCTION_Unique = 		0x00000001,
			FUNCTION_Sorted = 		0x00000002,
			FUNCTION_Equal =			0x00000004,
			FUNCTION_GreaterThan =	0x00000008,
			FUNCTION_LessThan =		0x00000010,
			FUNCTION_NotEqual =		0x00000020,
			FUNCTION_AND =				0x00000040,
			FUNCTION_OR =				0x00000080,
			FUNCTION_NoClear =		0x00000100,	//!< Add to previous function
			FUNCTION_ForUpdate =		0x00000200	//!< To allow writes
			};
		
		//! Flags for GetIndex
		enum INDEXFLAG {
			INDEXFLAG_None = 0,
			INDEXFLAG_IndexOffset = 0x00000001, //<! Give index offset indstead of record number
			INDEXFLAG_LeaveRunBit = 0x00000002 //<! Leave the high bit set on the record number if the record is the first in a run of same-value records
			};

		//! Default constructor
		explicit CURSOR (
			const RVC::DBTABLE& table
			);
			
		//! Destructor
		~CURSOR (
			);
			
		//! Equality
		bool operator== (
			const CURSOR& rhs
			) const;
			
		//! Get the current bookmark		
		ERRVALUE GetBookmark (
			BOOKMARK& Bookmark
			) const;
			
		//! Get the offset into the rowset
		//! Note that INDEXFLAG_LeaveRunBit is meaningless for ODBC and OLE DB
		ERRVALUE GetIndex (
			INT32& Index,
			INDEXFLAG flags = INDEXFLAG_None
			) const;

	#ifndef NO_DEPRECATED
		//! Get the current record number or < 0 if error
		//! \deprecated Use GetBookmark instead.  
		//! This method will NOT work for ODBC or RDBMS.
		DEPRECATED INT32 GetRecordNum (
			) const;
	#endif
			
		//! Is the cursor "done"
		//! Note: this will return true right after a Rewind() until you call
		//! Next().
		bool IsDone (
			) const { return (!IsValid()); }

		//! Is the cursor valid
		//! @return True for a valid cursor
		bool IsValid (
			) const;

		//! Increment cursor
		//! @return > 0 if still valid, 0 if not, < 0 error
		//! \code
		//! RVC::DBTABLE::CURSOR Cursor(Table);
		//! while (Cursor.Next() > 0) {
		//!    ...
		//!    }
		//! \endcode
		int Next (
			);
			
		//! Reset cursor to the beginning
		//! Note: After calling Rewind, the cursor is actually pointing
		//! one record before the beginning.  It is intended that you should
		//! use it like this...
		//! \code
		//! RVC::DBTABLE::CURSOR Cursor(Table);
		//!   ...
		//! Cursor.Rewind();	// only necessary if you've already used Cursor
		//! while (Cursor.Next() > 0) {
		//!    ...
		//!    }
		//! \endcode
		ERRVALUE Rewind (
			);

		//! Seek to a spot in the table.
		//! \return 1 if value found, 0 if not found but seek to closest match
		//! This method will NOT work for ODBC or RDBMS.
		INT32 SeekValue (
			INT32 FieldNum,
			const void* vbuf,
			const FIELDINFO* finfo,
			bool bSingleField = true
			);

		//! Seek to a spot in the table.
		//! \return 1 if value found, 0 if not found but seek to closest match
		//! This method will NOT work for ODBC or RDBMS.
		INT32 SeekValue (
			INT32 FieldNum,
			const MISTRING& value
			);

		//! Set cursor behavior
		//! Note: the only functions that will work with this method are Sort
		//! and Unique.  Sort has its own function which is better.
		ERRVALUE SetFunction (
			INT32 FieldNum,
			FUNCTION Function
			);
	
		//! Set cursor behavior
		ERRVALUE SetSort (
			const SIMPLE_ARRAY<INT32>& FieldList	//!< fields to sort on
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const DBTABLE& m_Table;

		class PRIV;

		PRIV* m_Cursor;	

		CURSOR (const CURSOR& rhs);
		CURSOR& operator=(const CURSOR& rhs);
		
		friend class DBTABLE;
		friend class DBTABLE::RECORD;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

//! Database table object selection filter that excludes all objects except for tables.
class DBTABLE::GETOBJFILTER_ISSELECTABLE : public RVC::GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE() : GETOBJFILTER() { }
		virtual ~GETOBJFILTER_ISSELECTABLE() { }

	private:
		virtual bool v_IsSelectable (
			const RVC::OBJITEM& objitem,
			int position
			);

		virtual bool v_HasSelectableSubObjects (
			const RVC::OBJITEM& objitem
			);

		virtual bool v_IsValidParent (
			const RVC::OBJITEM& objitem
			);
	};

//! Database table object selection filter to determine if the table is a user defined table (it may be empty)
class DBTABLE::GETOBJFILTER_ISUSAGEVALID : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISUSAGEVALID (
			const RVC::OBJITEM& DBaseObjItem,
			const USAGESET& UsageSet
			);
			
		virtual ~GETOBJFILTER_ISUSAGEVALID () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const RVC::OBJITEM& m_DBaseObjItem;
		RVC::DBASE m_DBaseObj;
		const USAGESET m_UsageSet;
		
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Database table object selection filter to determine if the table is a user defined table (it may be empty)
class DBTABLE::GETOBJFILTER_ISUSERDEFINED : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISUSERDEFINED (
			const RVC::OBJITEM& DBaseObjItem
			);
			
		virtual ~GETOBJFILTER_ISUSERDEFINED () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GETOBJFILTER_ISUSAGEVALID m_UsageValidFilter;
		
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Parameters to link to external database tables.
class DBTABLE::LINKPARMS {
	public:

		//! Link flags.
		enum LINKFLAG {
			LINKFLAG_None = 				0,
			LINKFLAG_KeepPassword = 	0x00800000
			};

		//! Construct with FILEPATH.
		LINKPARMS (
			const FILEPATH& filepath,					//!< Local file path
			DBTABLE::LINKTYPE LinkType,				//!< Link type
			CHAR_ENCODING Encoding,						//!< External table character encoding
			LINKFLAG LinkFlag = LINKFLAG_None,		//!< Flags if needed
			char Delimiter = ','							//!< Field delimiter for CSV and similar
			);

		//! Construct with link string.
		LINKPARMS (
			const MISTRING& LinkString,				//!< Link string, for RDBMS must include separate 'TABLE=' to identify table being linked to
			DBTABLE::LINKTYPE LinkType,				//!< Link type
			CHAR_ENCODING Encoding,						//!< External table character encoding
			LINKFLAG LinkFlag = LINKFLAG_None,		//!< Flags if needed
			char Delimiter = ','							//!< Field delimiter for CSV and similar
			);
	
		//! Destructor
		~LINKPARMS ();

		char GetDelimiter () const
			{ return (m_Delimiter); }

		//! Get the encoding of the external table.
		CHAR_ENCODING GetEncoding () const
			{ return (m_Encoding); }

		LINKFLAG GetLinkFlag () const
			{ return (m_LinkFlag); }

		const MISTRING& GetLinkString () const
			{ return (m_LinkString); }

		//! Get the type of external link
		//! @return External link type
		DBTABLE::LINKTYPE GetLinkType () const
			{ return (m_LinkType); }

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_LinkString;
		DBTABLE::LINKTYPE m_LinkType;
		CHAR_ENCODING m_Encoding;
		LINKFLAG m_LinkFlag;
		char m_Delimiter;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
//	DBTABLE::MAKEPARMS and derived classes

//! Database table creation parameters class.  Manages the necessary parameters to create a database table
class DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARMS (
			const DBTABLE::FIELDINFOARRAY& FieldInfoArray,
			DBTABLE::ATTACHMENT Attachment = ATTACHMENT_RelatedOnly,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII,
			DBTABLE::USAGE Usage = USAGE_UserDefined
			) :
			m_FieldInfoArray(FieldInfoArray),
			m_Attachment(Attachment),
			m_Encoding(Encoding),
			m_Usage(Usage),
			m_RefElem(REFELEM_All),
			m_LinkObject(0),
			m_LinkType(LINKTYPE_Internal),
			m_Delimiter(0),
			m_bPromptTable(false),
			m_UseFieldNamesAsHeading(false)
			{}
			
		//! Constructor
		MAKEPARMS (
			DBTABLE::ATTACHMENT Attachment = ATTACHMENT_RelatedOnly,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII,
			DBTABLE::USAGE Usage = USAGE_UserDefined
			) :
			m_Attachment(Attachment),
			m_Encoding(Encoding),
			m_Usage(Usage),
			m_RefElem(REFELEM_All),
			m_LinkObject(0),
			m_LinkType(LINKTYPE_Internal),
			m_Delimiter(0),
			m_bPromptTable(false),
			m_UseFieldNamesAsHeading(false)
			{}
			
		//! Template Constructor
		MAKEPARMS (
			const DBTABLE& Template
			);
			
		virtual ~MAKEPARMS (){}

		//! Get attachment type
		//! @return Table attachment type
		DBTABLE::ATTACHMENT GetAttachmentType (
			) const {return (m_Attachment);}

		//! Get the delimiter for CSV tables
		//! if not set, ',' is assumed
		char GetDelimiter (
			) const { return (m_Delimiter); }

		//! Get encoding
		//! @return Table encoding
		CHAR_ENCODING GetEncoding (
			) const {return (m_Encoding);}

		//! Get field info array
		//! @return Table field info array
		const DBTABLE::FIELDINFOARRAY& GetFieldInfo (
			) const {return (m_FieldInfoArray);}

		//! Get table link file
		//! @return Link table file to create
		const MISTRING& GetLinkFile (
			) const {return (m_NewLinkFile);}
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Get object to create a link under the table			
		const RVC::OBJECT* GetLinkObject (
			) const {return (m_LinkObject);}
		#endif // GENERATING_DOXYGEN_OUTPUT
			
		//! Get table link type
		//! @return Table link type
		DBTABLE::LINKTYPE GetLinkType (
			) const {return (m_LinkType);}
			
		//! Get whether or not this table should "Prompt when adding elements"
		bool GetPromptOnAddElement (
			) const { return m_bPromptTable; }

		//! Get reference element (This table belongs to what element?)
		//! @return Reference flags
		DBTABLE::REFELEM GetReferenceElement (
			) const {return (m_RefElem);}
			
		//! Get table usage
		//! @return Table usage
		DBTABLE::USAGE GetUsage (
			) const {return (m_Usage);}

		//! Get whether to use field names as heading where supported.
		bool GetUseFieldNamesAsHeading () const
			{ return (m_UseFieldNamesAsHeading); }

		//! Set table attachment type
		void SetAttachmentType (
			ATTACHMENT Attachment
			) {m_Attachment = ((Attachment == ATTACHMENT_Unknown) ? ATTACHMENT_NoRestrictions : Attachment);}

		//! Set the delimiter
		//! Only used for CSV tables and only necessary if changing it from
		//! something other than ','
		void SetDelimiter (
			char delim
			) { m_Delimiter = delim; }

		//! Set table encoding
		void SetEncoding (
			CHAR_ENCODING Encoding
			) {m_Encoding = Encoding;}

		//! Set table field info array
		void SetFieldInfo (
			const FIELDINFOARRAY& FieldInfoArray
			) {m_FieldInfoArray = FieldInfoArray;}

		//! Set table link file, see SetLinkType()
		void SetLinkFile (
			const MISTRING& filename
			) {m_NewLinkFile = filename;}
			
		//! Set table link type, this will create a new table of this link type			
		void SetLinkType (
			LINKTYPE LinkType
			) {m_LinkType = LinkType;}
			
		//! Set whether or not this table should "Prompt when adding elements"
		void SetPromptOnAddElement (
			bool prompt = true
			) { m_bPromptTable = prompt; }

		//! Set table reference elements
		void SetReferenceElement (
			REFELEM RefElem
			) {m_RefElem = RefElem;}
			
		//! Set table usage
		void SetUsage (
			USAGE Usage
			) {m_Usage = Usage;}

		//! Set whether to use field names as heading where supported.
		void SetUseFieldNamesAsHeading (
			bool UseFieldNamesAsHeading
			) { m_UseFieldNamesAsHeading = UseFieldNamesAsHeading; }
			
	protected:

		DBTABLE::FIELDINFOARRAY& GetFieldArray (
			) {return (m_FieldInfoArray);}
			
		void SetLinkObject (
			const OBJECT& object
			) {m_LinkObject = &object;}
			
		void SetupRelatedFields (
			const DBTABLE& RelTable,
			INT32 RelFieldNum,
			FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			INT32 FirstFieldNum,
			INT32 NumStaticFields,
			bool UseRelatedSingle
			);
			
	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FIELDINFOARRAY m_FieldInfoArray;
		DBTABLE::ATTACHMENT m_Attachment;
		CHAR_ENCODING m_Encoding;
		DBTABLE::USAGE m_Usage;
		DBTABLE::REFELEM m_RefElem;
		const OBJECT* m_LinkObject;
		DBTABLE::LINKTYPE m_LinkType;
		MISTRING m_NewLinkFile;
		char m_Delimiter;	
		bool m_bPromptTable;
		bool m_UseFieldNamesAsHeading;

		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Element ID tables
class DBTABLE::MAKEPARM_ID_ELEMENT : public RVC::DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_ID_ELEMENT ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Point ID tables
class DBTABLE::MAKEPARM_ID_POINT : public RVC::DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_ID_POINT ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Element Style tables
class DBTABLE::MAKEPARM_STYLE_ELEMENT : public RVC::DBTABLE::MAKEPARMS {
	public:

		//! Setup static fields in array based on usage
		static void SetupStaticFields (
			FIELDINFOARRAY& FieldInfoArray,				//!< Field array to populate, will be resized if needed
			USAGE Usage											//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle, USAGE_CADStyle
			);

		//! Constructor
		MAKEPARM_STYLE_ELEMENT (
			const RVC::DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,	//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle
			bool UseRelatedSingle
			);

		//! Constructor with specified style object for link.
		MAKEPARM_STYLE_ELEMENT (
			const RVC::DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,	//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			const RVC::STYLE& StyleObj,					//!< Style object to link table to
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle
			bool UseRelatedSingle
			);

		//! Set style object to link to.
		void SetStyleLink (
			const RVC::STYLE& StyleObj						//!< Style object to link table to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		void Setup (
			const RVC::DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,	//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle
			bool UseRelatedSingle
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for multi-Style tables
class DBTABLE::MAKEPARM_STYLE_CAD : public RVC::DBTABLE::MAKEPARM_STYLE_ELEMENT {
	public:
		//! Constructor
		MAKEPARM_STYLE_CAD (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			const RVC::STYLE& StyleObj,
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, StyleObj, USAGE_MultiStyle, UseRelatedSingle) {}
		//! Constructor
		MAKEPARM_STYLE_CAD (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, USAGE_MultiStyle, UseRelatedSingle) {}
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Line Style tables
class DBTABLE::MAKEPARM_STYLE_LINE : public RVC::DBTABLE::MAKEPARM_STYLE_ELEMENT {
	public:
		//! Constructor
		MAKEPARM_STYLE_LINE (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			const RVC::STYLE& StyleObj,
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, StyleObj, USAGE_LineStyle, UseRelatedSingle) {}
		//! Constructor
		MAKEPARM_STYLE_LINE (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, USAGE_LineStyle, UseRelatedSingle) {}
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Point Style tables
class DBTABLE::MAKEPARM_STYLE_POINT : public RVC::DBTABLE::MAKEPARM_STYLE_ELEMENT {
	public:
		//! Constructor
		MAKEPARM_STYLE_POINT (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			const RVC::STYLE& StyleObj,
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, StyleObj, USAGE_PointStyle, UseRelatedSingle) {}
		//! Constructor
		MAKEPARM_STYLE_POINT (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, USAGE_PointStyle, UseRelatedSingle) {}
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Polygon Style tables
class DBTABLE::MAKEPARM_STYLE_POLYGON : public RVC::DBTABLE::MAKEPARM_STYLE_ELEMENT {
	public:
		//! Constructor
		MAKEPARM_STYLE_POLYGON (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			const RVC::STYLE& StyleObj,
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, StyleObj, USAGE_PolyStyle, UseRelatedSingle) {}
		//! Constructor
		MAKEPARM_STYLE_POLYGON (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			) : MAKEPARM_STYLE_ELEMENT(RelTable, RelFieldNum, RelFieldFlags, USAGE_PolyStyle, UseRelatedSingle) {}
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Scale Select tables
class DBTABLE::MAKEPARM_SCALESELECT : public RVC::DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_SCALESELECT (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Attribute Select tables
class DBTABLE::MAKEPARM_SELECTION : public RVC::DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_SELECTION (
			const RVC::DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBTABLE::FIELDINFO::FLAGS RelFieldFlags,				//!< FIELDINFO::FLAG_RangeAbove, FIELDINFO::FLAG_RangeBelow
			bool UseRelatedSingle
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM for legend element table.
class DBTABLE::MAKEPARM_LEGENDELEMENT : public RVC::DBTABLE::MAKEPARMS {
	public:
		//! Constructor for OneToOne attachment to element.
		MAKEPARM_LEGENDELEMENT (
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM for Node Turn Impedance table.
class DBTABLE::MAKEPARM_NODETURN : public RVC::DBTABLE::MAKEPARMS {
	public:
		//! Constructor for OneToOne attachment to element.
		MAKEPARM_NODETURN (
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Oracle table creation
class DBTABLE::MAKEPARM_ORACLE : public DBTABLE::MAKEPARMS {
	public:
		MAKEPARM_ORACLE (
			) : MAKEPARMS(ATTACHMENT_RelatedOnly, CHAR_ENCODING_UTF8) { 
			SetLinkType(LINKTYPE_ORACLE); 
			}
			
		MAKEPARM_ORACLE (
			const DBTABLE::FIELDINFOARRAY& FieldInfoArray
			) : MAKEPARMS(FieldInfoArray, ATTACHMENT_RelatedOnly, CHAR_ENCODING_UTF8) {
			SetLinkType(LINKTYPE_ORACLE); 
			}
	
		MAKEPARM_ORACLE (
			const DBTABLE& Template
			) : MAKEPARMS(Template) {
			SetLinkType(LINKTYPE_ORACLE);
			SetAttachmentType(ATTACHMENT_RelatedOnly);
			SetUsage(USAGE_UserDefined);
			}
	};
	

//===================================================================================================================
//	DBTABLE::RECORD and derived classes

//! Class to contain a database record
class DBTABLE::RECORD {
	public:

		//! Flags for reading field methods (i.e. RVCDBTABLE::ReadField())
		enum FIELDFLAG {
			FIELDFLAG_None = 				0x0000,
			FIELDFLAG_NaN = 				0x0010,	//!< Return NaN for unknown values 
			FIELDFLAG_NoUnitConv = 		0x0020,	//!< Don't do any unit conversion 
			FIELDFLAG_MultiKey = 		0x0040,	//!< String together multi-field keys with commas (for MfReadFieldStr()) 
			FIELDFLAG_LocalTime = 		0x0080,	//!< Show FIELD::TYPE_DateTime in local time zone
			FIELDFLAG_MemoEncoding = 	0x8000	//!< Memo field has table.encoding encoding
			};

		// Container for either string or numeric value.
		class VALUE {
			public:
				enum TYPE {
					TYPE_Invalid,
					TYPE_Double,
					TYPE_String,
					TYPE_DateTime,
					TYPE_Color
					};

				//! Default constructor, creates "invalid" value.
				VALUE (
					) : m_type(TYPE_Invalid), m_pValue(0) { }
				//! Copy constructor.
				VALUE (
					const VALUE& rhs
					) { Copy(rhs); }
				//! Construct from double.
				VALUE (
					double value
					) : m_type(TYPE_Double), m_pValue(new double(value)) { }
				//! Construct from MISTRING.
				VALUE (
					const MISTRING& value
					) : m_type(TYPE_String), m_pValue(new MISTRING(value)) { }
				//! Construct from DATETIME.
				VALUE (
					const DATETIME& value
					) : m_type(TYPE_String), m_pValue(new DATETIME(value)) { }
				//! Construct from COLOR.
				VALUE (
					const COLOR& value
					) : m_type(TYPE_String), m_pValue(new COLOR(value)) { }
				//! Destructor.
				~VALUE ()
					{ Free(); }
				//! Assignment from VALUE.
				VALUE& operator= (const VALUE& rhs)
					{ if (this != &rhs) { Free(); Copy(rhs); return (*this); } }
				//! Assignment from double.
				VALUE& operator= (double rhs)
					{ Free(); m_type = TYPE_Double; m_pValue = new double(rhs); return (*this); }
				//! Assignment from MISTRING.
				VALUE& operator= (const MISTRING& rhs)
					{ Free(); m_type = TYPE_String; m_pValue = new MISTRING(rhs); return (*this); }
				//! Assignment from DATETIME.
				VALUE& operator= (const DATETIME& rhs)
					{ Free(); m_type = TYPE_DateTime; m_pValue = new DATETIME(rhs); return (*this); }
				//! Assignment from COLOR.
				VALUE& operator= (const COLOR& rhs)
					{ Free(); m_type = TYPE_Color; m_pValue = new COLOR(rhs); return (*this); }
				//! Comparison for equality.
				bool operator== (const VALUE& rhs) const
					{ return (IsEqual(rhs)); }
				//! Comparison for inequality.
				bool operator!= (const VALUE& rhs) const
					{ return (!IsEqual(rhs)); }
				//! Comparison for sort ordering.
				bool operator< (const VALUE& rhs) const;
				//! Get COLOR value.
				COLOR GetColor () const { return ((m_type == TYPE_Color) ? *static_cast<const COLOR*>(m_pValue) : COLOR()); }
				//! Get DATETIME value.
				DATETIME GetDateTime () const { return ((m_type == TYPE_DateTime) ? *static_cast<const DATETIME*>(m_pValue) : DATETIME()); }
				//! Get double value.
				double GetDouble () const { return ((m_type == TYPE_Double) ? *static_cast<const double*>(m_pValue) : 0); }
				//! Get MISTRING value.
				const MISTRING& GetString () const { return ((m_type == TYPE_String) ? *static_cast<const MISTRING*>(m_pValue) : MISTRING::GetEmpty()); }
				//! Get type.
				TYPE GetType () const
					{ return (m_type); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TYPE m_type;
				void *m_pValue;

				void Copy (const VALUE& rhs);
				void Free ();
				bool IsEqual (const VALUE& rhs) const;
				#endif
			};

	
		//! Default constructor			
		RECORD (
			);
			
		//! Copy constructor			
		RECORD (
			const RECORD& rhs
			);
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD (
			const RVC::DBTABLE& table
			);

		//! Destructor			
		virtual ~RECORD (
			);
			
		//! Assignment
		RECORD& operator= (
			const RECORD& rhs
			);		
		
		//! Equality - compares if the information in the two records are the same, including memo and computed fields
		bool operator== (
			RECORD& rhs
			);		
		
		//bcolombe: added new CompareRecords function which works same as operator== but takes threshold value
		bool CompareRecords (
			RECORD& rhs,
			double threshhold = 0.0
			);

		//! Inequality - compares if the information in the two records are not the same, including memo and computed fields
		bool operator!= (
			RECORD& rhs
			) { return (!(*this == rhs)); }
		
		//! Initialize this record to work with this table, used to setup internal items
		ERRVALUE AssignTable (
			const RVC::DBTABLE& table
			);
			
		//! Clear the record
		void Clear (
			);

		//! Clear attachment.
		void ClearAttachment ();

		//! Compare two entries in a field to determine if they are equal
		bool CompareField (
			INT32 FieldNum,
			const RECORD& record,
			double threshhold = 0.0
			) const;

		//! Compare two entries in a field to determine sort order
		//! returns <0 if "this" is < record, >0 if this > record, = if same
		INT32 CompareFieldOrdered (
			INT32 FieldNum,
			const RECORD& record,
			double threshhold = 0.0
			) const;

		//! Compute a hash value for the current record.
		//! This is used by the database join code to quickly compare two records
		//! for equality.
		INT32 ComputeHash (
			) const;

		//! Copy the fields that match in name and type from one record to another.
		//! Unlike using the assignment operator, which assumes the same table
		//! (or at least same structure), this method will cope with the 
		//! possibility that the source record may have different field sizes, 
		//! indexes, count and offsets than the destination record.
		ERRVALUE CopyMatchingFieldsTo (
			RECORD& DestRecord
			) const;

		//! Copy the field values from one record to another.
		//! Unlike using the assignment operator, which assumes the same table
		//! (or at least same structure), this method will cope with the 
		//! possibility that the source record may have different field sizes
		//! and offsets than the destination record.
		//! Note: it assumes that the records come from tables with the same
		//! number of fields and won't do any field reordering.
		ERRVALUE CopyTo (
			RECORD& DestRecord
			) const;

		//! Initialize the record with the constraints set for the table			
		ERRVALUE Initialize (
			);

		//! Read an entire record into a byte array		
		void GetData (
			SIMPLE_ARRAY<UINT8>& Data
			) const;

		//! Obtain the RVC::DBTABLE::FIELDINFO for the specified field
		//! @return Valid FIELDINFO if 'FieldNum' is valid, invalid FIELDINFO if 'FieldNum' is out of range
		const FIELDINFO& GetFieldInfo (
			INT32 FieldNum
			) const;

		//! Get the current record number referred to by this instance
		//! @return Record number if >= 0 or -1 if not associated with a record in a table
		INT32 GetRecordNumber (
			) const;

		inline const DBTABLE *GetTable (
			) const { return (m_Table); }

		//! Get the current table number refered to by this instance
		//! @return Table number if >= 0 or TABLE_Invalid if not associated with a table
		INT32 GetTableNumber (
			) const;

		//! Read field from this record into a RECORD::VALUE.
		ERRVALUE GetValue (
			INT32 FieldNum,
			RECORD::VALUE& value,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			) const;

		//! Read a field from this record into an MISTRING		
		ERRVALUE GetValue (
			INT32 FieldNum,
			MISTRING& string,
			FIELDFLAG FieldFlag = FIELDFLAG_None,
			int DecimalPlaces = -1,
			const UNITCONV* UnitConv = 0
			) const;

		//! Read a field from this record and return a double
		//! @return Double value converted from the field specified
		double GetValue (
			INT32 FieldNum,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			) const;
			
		//! Read the value of a COLOR field
		ERRVALUE GetValue (
			INT32 FieldNum,
			COLOR& color
			) const;

		//! Read the value of a Date or DateTime field
		ERRVALUE GetValue (
			INT32 FieldNum,
			DATETIME& datetime
			) const;

		//! Read a binary field from this record into a SIMPLE_ARRAY<UINT8>
		//! This will return an empty array if the field is not a FIELDINFO::TYPE_Binary
		ERRVALUE GetValueBinary (
			INT32 FieldNum,
			SIMPLE_ARRAY<UINT8>& binary
			) const;

		//! Read an image out of a field.
		//!
		//! This will construct and return a SPATMOD::IMAGE::SOURCE from a
		//! field with USAGE_ImageBLOB or USAGE_ImageFile.  The caller is
		//! responsible for freeing the stage when they're done with it.
		//!
		//! If the bForThumbnail parameter, it will add a FILTER_ZOOM stage
		//! which will resize the image such that the maxium dimension is
		//! MaxThumbnailSize.
		//! 
		//! When requesting a thumbnail, if the field is followed by a field
		//! of USAGE_ImageThumbnail, it will use the thumbnail field instead
		//! of the requested field.  
		//! 
		//! If the requested field doesn't have an image USAGE, 
		//! this method will return EBadFuncParm.
		ERRVALUE GetValueImage (
			INT32 FieldNum,
			SPATMOD::IMAGE::STAGE*& pImage,		//!< Image returned
			bool bForThumbnail = false,
			INT32 MaxThumbnailSize = 100
			) const;

		//! Determine if 'this' is assigned to a table
		//! @return 'True' if assigned to a table, 'false' if not
		bool IsAssigned (
			) const;

		//! Write a set of fields from another RECORD field to this record if the tables are related 
		//! using the key field / next field association
		ERRVALUE SetKeyValues (
			INT32 StartFieldNum,
			const RECORD& SourceRecord
			);
			
		//! Write an entire record from a byte array		
		void SetData (
			const SIMPLE_ARRAY<UINT8>& Data
			);

	#ifndef NO_DEPRECATED
		//! Temprorary until all of the database editor is converted over.
		//! Assumes the caller allocated the buffer to the correct size.
		DEPRECATED void SetData (
			const void* buf
			);
	#endif

		//! Write a field from a RECORD::VALUE to this record.
		ERRVALUE SetValue (
			INT32 FieldNum,
			const RECORD::VALUE& value,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			);

		//! Write a field from a COLOR to this record (for COLOR fields only)
		ERRVALUE SetValue (
			INT32 FieldNum,
			const COLOR& color
			);

		//! Write the value of a Date or DateTime field 
		ERRVALUE SetValue (
			INT32 FieldNum,
			const DATETIME& datetime
			) ;

		//! Write a field from a MISTRING to this record
		ERRVALUE SetValue (
			INT32 FieldNum,
			const MISTRING& string,
			FIELDFLAG FieldFlag = FIELDFLAG_None,
			const UNITCONV* conv = 0
			);

		//! Write a field from a double to this record
		void SetValue (
			INT32 FieldNum,
			double value,
			bool NoUnitConv = false
			);
			
		//! Write a field from another RECORD field to this record
		ERRVALUE SetValue (
			INT32 FieldNum,
			const RECORD& SourceRecord,
			INT32 SourceFieldNum
			);
			
		//! Write a binary field to this record from a SIMPLE_ARRAY<UINT8>
		ERRVALUE SetValueBinary (
			INT32 FieldNum,
			const SIMPLE_ARRAY<UINT8>& binary
			) const;

	protected:
	
		virtual bool IsValidTable (const RVC::DBTABLE& /*table*/) {return (true);}
	
		const void *const GetFieldPtr (
			INT32 FieldNum
			) const;

		void *GetFieldPtr (
			INT32 FieldNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct BINARY {
			SIMPLE_ARRAY<UINT8> m_Binary;
			INT32 m_FieldNum;
			bool m_HasChanged;
			
			BINARY () : m_FieldNum(-1), m_HasChanged(false) {}
			};

		struct MEMO {
			MISTRING m_Memo;
			INT32 m_FieldNum;
			bool m_HasChanged;
			
			MEMO () : m_FieldNum(-1), m_HasChanged(false) {}
			};

		mutable void *m_Buffer;
		const DBTABLE* m_Table;
		RVC::DBTABLE::FIELDINFOARRAY m_DBFields;
		INT32 m_RecordNum;
		CURSOR* m_Cursor;
		BOOKMARK m_Bookmark;
		UINT32 m_Flags;
		int m_BufSize;
		mutable BITSET m_CompFieldRead;
		mutable MILIST<MEMO> m_MemoList;
		class TABLEOBSERVER;
		friend class TABLEOBSERVER;
		TABLEOBSERVER* m_TableObserver;
		mutable MILIST<BINARY> m_BinaryList;
		CHAR_ENCODING m_TableEncoding;
		
		INT32 CompareBinaryField (INT32 FieldNum, const RECORD& record) const;
		INT32 CompareMemoField (INT32 FieldNum, const RECORD& record) const;
		ERRVALUE ReadComputed (INT32 FieldNum) const;
		ERRVALUE ReadMemo (MEMO& Memo, INT32 FieldNum) const;
		ERRVALUE ReadBinary (BINARY& Binary, INT32 FieldNum) const;
		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Element ID table records
class DBTABLE::RECORD_ID_ELEMENT : public RVC::DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_ID_ELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_ID_ELEMENT (
			const RECORD_ID_ELEMENT& rhs
			) : RECORD(rhs) {}
			
		//! Copy constructor		
		RECORD_ID_ELEMENT (
			const RECORD& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVC::DBTABLE - used to setup internal items
		RECORD_ID_ELEMENT (
			const RVC::DBTABLE& table
			) : RECORD(table) {}

		//! Destructor			
		virtual ~RECORD_ID_ELEMENT (
			) {}
	
		//! Assignment
		RECORD_ID_ELEMENT& operator= (
			const RECORD_ID_ELEMENT& rhs
			) {
			RECORD::operator=(rhs);
			return (*this);
			}
		
		//! Get Current ID value
		//!	@return Current ID
		INT32 GetCurrent (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(0)));}
			
		//! Get Original ID value
		//!	@return Original ID
		INT32 GetOriginal (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(1)));}
			
		//! Set Current ID value
		void SetCurrent (
			INT32 current
			) {*static_cast<INT32*>(GetFieldPtr(0)) = current;}
			
		//! Set Original ID value
		void SetOriginal (
			INT32 original
			) {*static_cast<INT32*>(GetFieldPtr(1)) = original;}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_ElementID);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Point ID table records
class DBTABLE::RECORD_ID_POINT : public RVC::DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_ID_POINT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_ID_POINT (
			const RECORD_ID_POINT& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_ID_POINT (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_ID_POINT (
			) {}
	
		//! Assignment
		RECORD_ID_POINT& operator= (
			const RECORD_ID_POINT& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
		
		//! Get Current ID value
		//!	@return Current ID
		INT32 GetCurrent (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(0)));}
			
		//! Set Current ID value
		void SetCurrent (
			INT32 current
			) {*static_cast<INT32*>(GetFieldPtr(0)) = current;}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_ElementID);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Record for table containing single element style.
class DBTABLE::RECORD_STYLE_ELEMENT : public RVC::DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_STYLE_ELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_STYLE_ELEMENT (
			const RECORD_STYLE_ELEMENT& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_STYLE_ELEMENT (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_STYLE_ELEMENT (
			) {}
	
		//! Assignment
		RECORD_STYLE_ELEMENT& operator= (
			const RECORD_STYLE_ELEMENT& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		static INT32 GetRelationFieldNum (
			) {return (SAFIELD_SINGLE_RelField);}
		
		static INT32 GetSelectionFieldNum (
			) {return (SAFIELD_SINGLE_DrawFlag);}
	
		INT32 GetStyleIndex (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_SINGLE_StyleIndex)));}

		const RVC::OBJECTNAME& GetStyleName (
			) const {return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_SINGLE_StyleName)));}

		bool IsSelected (
			) const {return (*static_cast<const INT8*>(GetFieldPtr(SAFIELD_SINGLE_DrawFlag)) != 0);}

		ERRVALUE SetRelationValues (
			const RECORD& record
			) {return (SetKeyValues(SAFIELD_SINGLE_RelField, record));}
		
		void SetSelected (
			bool Selected
			) {*static_cast<INT8*>(GetFieldPtr(SAFIELD_SINGLE_DrawFlag)) = Selected;}
			
		void SetStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_SINGLE_StyleIndex)) = index;}

		void SetStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_SINGLE_StyleName)) = name;}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (
			const RVC::DBTABLE& table
			) {
			DBTABLE::USAGE usage = table.GetUsage();
			return (usage == DBTABLE::USAGE_PointStyle || usage == DBTABLE::USAGE_LineStyle || usage == DBTABLE::USAGE_PolyStyle);
			}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Record containing multiple element styles.
class DBTABLE::RECORD_STYLE_CAD : public RVC::DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_STYLE_CAD (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_STYLE_CAD (
			const RECORD_STYLE_CAD& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_STYLE_CAD (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_STYLE_CAD (
			) {}
	
		//! Assignment
		RECORD_STYLE_CAD& operator= (
			const RECORD_STYLE_CAD& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		INT32 GetLineStyleIndex (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_LineStyleIndex)));}

		const RVC::OBJECTNAME& GetLineStyleName (
			) const {return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_LineStyleName)));}

		INT32 GetPointStyleIndex (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_PointStyleIndex)));}

		const RVC::OBJECTNAME& GetPointStyleName (
			) const {return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PointStyleName)));}

		INT32 GetPolyStyleIndex (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_PolyStyleIndex)));}

		const RVC::OBJECTNAME& GetPolyStyleName (
			) const {return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PolyStyleName)));}

		static INT32 GetRelationFieldNum (
			) {return (SAFIELD_CAD_RelField);}
		
		static INT32 GetSelectionFieldNum (
			) {return (SAFIELD_CAD_DrawFlag);}
	
		INT32 GetStyleIndex (
			RVC::STYLETYPE StyleType
			) const {
			switch (StyleType) {
				case STYLETYPE_Point: return (GetPointStyleIndex());
				case STYLETYPE_Line: return (GetLineStyleIndex());
				case STYLETYPE_Polygon: return (GetPolyStyleIndex());
				case STYLETYPE_Text: return (GetTextStyleIndex());
				}
			return (-1);
			}

		const RVC::OBJECTNAME& GetStyleName (
			RVC::STYLETYPE StyleType
			) const {
			switch (StyleType) {
				case STYLETYPE_Point: return (GetPointStyleName());
				case STYLETYPE_Line: return (GetLineStyleName());
				case STYLETYPE_Polygon: return (GetPolyStyleName());
				}
			return (GetTextStyleName());		//! Good as anything for a default
			}

		INT32 GetTextStyleIndex (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_TextStyleIndex)));}

		const RVC::OBJECTNAME& GetTextStyleName (
			) const {return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_TextStyleName)));}

		bool IsSelected (
			) const {return (*static_cast<const INT8*>(GetFieldPtr(SAFIELD_CAD_DrawFlag)) != 0);}

		void SetLineStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_LineStyleIndex)) = index;}

		void SetLineStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_LineStyleName)) = name;}

		void SetPointStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_PointStyleIndex)) = index;}

		void SetPointStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PointStyleName)) = name;}

		void SetPolyStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_PolyStyleIndex)) = index;}

		void SetPolyStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PolyStyleName)) = name;}

		ERRVALUE SetRelationValues (
			const RECORD& record
			) {return (SetKeyValues(SAFIELD_CAD_RelField, record));}
		
		void SetSelected (
			bool Selected
			) {*static_cast<INT8*>(GetFieldPtr(SAFIELD_CAD_DrawFlag)) = Selected;}
			
		void SetStyleIndex (
			RVC::STYLETYPE StyleType,
			INT32 index
			) {
			switch (StyleType) {
				case STYLETYPE_Point: SetPointStyleIndex(index); break;
				case STYLETYPE_Line: SetLineStyleIndex(index); break;
				case STYLETYPE_Polygon: SetPolyStyleIndex(index); break;
				case STYLETYPE_Text: SetTextStyleIndex(index); break;
				}
			return;
			}

		void SetStyleName (
			RVC::STYLETYPE StyleType,
			const RVC::OBJECTNAME& name
			) {
			switch (StyleType) {
				case STYLETYPE_Point: SetPointStyleName(name); break;
				case STYLETYPE_Line: SetLineStyleName(name); break;
				case STYLETYPE_Polygon: SetPolyStyleName(name); break;
				case STYLETYPE_Text: SetTextStyleName(name); break;
				}
			return;
			}

		void SetTextStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_TextStyleIndex)) = index;}

		void SetTextStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_TextStyleName)) = name;}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_MultiStyle);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_SCALESELECT : public RVC::DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_SCALESELECT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_SCALESELECT (
			const RECORD_SCALESELECT& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_SCALESELECT (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_SCALESELECT (
			) {}
	
		//! Assignment
		RECORD_SCALESELECT& operator= (
			const RECORD_SCALESELECT& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		double GetMaxScale (
			) const {return (*static_cast<const double*>(GetFieldPtr(SCALESELFIELD_MaxScale)));}

		double GetMinScale (
			) const {return (*static_cast<const double*>(GetFieldPtr(SCALESELFIELD_MinScale)));}

		static INT32 GetRelationFieldNum (
			) {return (SCALESELFIELD_RelField);}
	
		void SetMaxScale (
			double scale
			) {*static_cast<double*>(GetFieldPtr(SCALESELFIELD_MaxScale)) = scale;}

		void SetMinScale (
			double scale
			) {*static_cast<double*>(GetFieldPtr(SCALESELFIELD_MinScale)) = scale;}

		ERRVALUE SetRelationValues (
			const RECORD& record
			) {return (SetKeyValues(SCALESELFIELD_RelField, record));}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_ScaleSelect);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_SELECTION : public RVC::DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_SELECTION (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_SELECTION (
			const RECORD_SELECTION& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_SELECTION (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_SELECTION (
			) {}
	
		//! Assignment
		RECORD_SELECTION& operator= (
			const RECORD_SELECTION& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		bool IsSelected (
			) const {return (*static_cast<const UINT8*>(GetFieldPtr(GetSelectionFieldNum())) != 0);}

		static INT32 GetRelationFieldNum (
			) {return (1);}
	
		static INT32 GetSelectionFieldNum (
			) {return (0);}
	
		void SetSelected (
			bool Selected
			) {*static_cast<UINT8*>(GetFieldPtr(GetSelectionFieldNum())) = Selected;}

		ERRVALUE SetRelationValues (
			const RECORD& record
			) {return (SetKeyValues(GetRelationFieldNum(), record));}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_Selection);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_LEGENDELEMENT : public RVC::DBTABLE::RECORD {
	public:

		enum FIELDNUM {
			FIELDNUM_ID =			0,
			FIELDNUM_PriLabel =	1,
			FIELDNUM_SecLabel =	2
			};

		//! Default constructor		
		RECORD_LEGENDELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_LEGENDELEMENT (
			const RECORD_LEGENDELEMENT& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVC::DBTABLE - used to setup internal items
		RECORD_LEGENDELEMENT (
			const RVC::DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_LEGENDELEMENT (
			) {}
	
		//! Assignment
		RECORD_LEGENDELEMENT& operator= (
			const RECORD_LEGENDELEMENT& rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		//! Get field number for ID field.
		static INT32 GetFieldNumID (
			) { return (FIELDNUM_ID); }
	
		//! Get field number for primary label field.
		static INT32 GetFieldNumPrimaryLabel (
			) { return (FIELDNUM_PriLabel); }
	
		//! Get field number for secondary label field.
		static INT32 GetFieldNumSecondaryLabel (
			) { return (FIELDNUM_SecLabel); }

		//! Get unique record ID.
		UINT32 GetID (
			) const { return (*static_cast<const UINT32*>(GetFieldPtr(FIELDNUM_ID))); }

		//! Get string for primary label.
		ERRVALUE GetPrimaryLabel (
			MISTRING& str
			) const { return (GetValue(FIELDNUM_PriLabel,str)); }
	
		//! Get string for secondary label.
		ERRVALUE GetSecondaryLabel (
			MISTRING& str
			) const { return (GetValue(FIELDNUM_SecLabel,str)); }
	
		//! Set string for primary label.
		ERRVALUE SetPrimaryLabel (
			const MISTRING& str
			) { return (SetValue(FIELDNUM_PriLabel,str)); }
	
		//! Set string for secondary label.
		ERRVALUE SetSecondaryLabel (
			const MISTRING& str
			) { return (SetValue(FIELDNUM_SecLabel,str)); }
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_LegendElement); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Node Turn table records
class DBTABLE::RECORD_NODETURN : public RVC::DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_NODETURN (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_NODETURN (
			const RECORD_NODETURN& rhs
			) : RECORD(rhs) {}
			
		//! Copy constructor		
		RECORD_NODETURN (
			const RECORD& rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVC::DBTABLE - used to setup internal items
		RECORD_NODETURN (
			const RVC::DBTABLE& table
			) : RECORD(table) {}

		//! Destructor			
		virtual ~RECORD_NODETURN (
			) {}
	
		//! Assignment
		RECORD_NODETURN& operator= (
			const RECORD_NODETURN& rhs
			) {
			RECORD::operator=(rhs);
			return (*this);
			}
			
		//! Set node impedance to invalidate turn
		void DisableTurn (
			) {*static_cast<double*>(GetFieldPtr(2)) = -1.0;}
		
		//! Get Line number coming into this node
		//!	@return Line In Number
		INT32 GetLineIn (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(0)));}
			
		//! Get Line number coming out of this node
		//!	@return Line Out Number
		INT32 GetLineOut (
			) const {return (*static_cast<const INT32*>(GetFieldPtr(1)));}
			
		//! Get Node Impedance
		//!	@return Node Impedance
		double GetImpedance (
			) const {return (*static_cast<const double*>(GetFieldPtr(2)));}
			
		//! Set Line number coming into this node
		void SetLineIn (
			INT32 LineIn
			) {*static_cast<INT32*>(GetFieldPtr(0)) = LineIn;}
			
		//! Set Line number coming out of this node
		void SetLineOut (
			INT32 LineOut
			) {*static_cast<INT32*>(GetFieldPtr(1)) = LineOut;}
		
		//! Set Node Impedance
		void SetImpedance (
			double Impedance
			) {*static_cast<double*>(GetFieldPtr(2)) = Impedance;}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const RVC::DBTABLE& table) {return (table.GetUsage() == DBTABLE::USAGE_NodeTurn);}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Constant record iterator class
//!
//! This class is used to iterate through a table's records.  This construct replaces the GetNumRecords() and cursor routines.
//! See methods RecordBegin() and RecordEnd() for obtaining a record iterator
class DBTABLE::CONST_RECORDITERATOR {
	public:
		//! Default constructor
		CONST_RECORDITERATOR (
			) : 
			m_TableObj(0),
			m_Record(0),
			m_NeedRecord(true),
			m_CheckSingle(false),
			m_FieldNum(0),
			m_Cursor(0)
			{}

		//! Destructor
		virtual ~CONST_RECORDITERATOR (
			) {
			delete m_Cursor;
			delete m_Record;
			}

		//! Dereference
		const RECORD& operator* (
			) const;

		//! Dereference arrow
		const RECORD* operator-> (
			) const {return (&**this);}

		//! Pre-increment
		CONST_RECORDITERATOR& operator++ (
			);

		bool Done (
			) const {return (m_Cursor == 0 || !m_Cursor->IsValid());}
			
		ERRVALUE GetBookmark (
			DBTABLE::BOOKMARK& bookmark
			) const {return (m_Cursor->GetBookmark(bookmark));}

	protected:
		mutable RECORD* m_Record;
		INT32 m_FieldNum;
		CURSOR* m_Cursor;
		mutable bool m_NeedRecord;
		bool m_CheckSingle;
		const DBTABLE* m_TableObj;
		
		void SetupIterator (const RVC::DBTABLE* table, ITERATERECORD IterateRecord, INT32 FieldNum);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CONST_RECORDITERATOR (const CONST_RECORDITERATOR& rhs);
		CONST_RECORDITERATOR& operator= (const CONST_RECORDITERATOR& rhs);
		
		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Record iterator class
//!
//! This class is used to iterate through a table's records.  This construct replaces the GetNumRecords() and cursor routines.
//! See methods RecordBegin() and RecordEnd() for obtaining a record iterator
class DBTABLE::RECORDITERATOR : public RVC::DBTABLE::CONST_RECORDITERATOR {
	public:

		//! Default constructor
		RECORDITERATOR (
			) : CONST_RECORDITERATOR() {}

		//! Dereference
		RECORD& operator* (
			) {return (*const_cast<RECORD*>(&CONST_RECORDITERATOR::operator*()));}

		//! Dereference arrow
		RECORD* operator-> (
			) {return (&**this);}

		//! Pre-increment
		RECORDITERATOR& operator++ (
			) {
			CONST_RECORDITERATOR::operator++();
			return (*this);
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RECORDITERATOR (CONST_RECORDITERATOR& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! OBSERVER class for DBTABLE objects.  The SUBJECT class is DBTABLE.
class DBTABLE::OBSERVER : public RVC::OBJECTOBSERVER {
	public:

		//! Types of table structural changes made, used in RVCDBTABLE::OBSERVER::OnStructChanged()
		enum STRUCTCHANGE {
			STRUCTCHANGE_None = 					0x00000000,
			STRUCTCHANGE_Name = 					0x00000001,
			STRUCTCHANGE_Desc = 					0x00000002,
			STRUCTCHANGE_ViewMode = 			0x00000004,
			STRUCTCHANGE_Field = 				0x00000008,			//!< Something in one or more fields changed 
			STRUCTCHANGE_FieldsAdded = 		0x00000010,
			STRUCTCHANGE_Flags = 				0x00000020,
			STRUCTCHANGE_AttachmentType = 	0x00000040,
			STRUCTCHANGE_Encoding = 			0x00000080,
			STRUCTCHANGE_DBEditorUpdate = 	0x00000100,
			STRUCTCHANGE_DBEditorRebuild = 	0x00000200
			};	

		explicit OBSERVER (
			RVC::DBTABLE& dbtable
			) : OBJECTOBSERVER(dbtable) {}
			
		virtual ~OBSERVER (
			) {}
			
		//! Called before a multi-record change
		virtual void OnBulkModifyBegin (
			) {return;}
			
		//! Called after a multi-record change
		virtual void OnBulkModifyEnd (
			) {return;}
			
		//! Called when a record is added to the table
		virtual void OnRecordAdd (
			INT32											//!< RecordNum
			) {return;}
			
		//! Called when a record is about to be altered
		virtual void OnRecordChangeBegin (
			INT32											//!< RecordNum
			) {return;}
			
		//! Called when a record has been altered
		virtual void OnRecordChangeEnd (
			INT32											//!< RecordNum
			) {return;}
			
		//! Called when a record is about to be deleted
		virtual void OnRecordDeleteBegin (
			INT32											//!< RecordNum
			) {return;}
			
		//! Called when a record has been deleted
		virtual void OnRecordDeleteEnd (
			INT32											//!< RecordNum
			) {return;}
			
		//! Called when table structure information has changed
		virtual void OnStructChanged (
			STRUCTCHANGE								//!< SubReason
			) {return;}

		//! Called when the table becomes valid or invalid
		virtual void OnTableValid (
			bool											//!< WasInvalid
			) {return;}
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual void OnNotifyRaw (UINT32 reason, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::INDEXSEARCH)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::ITERATERECORD)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::FINDFLAG)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::KEYFLAGS)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::REFELEM)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::RECORD::FIELDFLAG)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::FIELDINFO::FLAGS)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::CONSTRAINT::RANGEFLAG)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::CONSTRAINT::PKTABLE)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::CURSOR::FUNCTION)
DEFINE_ENUM_OP_BITWISE(RVC::DBTABLE::CURSOR::INDEXFLAG)
DEFINE_ENUM_OP_INCREMENT(RVC::DBTABLE::FIELDINFO::TYPE)
DEFINE_ENUM_OP_INCREMENT(RVC::DBTABLE::USAGE)
#endif

}		// End of RVC namespace

#endif 	// INC_RVC_DBTABLE_H
