/**
 * \file rvc/ofont.h
 * \brief interface for the RVC::OFONT class.
 *
 * \if NODOC
 * $Id: ofont.h_v 1.5 2007/09/12 17:10:38 scowan Exp $
 *
 * $Log: ofont.h_v $
 * Revision 1.5  2007/09/12 17:10:38  scowan
 * Moved gen and basic info into private include files.
 *
 * Revision 1.4  2006/10/16 14:19:53  mju
 * Use point hdr.
 *
 * Revision 1.3  2005/12/09 17:29:44  scowan
 * Updated to use credentials.
 *
 * Revision 1.2  2005/09/02 16:23:08  dwilliss
 * Fixed doxygen comments - cut-paste error was causing it to document
 * TEXTOBJ as if it were an OFONT
 *
 * Revision 1.1  2003/12/26 20:10:46  scowan
 * Initial revision
 *
 *
 * \endif
**/

//!
//!	\class RVC::OFONT rvc/ofont.h
//!	\brief Manages RVC::OBJTYPE_OutlineFont objects
//!	
//!	RVC Object Types that this class supports: 
//!		- RVC::OBJTYPE_OutlineFont
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_OutlineFont.
//!
//!	Valid parents for this class:
//!		- OBJTYPE_File and OBJTYPE_Folder

#ifndef INC_RVC_OFONT_H
#define INC_RVC_OFONT_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef	INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

struct RVCOFONTCHAR {
	WPOINT2D basepoint;						//!<  base point for this character 
	WPOINT2D minval;							//!<  minimum coordinate of character 
	WPOINT2D maxval;							//!<  maximum coordinate of character 
	INT16 kerning;								//!<  horizontal adjustment for proportional kerning 
	UINT16 segments;							//!<  number of segments 
	UINT16 ascii;								//!<  ASCII code for this character 
	UINT8 spare[6];							//!<  extra, in case we need it 
	};												/* makes 24 bytes total */

struct RVCOFONTCHARSEG {
	UINT16 numpoints;
	UINT8 type;									//!<  outline or fill polygon, defines a circle, etc. 
	UINT8 subtype;								//!<  connect points w/ lines, splines, Bézier curves, etc. 
	UINT8 spare[4];							//!<  makes 8 bytes 
	};

//! RVFOFONTCHARSEG types: 
#define OF_NORMAL   0						//!<  normal segment type, fill polygon when done 
#define OF_MULTI    1						/* Multiple segments follow, read numpoints of them
													and fill resulting shape.  EXAMPLE:  (numpoints == 3)
													line seg, then b-spline, then a line,
													then fill entire shape */

//! RVFVCHARSEG subtypes:
#define OF_NORMAL   0						//!<  should be 0 
#define OF_STROKE   1						//!<  should be 1 
#define OF_BSPLINE  2						//!<  should be 2 

/* Do not mess with this structure: MOTU */
struct OFONTCHARCTRLPT {
	INT16 x,y;									//!<  relative to base point 
	UINT16 type;
	UINT16 spare;
	};		

//! OFONTCHARCTRLPT types:
#define OF_ONE   0							//!<  use control point once 
#define OF_TWO   1							//!<  use control point twice 
#define OF_THREE 2							//!<  use control point thrice 

namespace RVC {
//Forward Declarations
class OFONTHANDLE;
struct OFONTINFO;

class OFONT : public RVC::OBJECT {
	public:

		//! Default constructor
		OFONT (
			);

		//! Copy constructor
		OFONT (
			const RVC::OFONT& obj
			);

		//! Destructor
		virtual ~OFONT (
			);

		//! Assignment
		RVC::OFONT& operator= (
			const RVC::OFONT& rhs
			);

		INT32 GetAscent (
			) const;

		INT32 GetDescent (
			) const;

		INT32 GetInitChar (
			) const;

		INT32 GetLastChar (
			) const;

		INT32 GetLeading (
			) const;

		INT32 GetMaxGlyphSize (
			) const;

		INT32 GetNumGlyphs (
			) const;

		//! Read font character into the buffer (ptr)
		//! @return Size of the font character in bytes
		INT32 ReadCharacter (
			INT32 item,
			void *ptr
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);
		virtual bool v_ValidateObjectType (const OBJTYPE otype) const;

		OFONTHANDLE* m_OFontHandle;
		OFONTINFO* m_OFontInfo;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_OFONT_H
