/**
 * \file rvc/style.h 
 * \brief interface for the RVC::STYLE class.
 *
 * \if NODOC
 * $Id: style.h_v 1.87 2011/12/23 14:09:20 mju Exp $
 *
 * $Log: style.h_v $
 * Revision 1.87  2011/12/23 14:09:20  mju
 * Remove redundant hdr.
 *
 * Revision 1.86  2007/11/20 16:58:43  mju
 * Add defaults and docs for stylePatternCopy.
 *
 * Revision 1.85  2007/11/20 16:40:18  mju
 * Fix parm for prev chg.
 *
 * Revision 1.84  2007/11/20 16:34:45  mju
 * Add ui::styleCopyPattern and flags.
 *
 * Revision 1.83  2006/10/13 19:24:28  mju
 * use getobjfilter hdr.
 *
 * Revision 1.82  2006/09/20 17:01:08  scowan
 * Added deprecated warnings to methods that take internal style structure representations.
 *
 * Revision 1.81  2006/09/20 16:19:07  scowan
 * Removed all but one deprecated typedef.
 *
 * Revision 1.80  2006/09/20 15:34:14  scowan
 * Removed hatch pattern typedef.
 *
 * Revision 1.79  2006/09/20 15:10:41  mju
 * Move dialogs to uistyle hdr.
 *
 * Revision 1.78  2006/09/19 14:55:35  scowan
 * Removed legacy pattern enum.
 *
 * Revision 1.77  2006/09/19 13:13:36  mju
 * Deprecate legacy typedefs and exclude deprecated if no_deprecated defined.
 *
 * Revision 1.76  2006/09/19 13:10:25  mju
 * Remove nested "type" enum.
 *
 * Revision 1.75  2006/09/18 17:53:31  scowan
 * Moved pattern classes out into their own files.
 *
 * Revision 1.74  2006/09/15 19:57:02  scowan
 * Copied type enums to own include file.
 *
 * Revision 1.73  2006/06/23 15:30:16  scowan
 * Now uses style handle class.
 *
 * Revision 1.72  2006/05/01 19:44:12  dwilliss
 * never mind -- the flag I added needed to go on a different thing
 *
 * Revision 1.71  2006/05/01 19:36:13  dwilliss
 * added an editstyle flag
 *
 * Revision 1.70  2006/03/07 16:05:51  mju
 * Add dftStyleType for dlgEditStyleByAttribute with multistyle.
 *
 * Revision 1.69  2005/12/19 18:35:30  dwilliss
 * adjusted mandatory credentials to account for prior optional parameters
 *
 * Revision 1.68  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.67  2005/12/09 17:29:59  scowan
 * Updated to use credentials.
 *
 * Revision 1.66  2005/08/22 20:41:14  vdronov
 * text rendering in 3d
 *
 * Revision 1.65  2005/06/02 19:20:08  vdronov
 * *** empty log message ***
 *
 * Revision 1.64  2005/05/20 21:50:54  vdronov
 * added GetWidth
 *
 * Revision 1.63  2005/04/29 22:07:20  vdronov
 * extended SYMBOLPATTERN
 *
 * Revision 1.62  2005/01/19 21:38:08  scowan
 * Added support for descriptor.
 *
 * Revision 1.61  2004/12/14 17:50:39  mju
 * Add parm to getobjfilter for saving to allow navigate under geometric.
 *
 * Revision 1.60  2004/11/19 17:54:27  mju
 * Add basepattern.isEqualDef.
 *
 * Revision 1.59  2004/11/19 14:45:52  mju
 * Add TYPE_First for more intuitive looping.
 *
 * Revision 1.58  2004/09/03 16:02:34  scowan
 * Removed text font name typedef.
 *
 * Revision 1.57  2004/06/16 17:29:08  scowan
 * Added style edit for multi style.
 *
 * Revision 1.56  2003/10/31 15:41:35  mju
 * Change DlgGetPattern to just have const mistring& for prompt.
 *
 * Revision 1.55  2003/10/15 16:27:50  scowan
 * Added more standard filters.
 *
 * Revision 1.54  2003/10/14 21:52:50  scowan
 * Added is selectable filter.
 *
 * Revision 1.53  2003/10/08 22:18:13  scowan
 * Doxygen.
 *
 * Revision 1.52  2003/10/07 17:30:20  scowan
 * Object virtual method changes.
 *
 * Revision 1.51  2003/10/03 20:15:16  linux32build!build
 * Don't capitalize enum any more. Genitor needed it, Doxygen doesn't like it.
 *
 * Revision 1.50  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.49  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.48  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.47  2003/09/08 20:25:57  dwilliss
 * Added doxygen tags for function groups
 *
 * Revision 1.46  2003/07/15 15:40:13  dwilliss
 * Gave HATCHPATTERN, BITMAPPATTERN and SYMBOLPATTERN copy constructors and
 * assignment operators.
 *
 * Revision 1.45  2003/03/19 22:38:56  scowan
 * Added unique filters for styles and patterns.
 *
 * Revision 1.44  2003/03/13 22:11:54  mju
 * Move pattern element offset array to base class protected so can clear on read.
 *
 * Revision 1.43  2003/03/04 16:53:45  scowan
 * Added write pattern name.
 *
 * Revision 1.42  2003/03/03 15:21:24  mju
 * nc.
 *
 * Revision 1.41  2003/02/27 18:53:58  mju
 * Make bitmappattern::create non-inline and remove unimplemented DeleteUnusedColors.
 *
 * Revision 1.40  2003/02/24 17:29:51  mju
 * Had comparison reversed in setOffsetAside.
 *
 * Revision 1.39  2003/02/24 15:02:24  mju
 * In line pattern get/set OffsetAside flip sign for non-line elements so consistent.
 *
 * Revision 1.38  2003/02/20 19:02:20  mju
 * Add linepattern iterator dtor.
 *
 * Revision 1.37  2003/02/06 21:33:46  scowan
 * nc.
 *
 * Revision 1.36  2003/01/31 18:48:30  mju
 * Add parm to TableEditStyleAssignment for optional dft style obj item.
 *
 * Revision 1.35  2003/01/30 23:04:33  mju
 * Allow dft ctor/op= for linepattern::element_shared since behavior would be correct.
 *
 * Revision 1.34  2003/01/30 22:36:55  mju
 * In style editor allow return of modified dftstyle.
 *
 * Revision 1.33  2003/01/30 18:27:18  mju
 * Use explicit on element_shared ctor.
 *
 * Revision 1.32  2003/01/24 17:20:37  mju
 * *** empty log message ***
 *
 * Revision 1.31  2003/01/24 16:42:41  mju
 * Try again to fix for Unix.
 *
 * Revision 1.30  2003/01/24 14:09:58  mju
 * Attempt to fix Unix compile problem.
 *
 * Revision 1.29  2003/01/23 22:37:48  mju
 * Add LINEPATTERN element stuff, iterator, etc.
 *
 * Revision 1.28  2002/10/14 16:28:01  mju
 * Fix EDITSTYLEFLAGS.
 *
 * Revision 1.27  2002/10/14 15:06:16  mju
 * Reorganize EDITSTYLEFLAGS.
 *
 * Revision 1.26  2002/10/04 17:50:46  mju
 * Add Dlg for editing cad style by attrib.
 *
 * Revision 1.25  2002/08/22 14:24:47  scowan
 * nc.
 *
 * Revision 1.24  2002/08/21 20:39:16  scowan
 * Deprecate old object name define.
 *
 * Revision 1.23  2002/08/13 13:28:59  mju
 * Change dlggetpatterns naming.
 *
 * Revision 1.22  2002/08/12 13:29:36  mju
 * Add dlgStyleEdit and dlgTableEdit... fns.
 *
 * Revision 1.21  2002/05/24 16:30:50  scowan
 * Fixed base pattern allocations.
 *
 * Revision 1.20  2002/05/23 22:28:37  scowan
 * Added more pattern methods.
 *
 * Revision 1.19  2002/05/23 20:44:16  scowan
 * Added multiple methods.
 *
 * Revision 1.18  2002/04/11 19:24:31  scowan
 * Added multiple methods dealing with in-memory style structures.
 *
 * Revision 1.17  2001/10/24 16:18:49  scowan
 * Changed return types of delete methods.
 *
 * Revision 1.14  2001/09/04 17:10:28  mju
 * Add DlgEditPattern/PatternSet method defns.
 *
 * Revision 1.13  2001/08/31 20:04:22  scowan
 * Nothing.
 *
 * Revision 1.12  2001/08/31 19:26:31  mju
 * Add WriteStyleName() method defn.
 *
 * Revision 1.11  2001/08/31 15:57:05  mju
 * Use const in FindFontName().
 *
 * Revision 1.10  2001/08/28 14:56:39  mju
 * Change StyleAdd to return style number.
 *
 * Revision 1.9  2001/08/27 14:53:28  mju
 * Use DEFINE_ENUM_OP_INCREMENT on style type.
 * Fix inclusion guard name.
 *
 * Revision 1.8  2001/06/14 16:13:02  scowan
 * Changed source type.
 *
 * Revision 1.7  2001/06/13 22:11:43  scowan
 * Added to RVC namespace.
 *
 * Revision 1.6  2001/06/13 15:04:20  scowan
 * Added new make method.
 *
 * Revision 1.5  2001/06/13 15:02:03  scowan
 * Changed enum value.
 *
 * Revision 1.4  2000/12/01 16:01:45  scowan
 * Fixed MAC errors.
 *
 * Revision 1.3  2000/11/16 17:55:51  scowan
 * Different inclusion guards for this file.
 *
 * Revision 1.1  2000/09/29 15:49:58  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_STYLE_H
#define INC_RVC_STYLE_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#ifndef INC_RVC_GETOBJFILTER_H
	#include <rvc/getobjfilter.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct POINTSTYLE;
struct LINESTYLE;
struct MULTISTYLE;
struct POLYSTYLE;
struct TEXTSTYLE;
struct RVCPOINTSTYLE;
struct RVCLINESTYLE;
struct RVCPOLYGONSTYLE;
struct RVCTEXTSTYLE;
namespace RVC {
	class DESCRIPTOR;
	class STYLEPATTERN_BASE;
	class STYLEPATTERN_BITMAP;
	class STYLEPATTERN_HATCH;
	class STYLEPATTERN_LINE;
	class STYLEPATTERN_SYMBOL;
	class STYLEHANDLE;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Style RVC objects.
//!	
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_TIN
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_Table
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_Hyperspectral
//!		- RVC::OBJTYPE_Script_SML
//!		- RVC::OBJTYPE_Script_Query
//!		- RVC::OBJTYPE_Theme
class STYLE : public RVC::OBJECT {
	public:
	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GETOBJFILTER_ISSELECTABLE;
		class GETOBJFILTER_HASPATTERNTYPE;
		class GETOBJFILTER_HASSTYLETYPE;
		class UNIQUEPATTERNNAMEFILTER;
		class UNIQUESTYLENAMEFILTER;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		//! Default constructor
		STYLE (
			);

		//! Copy constructor
		STYLE (
			const RVC::STYLE& obj
			);

		//! Destructor
		virtual ~STYLE (
			);

		//! Assignment.  Will close if necessary and reopen if 'rhs' is open
		RVC::STYLE& operator= (
			const RVC::STYLE& rhs
			);

		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			RVC::PATTERNTYPE PattType,		 			//!< Type of pattern to add
			void *Pattern			 			//!< Pattern to add
			);
										
		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			RVC::PATTERNTYPE PattType,				//!< Type of pattern to add
			const STYLEPATTERN_BASE& Pattern					//!< Pattern to add
			);
			
		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			const STYLEPATTERN_BITMAP& Pattern					//!< Pattern to add
			);
			
		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			const STYLEPATTERN_HATCH& Pattern					//!< Pattern to add
			);
			
		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			const STYLEPATTERN_LINE& Pattern					//!< Pattern to add
			);
			
		//! Add pattern to rvc style object
		//! @return New pattern number
		INT32 AddPattern (
			const STYLEPATTERN_SYMBOL& Pattern					//!< Pattern to add
			);
			
		//! Add point style to rvc style object	
		//! @return New style number or error < 0.
		//! @deprecated Use AddStyle(const RVC::OBJECTNAME&, const POINTSTYLE&)
		DEPRECATED INT32 AddStyle (				
	    	RVCPOINTSTYLE& style	 			//!< Point style to add
			);
										
		//! Add point style to rvc style object	
		//! @return New style number or error < 0.
		INT32 AddStyle (
			const RVC::OBJECTNAME& name,
	    	const POINTSTYLE& style	 			//!< Point style to add
			);
										
		//! Add polygon style to rvc style object	
		//! @return New style number or error < 0.
		//! @deprecated Use AddStyle(const RVC::OBJECTNAME&, const POLYSTYLE&)
		DEPRECATED INT32 AddStyle (				
			RVCPOLYGONSTYLE& style		 		//!< Polygon style to add
			);
										
		//! Add polygon style to rvc style object	
		//! @return New style number or error < 0.
		INT32 AddStyle (
			const RVC::OBJECTNAME& name,
	    	const POLYSTYLE& style	 			//!< Polygon style to add
			);
										
		//! Add line style to rvc style object		
		//! @return New style number or error < 0.
		//! @deprecated Use AddStyle(const RVC::OBJECTNAME&, const LINESTYLE&)
		DEPRECATED INT32 AddStyle (				
			RVCLINESTYLE& style		 		//!< Line style to add
			);
										
		//! Add line style to rvc style object	
		//! @return New style number or error < 0.
		INT32 AddStyle (
			const RVC::OBJECTNAME& name,
	    	const LINESTYLE& style	 			//!< Line style to add
			);
										
		//! Add text style to rvc style object		
		//! @return New style number or error < 0.
		//! @deprecated Use AddStyle(const RVC::OBJECTNAME&, const TEXTSTYLE&)
		DEPRECATED INT32 AddStyle (				
			RVCTEXTSTYLE& style		 		//!< Text style to add
			);

		//! Add text style to rvc style object	
		//! @return New style number or error < 0.
		INT32 AddStyle (
			const RVC::OBJECTNAME& name,
	    	const TEXTSTYLE& style	 			//!< Text style to add
			);

		//! Combine two style objects together using 'this' as the source and either findint or making
		//! the destination style object.  The destination style object is returned in 'DestStyleObj'.
		ERRVALUE Combine (
			const RVC::OBJECT& DestParentObj,				//!< Parent object to combine / copy to
			RVC::STYLE& DestStyleObj,							//!< Destination object created or found
			SIMPLE_ARRAY<INT32>& FontTrans,
			SIMPLE_ARRAY<INT32> PattTrans[PATTERNTYPE_NumTypes]
			);
			
		//! Combine two style objects together using 'this' as the source and either findint or making
		//! the destination style object.  The destination style object is returned in 'DestStyleObj'.
		ERRVALUE Combine (
			const RVC::OBJECT& DestParentObj,				//!< Parent object to combine / copy to
			RVC::STYLE& DestStyleObj							//!< Destination object created or found
			);
			
		//! Convert between memory and rvc line styles			
		//! @deprecated Do not use
		DEPRECATED void ConvertStyle (
			RVCLINESTYLE& rvcstyle,
			const LINESTYLE& linestyle
			) const;
			
		//! Convert between memory and rvc line styles			
		//! @deprecated Use ReadStyle(INT32 StyleNum, const LINESTYLE&)
		DEPRECATED void ConvertStyle (
			LINESTYLE& linestyle,
			const RVCLINESTYLE& rvcstyle
			) const;
			
		//! Convert between memory and rvc point styles			
		//! @deprecated Do not use
		DEPRECATED void ConvertStyle (
			RVCPOINTSTYLE& rvcstyle,
			const POINTSTYLE& pointstyle
			) const;
			
		//! Convert between memory and rvc point styles			
		//! @deprecated Use ReadStyle(INT32 StyleNum, const POINTSTYLE&)
		DEPRECATED void ConvertStyle (
			POINTSTYLE& pointstyle,
			const RVCPOINTSTYLE& rvcstyle
			) const;
			
		//! Convert between memory and rvc polygon styles			
		//! @deprecated Do not use
		DEPRECATED void ConvertStyle (
			RVCPOLYGONSTYLE& rvcstyle,
			const POLYSTYLE& polystyle
			) const;
			
		//! Convert between memory and rvc polygon styles			
		//! @deprecated Use ReadStyle(INT32 StyleNum, const POLYSTYLE&)
		DEPRECATED void ConvertStyle (
			POLYSTYLE& polystyle,
			const RVCPOLYGONSTYLE& rvcstyle
			) const;
			
		//! Convert between memory and rvc text styles			
		//! @deprecated Do not use
		DEPRECATED void ConvertStyle (
			RVCTEXTSTYLE& rvcstyle,
			const TEXTSTYLE& textstyle
			);					//! Not const, can change object
			
		//! Convert between memory and rvc text styles			
		//! @deprecated Use ReadStyle(INT32 StyleNum, const TEXTSTYLE&)
		DEPRECATED void ConvertStyle (
			TEXTSTYLE& textstyle,
			const RVCTEXTSTYLE& rvcstyle
			) const;
			
		//! Delete pattern from rvc style object, modifies styles that referenced the pattern
		//! @return Pattern number that replaced the deleted pattern or <0 if error
		INT32 DeletePattern (
			PATTERNTYPE PattType,					//!< Type of pattern to delete
			INT32 PattNum						//!< Pattern index to delete
			);

		//! Delete style from rvc style object
		//! @return Style number that replaced the deleted style or <0 if error
		INT32 DeleteStyle (
			STYLETYPE StyleType,					//!< Type of style to delete
			INT32 StyleNum						//!< Style index to delete
			);

		//! Delete all styles of a specified type from rvc style object
		ERRVALUE DeleteStyleType (
			STYLETYPE StyleType						//!< Type of style to delete
			);

		//! Locate a font based on a font name and initial position
		//! @return Index of font name if found, -1 if none found, < -1 error code
		INT32 FindFontName (
			INT32 start,						//!< Best guess of initial font location
			const MISTRING& name				//!< Font to search for
			) const;

		//! Add a font name to the font name list
		//! @return Index of added font name or error if < 0
		INT32 FontTransAdd (
			const MISTRING& name				//!< Name of font to add
			);

		//! Obtain font handle from a font index
		//! @return font handle of error if < 0
		INT32 FontTransIndexToHandle (
			INT32 FontNum						//!< Font index to translate into a font handle
			);

		//! Obtain font index from a font name
		//! @return font index of error if < 0
		ERRVALUE FontTransIndexToName (
			INT32 FontNum,				
			MISTRING& name						//!< Name of font to translate into an index
			) const;

		//! Find a pattern given a name and the pattern type
		//! @return -1 if pattern not found, >= 0 Pattern index if found, < -1 error
		INT32 FindPattern (
			PATTERNTYPE PattType,					//!< Type of pattern to search for
			INT32 StartPattNum,				//!< Best guess of initial pattern location
			const RVC::OBJECTNAME& name	//!< Name of pattern to find
			) const;

		//! Find a pattern given a name and the pattern type
		//! @return -1 if pattern not found, >= 0 Pattern index if found, < -1 error
		INT32 FindPattern (
			PATTERNTYPE PattType,					//!< Type of pattern to search for
			const RVC::DESCRIPTOR& Descriptor,	//!< Name of pattern to find
			INT32 StartPattNum = 0				//!< Best guess of initial pattern location
			) const;

		//! Find a style given a name and the style type
		//! @return -1 if style not found, >= 0 Style index if found, < -1 error
		INT32 FindStyle (
			STYLETYPE StyleType,					//!< Type of style to search for
			INT32 StartStyleNum,				//!< Best guess of initial style location
			const RVC::OBJECTNAME& name	//!< Name of style to find
			) const;

		//! Find a style given a name and the style type
		//! @return -1 if style not found, >= 0 Style index if found, < -1 error
		INT32 FindStyle (
			STYLETYPE StyleType,					//!< Type of style to search for
			const RVC::DESCRIPTOR& Descriptor,	//!< Name of style to find
			INT32 StartStyleNum = 0				//!< Best guess of initial style location
			) const;

		//! Obtain the number of font names in the list
		//! @return Number of font names
		INT32 GetFontNameCount (
			) const;

		//! Obtain the number of patterns of a specific pattern type
		//! @return Number of patterns of the requested pattern type or < 0 if error
		INT32 GetPatternCount (
			PATTERNTYPE PattType					//!< Type of pattern to get count of
			) const;

		//! Obtain the maximum pattern size of a specific pattern type
		//! @return Maximum pattern size of the requested pattern type or < 0 if error
		INT32 GetPatternMaxSize (
			PATTERNTYPE PattType					//!< Type of pattern to get the maximum size of
			) const;

		//! Obtain the pattern size of a specific pattern type
		//! @return Pattern size of the requested pattern type or < 0 if error
		INT32 GetPatternSize (
			PATTERNTYPE PattType,					//!< Type of pattern to get the size of
			INT32 PattNum						//!< Pattern index to get the size of
			) const;

		//! Obtain the number of styles of a specific style type
		//! @return Number of styles of the requested style type or < 0 if error
		INT32 GetStyleCount (
			STYLETYPE StyleType						//!< Type of style to get count of
			) const;

		//! Create a new rvc style object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,		//!< Parent object to create style object under
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc style object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,	//!< Name, Desc of object to create, location RETURNED 
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			PATTERNTYPE PattType,					//!< Type of pattern to read
			INT32 PatternNum,					//!< Index of pattern to read
			void *Pattern						//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			PATTERNTYPE PattType,					//!< Type of pattern to read
			INT32 PatternNum,					//!< Index of pattern to read
			STYLEPATTERN_BASE& Pattern				//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			INT32 PatternNum,					//!< Index of pattern to read
			STYLEPATTERN_BITMAP& Pattern				//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			INT32 PatternNum,					//!< Index of pattern to read
			STYLEPATTERN_HATCH& Pattern				//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			INT32 PatternNum,					//!< Index of pattern to read
			STYLEPATTERN_LINE& Pattern				//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern into a buffer.  The format of the pattern is pattern type specific
		ERRVALUE ReadPattern (
			INT32 PatternNum,					//!< Index of pattern to read
			STYLEPATTERN_SYMBOL& Pattern				//!< Pattern buffer to read into
			) const;
		
		//! Read a pattern name
		ERRVALUE ReadPatternDescriptor (
			PATTERNTYPE PattType,					//!< Type of pattern to read
			INT32 PattNum,						//!< Index of pattern to read
			RVC::DESCRIPTOR& Descriptor	//!< Place to store pattern name
			) const;

		//! Read a pattern header into a buffer.
		ERRVALUE ReadPatternHdr (
			PATTERNTYPE PattType,					//!< Type of pattern to read
			INT32 PatternNum,					//!< Index of pattern to read
			void *PattHdr						//!< Pattern header buffer to read into
			) const;

		//! Read a pattern name
		ERRVALUE ReadPatternName (
			PATTERNTYPE PattType,					//!< Type of pattern to read
			INT32 PattNum,						//!< Index of pattern to read
			RVC::OBJECTNAME& name			//!< Place to store pattern name
			) const;

		//! Read a point style item
		//! @deprecated Use ReadStyle(INT32 StyleNum, const POINTSTYLE&)
		DEPRECATED ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of point style to read
			RVCPOINTSTYLE& style				//!< Point style instance to read into
			) const;

		//! Read a point style item
		ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of point style to read
			POINTSTYLE& style				//!< Point style instance to read into
			) const;

		//! Read a polygon style item
		//! @deprecated Use ReadStyle(INT32 StyleNum, const POLYSTYLE&)
		DEPRECATED ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of polygon style to read
			RVCPOLYGONSTYLE& style				//!< Polygon style instance to read into
			) const;

		//! Read a polygon style item
		ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of polygon style to read
			POLYSTYLE& style				//!< Polygon style instance to read into
			) const;

		//! Read a line style item
		//! @deprecated Use ReadStyle(INT32 StyleNum, const LINESTYLE&)
		DEPRECATED ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of line style to read
			RVCLINESTYLE& style				//!< Line style instance to read into
			) const;

		//! Read a line style item
		ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of line style to read
			LINESTYLE& style				//!< Line style instance to read into
			) const;

		//! Read a text style item
		//! @deprecated Use ReadStyle(INT32 StyleNum, const TEXTSTYLE&)
		DEPRECATED ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of text style to read
			RVCTEXTSTYLE& style				//!< Tezt style instance to read into
			) const;

		//! Read a text style item
		ERRVALUE ReadStyle (
			INT32 StyleNum,					//!< Index of text style to read
			TEXTSTYLE& style				//!< Tezt style instance to read into
			) const;

		//! Read a style descriptor
		ERRVALUE ReadStyleDescriptor (
			STYLETYPE StyleType,					//!< Type of pattern to read
			INT32 StyleNum,						//!< Index of pattern to read
			RVC::DESCRIPTOR& Descriptor	//!< Place to store pattern name
			) const;

		//! Read style name given a style type
		ERRVALUE ReadStyleName (
			STYLETYPE StyleType,					//!< Type of style to get the name of
			INT32 StyleNum,					//!< Index of style to get the name of
			RVC::OBJECTNAME& name			//!< Place to store style name
			) const;

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			PATTERNTYPE PattType,					//!< Type of pattern to write
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			void *Pattern						//!< Pattern to write
			);

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			PATTERNTYPE PattType,					//!< Type of pattern to write
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			const STYLEPATTERN_BASE& Pattern						//!< Pattern to write
			);

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			const STYLEPATTERN_BITMAP& Pattern						//!< Pattern to write
			);

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			const STYLEPATTERN_HATCH& Pattern						//!< Pattern to write
			);

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			const STYLEPATTERN_LINE& Pattern						//!< Pattern to write
			);

		//! Write pattern out to rvc style object
		ERRVALUE WritePattern (
			INT32 PatternNum,					//!< Index of pattern to write, Use AddPattern() to write out a new pattern
			const STYLEPATTERN_SYMBOL& Pattern						//!< Pattern to write
			);

		//! Write pattern name given a pattern type.
		ERRVALUE WritePatternDescriptor (
			PATTERNTYPE PattType,					//!< Type of pattern to write name
			INT32 PatternNum,					//!< Index of pattern to write name
			const DESCRIPTOR& Descriptor	//!< New name of pattern
			);
	
		//! Write pattern name given a pattern type.
		ERRVALUE WritePatternName (
			PATTERNTYPE PattType,					//!< Type of pattern to write name
			INT32 PatternNum,					//!< Index of pattern to write name
			const OBJECTNAME& name			//!< New name of pattern
			);
	
		//! Write point style to rvc style object
		//! @deprecated Use WriteStyle(INT32 StyleNum, const RVC::OBJECTNAME& name, const POINTSTYLE&)
		DEPRECATED ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of point style to write.  Use AddStyle() to add a style
			RVCPOINTSTYLE& style				//!< Point style to write
			);

		//! Write point style to rvc style object
		ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of point style to write.  Use AddStyle() to add a style
			const RVC::OBJECTNAME& name,
			const POINTSTYLE& style				//!< Point style to write
			);

		//! Write polygon style to rvc style object
		//! @deprecated Use WriteStyle(INT32 StyleNum, const RVC::OBJECTNAME& name, const POLYSTYLE&)
		DEPRECATED ERRVALUE WriteStyle (				
			INT32 StyleNum,					//!< Index of polygon style to write.  Use AddStyle() to add a style.
			RVCPOLYGONSTYLE& style				//!< Polygon style to write
			);

		//! Write polygon style to rvc style object
		ERRVALUE WriteStyle (				
			INT32 StyleNum,					//!< Index of polygon style to write.  Use AddStyle() to add a style.
			const RVC::OBJECTNAME& name,
			const POLYSTYLE& style				//!< Polygon style to write
			);

		//! Write line style to rvc style object
		//! @deprecated Use WriteStyle(INT32 StyleNum, const RVC::OBJECTNAME& name, const LINESTYLE&)
		DEPRECATED ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of line style to write.  Use AddStyle() to add a style.
			RVCLINESTYLE& style				//!< Line style to write
			);

		//! Write line style to rvc style object
		ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of line style to write.  Use AddStyle() to add a style.
			const RVC::OBJECTNAME& name,
			const LINESTYLE& style				//!< Line style to write
			);

		//! Write text style to rvc style object.
		//! @deprecated Use WriteStyle(INT32 StyleNum, const RVC::OBJECTNAME& name, const TEXTSTYLE&)
		DEPRECATED ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of text style to write.  Use AddStyle() to add a style.
			RVCTEXTSTYLE& style				//!< Text style to write
			);

		//! Write text style to rvc style object.
		ERRVALUE WriteStyle (
			INT32 StyleNum,					//!< Index of text style to write.  Use AddStyle() to add a style.
			const RVC::OBJECTNAME& name,
			const TEXTSTYLE& style				//!< Text style to write
			);

		//! Write style descriptor given a style type.
		ERRVALUE WriteStyleDescriptor (
			STYLETYPE StyleType,					//!< Type of style
			INT32 StyleNum,					//!< Index of style
			const DESCRIPTOR& Descriptor	//!< Style name to write
			);

		//! Write style name given a style type.
		ERRVALUE WriteStyleName (
			STYLETYPE StyleType,					//!< Type of style
			INT32 StyleNum,					//!< Index of style
			const RVC::OBJECTNAME& name	//!< Style name to write
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);

		STYLEHANDLE* m_StyleHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! Style object selection filter to disallow selection of non style objects
class STYLE::GETOBJFILTER_ISSELECTABLE : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE (
			bool IsSaving = false		//!< Set if saving to allow navigation under geometric objects.
			);
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_IsSaving;
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Style object selection filter to disallow selection of non style objects 
//! or style objects that do not have the requested pattern type
class STYLE::GETOBJFILTER_HASPATTERNTYPE : public GETOBJFILTER {
	public:
		GETOBJFILTER_HASPATTERNTYPE (
			PATTERNTYPE PatternType
			) : m_PatternType(PatternType) {}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PATTERNTYPE m_PatternType;
		
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Style object selection filter to disallow selection of non style objects 
//! or style objects that do not have the requested style type
class STYLE::GETOBJFILTER_HASSTYLETYPE : public GETOBJFILTER {
	public:
		GETOBJFILTER_HASSTYLETYPE (
			STYLETYPE StyleType
			) : m_StyleType(StyleType) {}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STYLETYPE m_StyleType;
		
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Style pattern name uniquification filter
class STYLE::UNIQUEPATTERNNAMEFILTER : public OBJECT::UNIQUENAMEFILTER {
	public:
		UNIQUEPATTERNNAMEFILTER (
			const STYLE& StyleObj,
			PATTERNTYPE PatternType
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const STYLE& m_StyleObj;
		PATTERNTYPE m_PatternType;
		
		virtual int Filter (const RVC::OBJECTNAME& name);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
	
//! Style type name uniquification filter
class STYLE::UNIQUESTYLENAMEFILTER : public OBJECT::UNIQUENAMEFILTER {
	public:
		UNIQUESTYLENAMEFILTER (
			const STYLE& StyleObj,
			STYLETYPE StyleType
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const STYLE& m_StyleObj;
		STYLETYPE m_StyleType;
		
		virtual int Filter (const RVC::OBJECTNAME& name);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#ifdef USE_MGUI
namespace UI {

enum STYLECOPYPATTERNFLAGS {
	STYLECOPYPATTERNFLAG_None =					0,
	STYLECOPYPATTERNFLAG_NoCheckDupPattern =	0x01,		//!< Don't check for duplicate pattern, always copy
	STYLECOPYPATTERNFLAG_AskUserDupPattern =	0x02,		//!< Ask user whether to copy duplicate pattern
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(STYLECOPYPATTERNFLAGS)
#endif

//! Copy pattern from one style object to another.
//! @return Pattern number in target object, < 0 if error.
INT32 StyleCopyPattern (
	RVC::STYLE& ObjStyleTgt,			//!< Source style object
	RVC::STYLE& ObjStyleSrc,			//!< Target style object, must be open for write
	RVC::PATTERNTYPE PatternType,		//!< Pattern type
	INT32 SrcPatternNum,					//!< Source patter number
	STYLECOPYPATTERNFLAGS flags = STYLECOPYPATTERNFLAG_None,	//!< Flags
	MDLGPARENT dlgparent = 0			//!< Parent for dialog if required
	);
}	// End namespace UI
#endif


}		// End of RVC namespace

#endif 	// INC_RVC_STYLE_H
