/**
 * \file rvc/uidlgselectrecordset.h
 * \brief RVC::UI::DLG_SELECT_RECORDSET definitions.
 *
 * \if NODOC
 * $Id: uidlgselectrecordset.h_v 1.2 2011/10/21 20:26:13 mju Exp $
 *
 * $Log: uidlgselectrecordset.h_v $
 * Revision 1.2  2011/10/21 20:26:13  mju
 * Remove redundant hdrs.
 *
 * Revision 1.1  2011/06/20 15:21:40  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIDLGSELECTRECORDSET_H
#define	INC_RVC_UIDLGSELECTRECORDSET_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_GRID_H
#include <mgui/grid.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Dialog to allow user to select "set" of records from table.
class DLG_SELECT_RECORDSET : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG_SELECT_RECORDSET (
			const RVC::OBJITEM& ParentObjItem,
			ELEMTYPE ElemType
			);

		//! Destructor.
		virtual ~DLG_SELECT_RECORDSET ();

		//! Get selected record set.
		const BITSET& GetRecordSet (
			) const { return (m_RecordSet); }

		//! Get selected table "short" name.
		const RVC::OBJECTNAME& GetTableNameShort (
			) const { return (m_TableName); }

		//! Set initial selection.
		//! Must do before dialog created.
		void SetSelection (
			const RVC::OBJECTNAME& TableNameShort,		//!< "Short" table name
			const BITSET& RecordSet							//!< Selected records
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_SELECT_RECORDSET THISCLASS;

		enum CMDID {
			CMDID_AddSelected =		1,
			CMDID_AddAll,
			CMDID_RemoveSelected,
			CMDID_RemoveAll,
			};

		MGUI::FORM_COMBOBOX m_CbxField;
		MGUI::CTRL_TOOLBAR m_ToolBar;
		MGUI::CTRL_GRID m_GridRecords;

		RVC::OBJITEM m_ObjItem;
		RVC::DBASE m_DBaseObj;
		RVC::DBTABLE m_TableObj;
		RVC::DBTABLE::RECORD m_Record;
		SIMPLE_ARRAY<INT32> m_RecIndex;
		BITSET m_DuplicateBits;
		RVC::OBJECTNAME m_TableName;
		INT32 m_FieldNum;
		BITSET m_RecordSet;
		ELEMTYPE m_ElemType;

		void OnCbxField ();
		void OnGridClickItem (int itemnum, int column, MGUI::KEYSTATE state);
		bool OnGridGetDispInfo (int row, int col, MGUI::CTRL_GRID::DISPINFO& dispinfo);
		void OnToolBar (MGUI::ID cmdid, bool ToggleOn);

		virtual ERRVALUE v_CreateContent ();

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIDLGSELECTRECORDSET_H
