/** 
 * \file rvc/uieditdescriptor.h 
 * \brief Definitions for RVC::UI::FORM_EDIT_DESCRIPTOR classes.
 *
 * \if NODOC
 * $Id: uieditdescriptor.h_v 1.16 2011/12/27 16:45:04 mju Exp $
 *
 * $Log: uieditdescriptor.h_v $
 * Revision 1.16  2011/12/27 16:45:04  mju
 * Remove redundant hdr.
 *
 * Revision 1.15  2008/09/16 21:04:58  mju
 * Remove flags for legacy controls.
 *
 * Revision 1.14  2008/09/16 20:51:33  mju
 * Remove controls for legacy name/desc and toggle.
 *
 * Revision 1.13  2008/08/13 15:12:38  mju
 * Change default description text field height to 5.
 *
 * Revision 1.11  2006/06/28 15:55:47  mju
 * Move to rvc::ui namespace.
 *
 * Revision 1.10  2006/04/17 16:31:15  mju
 * Add setDelegateTestUniqueShortName.
 *
 * Revision 1.8  2006/02/28 18:51:40  mju
 * Add setFocusToName.
 *
 * Revision 1.7  2005/08/23 23:27:19  scowan
 * Added on any change delegate.
 *
 * Revision 1.6  2005/08/12 22:23:47  scowan
 * Added description height values.
 *
 * Revision 1.4  2005/08/02 22:18:59  scowan
 * Added file name support.
 *
 * Revision 1.1  2005/07/20 22:08:09  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIEDITDESCRIPTOR_H
#define	INC_RVC_UIEDITDESCRIPTOR_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_RVC_DESCRIPTOR_H
#include <rvc/descriptor.h>
#endif

namespace RVC {
namespace UI {

//===================================================================================================================
//! Object name edit form with label and edit field.
class FORM_EDIT_DESCRIPTOR : public MGUI::FORM_COMPOSITE {
	public:
		enum FLAGS {
			FLAG_Default = 				0x0000,
			FLAG_NoModifyName =		 	0x0008,
			FLAG_IsFileDescriptor =	 	0x0010
			};
	
		//! Constructor.
		FORM_EDIT_DESCRIPTOR (
			);

		//! Destructor.
		virtual ~FORM_EDIT_DESCRIPTOR (
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			MGUI::CTRL_EDIT_TEXT::FLAGS textflags = MGUI::CTRL_EDIT_TEXT::FLAG_Default,
			int DescriptionHeight = 5
			);

		//! Get current descriptor value.
		//! Equivalent to GetEditCtrl().GetValue().
		const RVC::DESCRIPTOR& GetValue (
			);

		//! Set delegate to call to get notified on any descriptor changes
		void SetDelegateOnAnyChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnAnyChange = delegate; }

		//! Set delegate to call to get notified on name changes
		void SetDelegateOnNameChange (
			FastDelegate<void(const MISTRING&)> delegate
			) { m_DelegateOnNameChange = delegate; }

		//! Set delegate to call to test if short name unique.
		//! Delegate must return TRUE if name unique, FALSE if not, < 0 if error.
		void SetDelegateOnTestUniqueShortName (
			FastDelegate<int(const RVC::OBJECTNAME&)> delegate
			) { m_DelegateOnTestUniqueShortName = delegate; }

		//! Set delegate to call to validate short name.
		void SetDelegateOnValidateShortName (
			FastDelegate<bool(RVC::OBJECTNAME&)> delegate
			) { m_DelegateOnValidateShortName = delegate; }

		void SetFocusToName (
			) { m_NameForm.SetFocus(); }
			
		void SetParentDirectory (
			const FILEPATH* pParentDirectory
			) { m_pParentDirectory = pParentDirectory; }

		//! No comparison is made with the current string value.
		//! Equivalent to GetEditCtrl().SetValue(...).
		void SetValue (
			const RVC::DESCRIPTOR& Descriptor,			//!< New string to set
			bool notify = true								//!< Call OnChangeValue() if string actually updated
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::FORM_EDIT_STRING m_NameForm;
		MGUI::CTRL_LABEL m_DescLabel;
		MGUI::CTRL_EDIT_TEXT m_DescForm;
		
		RVC::DESCRIPTOR m_Descriptor;
		bool m_LegacyNameUnique;
		const FILEPATH *m_pParentDirectory;
		FastDelegate<void(const MISTRING&)> m_DelegateOnNameChange;
		FastDelegate<int(const RVC::OBJECTNAME&)> m_DelegateOnTestUniqueShortName;
		FastDelegate<bool(RVC::OBJECTNAME&)> m_DelegateOnValidateShortName;
		DELEGATE_VOID_NOPARMS m_DelegateOnAnyChange;
		
		void OnNameUserEditChange ();
		void OnNameChangeValue ();
		void OnDescUserEditChange ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_EDIT_DESCRIPTOR::FLAGS);
#endif // GENERATING_DOXYGEN_OUTPUT


//===================================================================================================================
}	// End of UI namespace
}	// End of RVC namespace

#endif	// INC_RVC_UIEDITDESCRIPTOR_H

