/**
 * \file rvc/uiformobjectsingle.h
 * \brief RVC::UI::FORM_OBJECT_SINGLE class definitions
 *
 * \if NODOC
 * $Id: uiformobjectsingle.h_v 1.6 2012/10/18 12:54:55 mju Exp $
 *
 * $Log: uiformobjectsingle.h_v $
 * Revision 1.6  2012/10/18 12:54:55  mju
 * Change setObjTypeSet to setFilters.
 *
 * Revision 1.5  2012/10/17 13:28:34  mju
 * Add setObjTypeSet.
 *
 * Revision 1.4  2011/09/02 12:57:34  mju
 * Dox hdr.
 *
 * Revision 1.3  2007/07/13 16:29:38  mju
 * Add create() variant with width parm.
 *
 * Revision 1.2  2006/10/16 14:26:18  mju
 * Use rvc getobjflags hdr.
 *
 * Revision 1.1  2006/07/21 20:50:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFORMOBJECTSINGLE_H
#define	INC_RVC_UIFORMOBJECTSINGLE_H

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef	INC_RVC_OTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

namespace RVC {
	class GETOBJFILTER;
	}

namespace RVC {
namespace UI {
	
//===================================================================================================================
//! Form to manage single object selection, containing button on text.
class FORM_OBJECT_SINGLE : public MGUI::FORM_BUTTONTEXT {
	public:
	
		//! Constructor.
		FORM_OBJECT_SINGLE (
			);

		//! Destructor.
		virtual ~FORM_OBJECT_SINGLE (
			);
			
		//! Clears the selection.
		void Clear (
			bool notify							//!< Notify 
			);

		//! Create object list form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			int width,												//!< Width in "em" characters
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER *pGetObjFilter = 0,			//!< Filter for DlgGetObjects
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create object list form with default width.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER *pGetObjFilter = 0,			//!< Filter for DlgGetObjects
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get selected object.
		const RVC::OBJITEM& GetObjItem (
			) const { return (m_ObjItem); }
			
		//! Select object via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify							//!< Send notification to delegate after selection
			);
			
		//! Set delegate to call when change to selected object occurs.
		void SetDelegateOnChangeSelected (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSelected = delegate; }

		//! Set filters with OBJTYPESET and filter instance.
		//! If the currently selected object is not allowed by the new filter/set, the additional parameters affect whether
		//! the user will be prompted for a new selection and/or whether to call the delegate to notify.  If the
		//! user cancels the selection (or is not prompted) the existing selection will be cleared.
		//! If no object is currently selected, the user will not be prompted and the delegate will not be called.
		void SetFilters (
			const RVC::OBJTYPESET *pObjTypeSet,		//!< Set of allowed object types, NULL to leave unchanged
			RVC::GETOBJFILTER *pGetObjFilter,		//!< Filter for DlgGetObjects, NULL for none
			bool SelectIfCurrentNotAllowed,			//!< Reselect if current selection no longer allowed
			bool NotifyIfChangeSelected				//!< Notify if selection changed
			);

		//! Set object.
		void SetObjItem (
			const RVC::OBJITEM& ObjItem,	//!< Object location to set
			bool notify							//!< Send notification to delegate after selection
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::OBJITEM m_ObjItem;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSelected;
		MISTRING m_PromptStr;
		RVC::OBJTYPESET m_ObjTypeSet;
		GETOBJFLAGS m_GetObjFlags;
		RVC::GETOBJFILTER *m_pGetObjFilter;

		void OnBtnSelect ();

		FORM_OBJECT_SINGLE (const FORM_OBJECT_SINGLE&);
		FORM_OBJECT_SINGLE& operator= (const FORM_OBJECT_SINGLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMOBJECTSINGLE_H
