/******************************************************************************
 *
 * \file sml/cadobject.h
 * \brief SML CADOBJECT
 *
 * \if NODOC
 * $Log: cadobject.h_v $
 * Revision 1.3  2007/05/09 19:18:56  dwilliss
 * methods were all private.  Not real useful that way :-)
 *
 * Revision 1.2  2007/05/08 15:45:19  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:07:19  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_CADOBJECT_H
#define INC_SML_CADOBJECT_H

#ifndef INC_SML_OBJECT_H
#include <sml/object.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class CAD;
	class CADHANDLE;
	}
struct CADBLOCK;
#endif

namespace SML {

class RVCSYSLIBCLASSEXPORT CADOBJECT : public OBJECT {
	public:
		CADBLOCK *GetBlock (
			) const;						//!<  current block being accessed

		//! Get a pointer to the RVC::CAD interface class.
		//! Do NOT free this pointer.  It will be destroyed when the SMLOBJECT is
		//! destroyed.  The interface returned is Attached to the object but may
		//! or may not be open.
		//! If the SMLBOJECT does not refer to any existing real object, this
		//! method will return NULL.
		RVC::CAD* GetInterface (
			) const;

		void SetBlock (
			CADBLOCK* block
			);

		//! Set a pointer to the RVC::CAD interface class.
		//! After setting the interface, the SMLOBJECT "owns" it and will 
		//! delete it in its own destructor.  Do not under any circumstances
		//! pass a pointer to an local variable that will go out of scope.
		//! Since SMLOBJECT is implemented in rvcsys.dll it does not have
		//! have access to the copy constructor.
		void SetInterface (
			RVC::CAD* obj
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		// Constructor
		CADOBJECT (
			CONTEXT* context
			);

		// Destructor
		virtual ~CADOBJECT (
			);

		virtual int v_OpenLow (
			int mode
			);

		virtual void v_CloseLow (
			);

		virtual RVC::OBJTYPE v_GetObjType (
			) const;


		CADBLOCK *m_CADBlock;						//!<  current block being accessed -- Move to SMLCADOBJECT
		RVC::CADHANDLE* m_handle;

		friend class SML::CONTEXT;		//!< Only SMLCONTEXT can call our ctor/dtor
		friend class SML::OBJECT;			//!< All our virtual methods are private and called through SMLOBJECT
#endif // GENERATING_DOXYGEN_OUTPUT
	};


} // end of SML namespace
/*****************************************************************************/

typedef SML::CADOBJECT SMLCADOBJECT;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_CADOBJECT_H


