/**
 * \file sml/guigadget.h
 * \brief SML::GUI_GADGET class definitions
 *
 * \if NODOC
 * $Id: guigadget.h_v 1.8 2011/12/07 21:08:36 mju Exp $
 *
 * $Log: guigadget.h_v $
 * Revision 1.8  2011/12/07 21:08:36  mju
 * Fwd decl dlgshell.
 *
 * Revision 1.7  2011/03/29 17:07:54  bcolombe
 * Add SetManaged method
 *
 * Revision 1.6  2008/12/08 13:57:48  mju
 * Put get_tool classes in sml namespace so friendship works on Mac.
 *
 * Revision 1.5  2008/12/05 23:03:12  mju
 * Make sml_gre_tools friends.
 *
 * Revision 1.4  2008/05/13 17:47:46  mju
 * Make ctor public.
 *
 * Revision 1.3  2008/05/09 22:05:40  bcolombe
 * Added m_HasForm for use when creating tool control form
 *
 * Revision 1.2  2008/05/08 21:47:22  bcolombe
 * Added dlgshell for holding tool form
 *
 * Revision 1.1  2008/02/15 18:16:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SML_GUIGADGET_H
#define	INC_SML_GUIGADGET_H

#ifndef	INC_SML_CLASS_H
	#include <sml/class.h>
#endif

#ifndef	INC_MGUI_TOOLBASE_H
	#include <mgui/toolbase.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
namespace MGUI {
	class DLGSHELL;
	class TOOL_RECTANGLE;
	class TOOL_POINT;
	class TOOL_SEGMENT;
	class TOOL_POLYLINE;
	class TOOL_REGION;
	}
namespace SML {
	class CONTEXT;
	class GRE_TOOL_GENERAL;
	class GRE_TOOL_POINT;
	}
#endif

namespace SML {

//=====================================================================================================================

class GUI_GADGET {
	public:

		GUI_GADGET (
			SML::CONTEXT *pContext,
			MGUI::TOOL_BASE *pGadget
			);

		virtual ~GUI_GADGET ();

		MGUI::TOOL_BASE* GetGadgetPtr (
			) const { return (m_pGadget); }

		bool HasPosition (
			) const;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;
		static void SetGPS (int numargs, SML::ARG *args, UINT8* /*type*/, SML::CONTEXT*);
		#endif

		void SetManaged(bool bManaged);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef GUI_GADGET THISCLASS;

		enum CALLBACKTYPE {
			CALLBACKTYPE_RightButtonUp,
			CALLBACKTYPE_Activate,
			CALLBACKTYPE_Deactivate,
			CALLBACKTYPE_PositionSet,
			CALLBACKTYPE_PositionClear,
			CALLBACKTYPE_NUMITEMS
			};

		class CALLBACKLIST;

		SML::CONTEXT *m_pContext;
		MGUI::TOOL_BASE *m_pGadget;
		MGUI::DLGSHELL *m_pDlgShell;
		CALLBACKLIST *m_cblist[CALLBACKTYPE_NUMITEMS];
		bool m_HasForm;

		static void GetCallbackList (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetHasPosition (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetManaged (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetRegion (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetSuspended (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetTrackGPS (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);

		static void SetHasPosition (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetManaged (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetSuspended (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetTrackGPS (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetTransParm (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);

		static void Setup();

		bool DoCallbackList (CALLBACKTYPE cbtype);
		void OnGadgetChange (MGUI::TOOL_BASE::CHANGESEQ ChgSeq, MGUI::TOOL_BASE::CHANGETYPE ChgType);
		void OnGadgetDestroy ();
		bool OnRightButtonDown (MGUI::POINT point, MGUI::KEYSTATE keystate);
		bool OnRightButtonUp (MGUI::POINT point, MGUI::KEYSTATE keystate);

		friend class SML::GRE_TOOL_GENERAL;
		#endif
	};

//=====================================================================================================================

struct GUI_GADGET_POINT : public GUI_GADGET {
	public:

		GUI_GADGET_POINT (
			SML::CONTEXT *pContext,
			MGUI::TOOLCANVAS& Canvas
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;

	private:

		MGUI::TOOL_POINT& GetGadget ();

		static void GetPoint (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetPoint (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);

		static void Setup();

		friend class SML::GRE_TOOL_POINT;
		#endif
	};

//=====================================================================================================================

struct GUI_GADGET_POLYLINE : public GUI_GADGET {
	public:

		GUI_GADGET_POLYLINE (
			SML::CONTEXT *pContext,
			MGUI::TOOLCANVAS& Canvas,
			bool closed
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;

	private:

		MGUI::TOOL_POLYLINE& GetGadget ();

		static void* GetPolyLine(int n, SML::ARG*, UINT8*, SML::CONTEXT*);
		void SetPolyLine(POLYLINE* pl);
		static void Setup();

		#endif
	};

//=====================================================================================================================

struct GUI_GADGET_RECTANGLE : public GUI_GADGET {
	public:

		GUI_GADGET_RECTANGLE (
			SML::CONTEXT *pContext,
			MGUI::TOOLCANVAS& Canvas
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;

	private:

		MGUI::TOOL_RECTANGLE& GetGadget ();

		static void Setup();

		#endif
	};

//=====================================================================================================================

struct GUI_GADGET_REGION : public GUI_GADGET {
	public:

		GUI_GADGET_REGION (
			SML::CONTEXT *pContext,
			MGUI::TOOLCANVAS& Canvas
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;

	private:

		MGUI::TOOL_REGION& GetGadget ();

		static void Setup();

		#endif
	};

//=====================================================================================================================

struct GUI_GADGET_SEGMENT : public GUI_GADGET {
	public:

		GUI_GADGET_SEGMENT (
			SML::CONTEXT *pContext,
			MGUI::TOOLCANVAS& Canvas
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_DELETABLE_NOCOPY_T<GUI_GADGET> s_AutoClass;

	private:

		MGUI::TOOL_SEGMENT& GetGadget ();

		static void GetEnd (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void GetStart (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetEnd (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);
		static void SetStart (SML::CONTEXT *context, SML::CLASSMEMBER *member, void *vdata, SML::ARG *ret);

		static void Setup();

		#endif
	};

//=====================================================================================================================

void InstallGadgetItems (SML::CONTEXT *context);

//=====================================================================================================================

}	// End namespace SML

#endif	// INC_SML_GUIGADGET_H
