/******************************************************************************
 *
 * \file sml/hash.h
 * \brief SML HASH
 *
 * \if NODOC
 * $Log: hash.h_v $
 * Revision 1.3  2011/10/12 14:34:57  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2007/05/08 15:46:02  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 22:12:02  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_HASH_H
#define INC_SML_HASH_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

class MISTRING;
class MISTRINGLIST;

namespace SML {

class CLASS;
class CONTEXT;

class RVCSYSLIBCLASSEXPORT HASH {
	public:
		HASH (
			);
		virtual ~HASH();

		const CLASS* GetClass() const;	// will return 0 if numeric

		int GetNumItems (
			) const;

		static void NumIndexToStrIndex (
			MISTRING& mistr,
			double indx
			);

		bool Exists (
			const MISTRING& key
			) const;

		void Delete (
			const MISTRING& key
			);

		void Clear ();

		MISTRINGLIST* GetKeyList (
			) const;

	private:
		virtual const CLASS* v_GetClass() const = 0;	// will return 0 if numeric
		virtual int v_GetNumItems() const = 0;
		virtual bool v_Exists(const MISTRING&) const = 0;
		virtual void v_Delete(const MISTRING&) = 0;
		virtual void v_Clear() = 0;
		virtual MISTRINGLIST* v_GetKeyList() const = 0;
	};

class RVCSYSLIBCLASSEXPORT NUMHASH : public HASH {
	public:
		NUMHASH();
		virtual ~NUMHASH();
		double GetValue(const MISTRING& indx, bool bConstructIfEmpty) const;
		void SetValue(const MISTRING& indx, double val);

		//! Get the hash as an array.  This will ONLY contain the values which
		//! were populated using integer indexes.  Anything filled in using a
		//! non-integer or non-numeric key will not be in this list.
		//! This allows a numeric hash to be passed to functions that want a
		//! one-dimensional array.
		double* GetArray (
			);

		//! Get the hash as an array.  This will ONLY contain the values which
		//! were populated using integer indexes.  Anything filled in using a
		//! non-integer or non-numeric key will not be in this list.
		//! This allows a numeric hash to be passed to functions that want a
		//! one-dimensional array.
		INT32 GetArraySize (
			) const;

		//! Resize the array.
		//! Note: this will clear any non-integer index entries from the hash.
		ERRVALUE ResizeArray (
			INT32 numitems, 		//!< # of desired entries (starting at 0)
			double fillval,		//!< Value to initialize array to
			bool bKeepExisting = true	//!< true to keep existing data
			);

	private:
		class PRIVDATA;
		PRIVDATA* m_priv;
		virtual const CLASS* v_GetClass() const;	// will return 0 if numeric
		virtual int v_GetNumItems() const;
		virtual bool v_Exists(const MISTRING&) const;
		virtual void v_Delete(const MISTRING&);
		virtual void v_Clear();
		virtual MISTRINGLIST* v_GetKeyList() const;
	};


class RVCSYSLIBCLASSEXPORT CLASSHASH : public HASH {
	public:
		CLASSHASH(CONTEXT* context, const CLASS*);
		virtual ~CLASSHASH();
		void* GetValue(const MISTRING& indx, bool bConstructIfEmpty) const;
		void SetValue(const MISTRING& indx, void* val);
	protected:
		CONTEXT* GetContext() const;
	private:
		class PRIVDATA;
		PRIVDATA* m_priv;
		virtual const CLASS* v_GetClass() const;	// will return 0 if numeric
		virtual int v_GetNumItems() const;
		virtual bool v_Exists(const MISTRING&) const;
		virtual void v_Delete(const MISTRING&);
		virtual void v_Clear();
		virtual MISTRINGLIST* v_GetKeyList() const;
	};


class RVCSYSLIBCLASSEXPORT STRINGHASH : public CLASSHASH {
	public:
		STRINGHASH(CONTEXT* context);
		virtual ~STRINGHASH();
		bool GetValue(const MISTRING& indx, MISTRING& value) const;
		void SetValue(const MISTRING& indx, const MISTRING& value);
	};

} // end of SML namespace

typedef SML::HASH SMLHASH;
typedef SML::NUMHASH SMLNUMHASH;
typedef SML::CLASSHASH SMLCLASSHASH;
typedef SML::STRINGHASH SMLSTRINGHASH;


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT


#endif		//  INC_SML_HASH_H


