/******************************************************************************
 *
 * \file sml/shapeobject.h
 * \brief SML SHAPEOBJECT
 *
 * \if NODOC
 * $Log: shapeobject.h_v $
 * Revision 1.2  2007/05/08 15:46:19  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:07:21  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_SHAPEOBJECT_H
#define INC_SML_SHAPEOBJECT_H

#ifndef INC_SML_OBJECT_H
#include <sml/object.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class SHAPE;
	class SHAPEHANDLE;
	}
#endif

namespace SML {

class RVCSYSLIBCLASSEXPORT SHAPEOBJECT : public OBJECT {
	public:

		//! Get a pointer to the RVC::SHAPE interface class.
		//! Do NOT free this pointer.  It will be destroyed when the SML::OBJECT is
		//! destroyed.  The interface returned is Attached to the object but may
		//! or may not be open.
		//! If the SML::BOJECT does not refer to any existing real object, this
		//! method will return NULL.
		RVC::SHAPE* GetInterface (
			) const;

		//! Set a pointer to the RVC::SHAPE interface class.
		//! After setting the interface, the SML::OBJECT "owns" it and will 
		//! delete it in its own destructor.  Do not under any circumstances
		//! pass a pointer to an local variable that will go out of scope.
		//! Since SML::OBJECT is implemented in rvcsys.dll it does not have
		//! have access to the copy constructor.
		void SetInterface (
			RVC::SHAPE* obj
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		// Constructor
		SHAPEOBJECT (
			CONTEXT* context
			);

		// Destructor
		virtual ~SHAPEOBJECT (
			);

		virtual int v_OpenLow (
			int mode
			);

		virtual void v_CloseLow (
			);

		virtual RVC::OBJTYPE v_GetObjType (
			) const;

		friend class CONTEXT;		//!< Only SML::CONTEXT can call our ctor/dtor
		friend class OBJECT;			//!< All our virtual methods are private and called through SML:OBJECT

		RVC::SHAPEHANDLE* m_handle;
#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // end of SML namespace


typedef SML::SHAPEOBJECT SMLSHAPEOBJECT;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_SHAPEOBJECT_H


