/**
 * \file spatmod/imagefiltercolorbalance.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_COLORBALANCE definition.
 *
 * \if NODOC
 * $Id: imagefiltercolorbalance.h_v 1.1 2006/11/28 21:43:06 mju Exp $
 *
 * $Log: imagefiltercolorbalance.h_v $
 * Revision 1.1  2006/11/28 21:43:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERCOLORBALANCE_H
#define	INC_SPATMOD_IMAGEFILTERCOLORBALANCE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

struct CBPARMS;
class COLORBALANCE;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform color balancing.
//! Source image must have Red, Green and Blue samples of type UINT8.
class FILTER_COLORBALANCE : public FILTER {
	public:

		//! Constructor.
		FILTER_COLORBALANCE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const CBPARMS& cbparms					//!< Color balance parameters
			);

		//! Destructor.
		virtual ~FILTER_COLORBALANCE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		COLORBALANCE *m_pColorBalance;
		UINT16 m_SampleR;
		UINT16 m_SampleG;
		UINT16 m_SampleB;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_COLORBALANCE (const FILTER_COLORBALANCE&);
		FILTER_COLORBALANCE& operator= (const FILTER_COLORBALANCE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERCOLORBALANCE_H

