/**
 * \file imagefiltercomposite.h <spatmod/imagefiltercomposite.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_COMPOSITE definition.
 *
 * \if NODOC
 * $Id: imagefiltercomposite.h_v 1.3 2006/11/03 17:37:00 mju Exp $
 *
 * $Log: imagefiltercomposite.h_v $
 * Revision 1.3  2006/11/03 17:37:00  mju
 * Add option parm to override pixel type in ctor.
 *
 * Revision 1.2  2006/01/27 22:11:28  mju
 * Add v_computeSampleStats.
 *
 * Revision 1.1  2005/08/16 15:09:17  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERCOMPOSITE_H
#define	INC_SPATMOD_IMAGEFILTERCOMPOSITE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to create composite image from multiple source images.
//! If any source stage has a pixel type of Indexed it will be converted to non-indexed (using FILTER_DEINDEX).
class FILTER_COMPOSITE : public FILTER {
	public:

		//! Construct using array of previous stage pointers and common ownership.
		//! All previous stages must have same total dimensions.
		//! If pixel type is specified it must be appropriate for final number of samples or
		//! error will be returned from Initialize.
		FILTER_COMPOSITE (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			PIXEL::TYPE PixelType = PIXEL::TYPE_Invalid	//!< Pixel type override, TYPE_Invalid to determine automatically
			);

		// Destructor.
		virtual ~FILTER_COMPOSITE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL::TYPE m_PixelType;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_COMPOSITE (const FILTER_COMPOSITE&);
		FILTER_COMPOSITE& operator= (const FILTER_COMPOSITE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERCOMPOSITE_H

