/**
 * \file imagehelperrvc.h <spatmod/imagehelperrvc.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::HELPER_RVC definitions
 *
 * \if NODOC
 * $Id: imagehelperrvc.h_v 1.3 2006/12/13 21:08:21 mju Exp $
 *
 * $Log: imagehelperrvc.h_v $
 * Revision 1.3  2006/12/13 21:08:21  mju
 * Add getCellTypeFromPixelProp.
 *
 * Revision 1.2  2006/12/08 18:53:08  mju
 * Add getCellTypeFromSampleProp.
 *
 * Revision 1.1  2006/06/26 17:58:14  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEHELPERRVC_H
#define	INC_SPATMOD_IMAGEHELPERRVC_H

#ifndef	INC_SPATMOD_DATATYPE_H
#include <spatmod/datatype.h>
#endif

#ifndef	INC_SPATMOD_IMAGEPIXEL_H
#include <spatmod/imagepixel.h>
#endif

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Helper for interface with RVC IMAGE classes.
class HELPER_RVC {
	public:

		//! Get RVC::IMAGE::CELLTYPE from pixel properties.
		//! Will return CELLTYPE_Invalid if cannot translate.
		static RVC::IMAGE::CELLTYPE GetCellTypeFromPixelProp (
			const SPATMOD::IMAGE::PIXEL& PixelProp
			);

		//! Get RVC::IMAGE::CELLTYPE from single sample properties.
		//! Will return CELLTYPE_Invalid if cannot translate.
		static RVC::IMAGE::CELLTYPE GetCellTypeFromSampleProp (
			const SPATMOD::IMAGE::SAMPLE& SampleProp
			);

		//! Get SPATMOD::DATATYPE from RVC::IMAGE::CELLTYPE.
		//! If the cell type has multiple data types, DATATYPE_Invalid will be returned.
		//! If the cell type uses < 8 bits per sample, DATATYPE_UINT8 will be returned.
		static DATATYPE GetDataTypeFromCellType (
			RVC::IMAGE::CELLTYPE CellType
			);

		//! Get SPATMOD::IMAGE::PIXEL::TYPE from RVC::IMAGE::CELLTYPE.
		static PIXEL::TYPE GetPixelTypeFromCellType (
			RVC::IMAGE::CELLTYPE CellType
			);
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEHELPERRVC_H

