/**
 * \file imagetargetmemory.h <spatmod/imagetargetmemory.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_MEMORY definition.
 *
 * \if NODOC
 * $Id: imagetargetmemory.h_v 1.2 2006/11/01 23:10:30 mju Exp $
 *
 * $Log: imagetargetmemory.h_v $
 * Revision 1.2  2006/11/01 23:10:30  mju
 * queryIterator.
 *
 * Revision 1.1  2006/07/27 21:49:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETMEMORY_H
#define	INC_SPATMOD_IMAGETARGETMEMORY_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Memory buffer image target.
class TARGET_MEMORY : public TARGET {
	public:

		//! Construct target with specified input stage
		explicit TARGET_MEMORY (
			STAGE *pInputStage							//!< Stage in pipeline to obtain image data from
			);

		//! Destructor.
		virtual ~TARGET_MEMORY ();

		//! Attach existing shared buffer to image.
		//! Must be done before initialize.
		//! Buffer must have same dimensions and pixel properties as this image.
		ERRVALUE AttachBuffer (
			IMAGE::BUFFER& buffer						//!< Buffer to attach
			);

		//! Get reference to buffer.
		IMAGE::BUFFER& GetBuffer (
			) { return (m_Buffer); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		IMAGE::BUFFER m_Buffer;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// Unimplemented
		TARGET_MEMORY (const TARGET_MEMORY&);
		TARGET_MEMORY& operator= (const TARGET_MEMORY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETMEMORY_H

